/*
 *    Brutus header file defining a Windows(R) INI file helper class.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef _INIFILE_H_
#define _INIFILE_H_

#include <string.h> 

class IniFile {
public:
	IniFile(const char *name = NULL);

	~IniFile();

	bool add_head(const char *head);

        // Will add a new section or merge this one if an identically
        // name already exists. All section names are upper case.
	bool add_section(const char *section_name, const char *data);

        // Does a section with that data exist?
	bool duplicate(const char *section_name, const char *data);

	bool add(const IniFile& ini_file);

	int commit(const char *file_name);

private:
        // All text is null terminated dosmode strings, a.k.a.: \r\n for newlines.
        // The strings are always terminated with a \r\n\0 tripple.
	struct section {
		char *name; 
		char *data;
	};

        char *head_; // all leading text before any section has been started
	struct section *content_; // all sections and their lines following head_

	size_t section_count_;
};

#endif // _INIFILE_H_

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
