/*
 *    Brutus source file implementing the main() function.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "gpl.h"
#include "inifile.h"

#if defined(_WIN32)
#define basename(__str1__) __str1__ 
#endif
#define VERSION "0.2.1"

static void print_usage(const char *name)
{
        fprintf(stdout,  
                "Usage: %s [No more than one option] <FILE ONE> <FILE TWO>\n\n"
                "  -help \tPrints this message.\n"
                "  -license \tPrint software license.\n"
                "  -merge \tMerge \"FILE TWO\" into \"FILE ONE\". Both files are required \n"
		"         \tto be in the correct Microsoft(R) INI file format.\n"
                "  -add-MSEMS \tAdd the MSEMS (Microsoft(R) Exchange Server) service into\n"
		"             \t\"FILE ONE\". \"FILE ONE\" must be in the correct Microsoft(R)\n"
		"             \tINI file format.\n\n"
                "  \t** The resulting output file is always \"RESULT.inf\" **\n\n"
                "\n", name);
}


static void print_license(void)
{
        fprintf(stdout, "%s\n", STR_GPL_V2);
}

static bool add_msems(const char *file_name,
		      const char *output = NULL)
{
	IniFile file(file_name);

	bool retv = true;
	retv = retv && file.add_section("[Services]\r\n", 
					"MSEMS=Microsoft Exchange Server\r\n");

	retv = retv && file.add_section("[Default Services]\r\n",
					"MSEMS=Microsoft Exchange Server\r\n");

	retv = retv && file.add_section("[MSEMS]\r\n",
					"PR_DISPLAY_NAME=Microsoft Exchange Server\r\n"
					"Sections=MSEMS_MSMail_Section\r\n"
					"PR_SERVICE_DLL_NAME=emsui.dll\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_SERVICE_ENTRY_NAME=EMSCfg\r\n"
					"PR_RESOURCE_FLAGS=SERVICE_SINGLE_COPY\r\n"
					"WIZARD_ENTRY_NAME=EMSWizardEntry\r\n"
					"Providers=EMS_DSA, EMS_MDB_public, EMS_MDB_private, EMS_RXP, EMS_MSX, EMS_Hook\r\n"
					"PR_SERVICE_SUPPORT_FILES=emsui.dll,emsabp.dll,emsmdb.dll\r\n");

	retv = retv && file.add_section("[EMS_MDB_public]\r\n",
					"PR_RESOURCE_TYPE=MAPI_STORE_PROVIDER\r\n"
					"PR_PROVIDER_DLL_NAME=EMSMDB.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_FLAGS=STATUS_NO_DEFAULT_STORE\r\n"
					"66090003=06000000\r\n"
					"660A0003=03000000\r\n"
					"34140102=78b2fa70aff711cd9bc800aa002fc45a\r\n"
					"PR_DISPLAY_NAME=Public Folders\r\n"
					"PR_PROVIDER_DISPLAY=Microsoft Exchange Message Store\r\n");

	retv = retv && file.add_section("[EMS_MDB_private]\r\n",
					"PR_PROVIDER_DLL_NAME=EMSMDB.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_TYPE=MAPI_STORE_PROVIDER\r\n"
					"PR_RESOURCE_FLAGS=STATUS_PRIMARY_IDENTITY|STATUS_DEFAULT_STORE|STATUS_PRIMARY_STORE\r\n"
					"66090003=0C000000\r\n"
					"660A0003=01000000\r\n"
					"34140102=5494A1C0297F101BA58708002B2A2517\r\n"
					"PR_DISPLAY_NAME=Private Folders\r\n"
					"PR_PROVIDER_DISPLAY=Microsoft Exchange Message Store\r\n");

	retv = retv && file.add_section("[EMS_DSA]\r\n",
					"PR_DISPLAY_NAME=Microsoft Exchange Directory Service\r\n"
					"PR_PROVIDER_DISPLAY=Microsoft Exchange Directory Service\r\n"
					"PR_PROVIDER_DLL_NAME=EMSABP.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_TYPE=MAPI_AB_PROVIDER\r\n");

	retv = retv && file.add_section("[MSEMS_MSMail_Section]\r\n",
					"UID=13DBB0C8AA05101A9BB000AA002FC45A\r\n"
					"66000003=01050000\r\n"
					"66010003=04000000\r\n"
					"66050003=03000000\r\n"
					"66040003=02000000\r\n");

	retv = retv && file.add_section("[EMS_RXP]\r\n",
					"PR_DISPLAY_NAME=Microsoft Exchange Remote Transport\r\n"
					"PR_PROVIDER_DISPLAY=Microsoft Exchange Remote Transport\r\n"
					"PR_PROVIDER_DLL_NAME=EMSUI.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_TYPE=MAPI_TRANSPORT_PROVIDER\r\n"
					"66090003=40000000\r\n"
					"660A0003=0A000000\r\n");

	retv = retv && file.add_section("[EMS_MSX]\r\n",
					"PR_DISPLAY_NAME=Microsoft Exchange Transport\r\n"
					"PR_PROVIDER_DISPLAY=Microsoft Exchange Transport\r\n"
					"PR_PROVIDER_DLL_NAME=EMSMDB.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_TYPE=MAPI_TRANSPORT_PROVIDER\r\n"
					"66090003=00000000\r\n");
 
	retv = retv && file.add_section("[EMS_Hook]\r\n",
					"PR_DISPLAY_NAME=MS Exchange Hook\r\n"
					"PR_PROVIDER_DISPLAY=MS Exchange Hook\r\n"
					"PR_PROVIDER_DLL_NAME=EMSMDB.DLL\r\n"
					"PR_SERVICE_INSTALL_ID={6485D26A-C2AC-11D1-AD3E-10A0C911C9C0}\r\n"
					"PR_RESOURCE_TYPE=MAPI_HOOK_PROVIDER\r\n"
					"PR_RESOURCE_FLAGS=HOOK_INBOUND\r\n");


	if (output)
		retv = retv && file.commit(output);
	else
		retv = retv && file.commit("RESULT.inf");

	return retv;
}


int main(int argc, char *argv[])
{
        /* mandatory legal blurb */
	const char *name = basename(argv[0]);
        fprintf(stdout,
                "    %s version %s, Copyright (C) 2005 OMC Denmark ApS.\n"
		"\n"
                "    %s comes with ABSOLUTELY NO WARRANTY. This is free software;\n"
		"    you can redistribute it and/or modify it under the term of the\n"
		"    GNU General Public License as published by the Free Software Foundation;\n"
		"    either version 2 of the License, or (at your option) any later version.\n"
		"\n"
                "    Please invoke %s as \"%s -license\" for the legal details.\n\n\n", name, VERSION, name, name, name);

	if (argc < 2) {
		print_usage(name);
		return EXIT_SUCCESS;
	}

	if (!strcmp("-help", argv[1])) {
		print_usage(name);
		return EXIT_SUCCESS;
	}

	if (!strcmp("-license", argv[1])) {
		print_license();
		return EXIT_SUCCESS;
	}

	if (!strcmp("-merge", argv[1])) {
		if (argc != 4)
			return EXIT_FAILURE;

		IniFile file_one(argv[2]);
		IniFile file_two(argv[3]);
		if (file_one.add(file_two)) {
			file_one.commit("RESULT.inf");
			return EXIT_SUCCESS;
		} else
			return EXIT_FAILURE;
	}

	if (!strcmp("-add-MSEMS", argv[1])) {
		if (add_msems(argv[2])) 
			return EXIT_SUCCESS;
		else
			return EXIT_FAILURE;
	}

	if (!strcmp("-add-MSEMS-no-backup", argv[1])) {
		if (add_msems(argv[2], argv[2])) 
			return EXIT_SUCCESS;
		else
			return EXIT_FAILURE;
	}

        fprintf(stdout, 
		"    ERROR: No recognized options given. Remember that all options\n"
		"           are case-sensitive.\n\n");

	return EXIT_FAILURE;
}
