/*
 *    Brutus header file for the ProxyManager servant.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __PROXY_MANAGERS_IMPL_H_
#define __PROXY_MANAGERS_IMPL_H_

#include <NT_Service/ProxyMgr/proxy_managerS.h>
#include <NT_Service/defines.h>
#include <NT_Service/macros.h>

#include <vector>
#include <string>
#include <map>

#include <windows.h>


#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

// Will check for and if possible enable required 
// privileges.
// Returns false on error, true otherwise.
static bool SetPrivilege(LPCTSTR lpszPrivilege,  // name of privilege to enable/disable
                         bool bEnablePrivilege)  // to enable or disable privilege
{
        TOKEN_PRIVILEGES tp;
        LUID luid;

        if (!LookupPrivilegeValue(NULL,            // lookup privilege on local system
                                  lpszPrivilege,   // privilege to lookup 
                                  &luid)) {        // receives LUID of privilege
                BRUTUS_LOG_GETLASTERROR("LookupPrivilegeValue error");
                return false; 
        }

        tp.PrivilegeCount = 1;
        tp.Privileges[0].Luid = luid;
        if (bEnablePrivilege)
                tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
        else
                tp.Privileges[0].Attributes = 0;

        // Enable the privilege or disable all privileges.
        HANDLE this_process = NULL;
        if (!OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &this_process)) {
                BRUTUS_LOG_GETLASTERROR("OpenProcessToken error");
                return false; 
        }
        if (!AdjustTokenPrivileges(this_process, 
                                   FALSE, 
                                   &tp, 
                                   sizeof(TOKEN_PRIVILEGES), 
                                   (PTOKEN_PRIVILEGES)NULL, 
                                   (PDWORD)NULL)) { 
                CloseHandle(this_process);
                BRUTUS_LOG_GETLASTERROR("AdjustTokenPrivileges failed");
                return false; 
        } 
        CloseHandle(this_process);

        if (GetLastError() == ERROR_NOT_ALL_ASSIGNED) {
		ACE_LOG_MSG->open("Brutus Server", ACE_Log_Msg::SYSLOG);
		ACE_DEBUG((LM_CRITICAL, ACE_TEXT("The token does not have the specified privilege")));
                BRUTUS_LOG_PRIVATE("The token does not have the specified privilege");
                return false;
        } 

        return true;
}

class  BRUTUS_ProxyManager_i : public virtual POA_BRUTUS::ProxyManager, 
			       public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_ProxyManager_i(void)
		{ 
                        ACE_LOG_MSG->open("Brutus Server", ACE_Log_Msg::SYSLOG);
                        proxy_list_.resize(0);
                        
                        bool privs_ok = true;
                        if (!SetPrivilege(SE_TCB_NAME, true)) {
                                privs_ok = false;
                                ACE_DEBUG((LM_CRITICAL, ACE_TEXT("Enable SE_TCB_NAME (Act as part of the operating system) failed")));
                                BRUTUS_LOG_CRITICAL("Enable SE_TCB_NAME (Act as part of the operating system) failed");
                        }
                        if (!SetPrivilege(SE_INCREASE_QUOTA_NAME, true)) {
                                privs_ok = false;
                                ACE_DEBUG((LM_CRITICAL, ACE_TEXT("Enable SE_INCREASE_QUOTA_NAME (Increase quotas) failed")));
                                BRUTUS_LOG_CRITICAL("Enable SE_INCREASE_QUOTA_NAME (Increase quotas) failed");
                        }
                        if (!SetPrivilege(SE_ASSIGNPRIMARYTOKEN_NAME, true)) {
                                privs_ok = false;
                                ACE_DEBUG((LM_CRITICAL, ACE_TEXT("Enable SE_ASSIGNPRIMARYTOKEN_NAME (Replace a process level token) failed")));
                                BRUTUS_LOG_CRITICAL("Enable SE_ASSIGNPRIMARYTOKEN_NAME (Replace a process level token) failed");
                        }

                        if (!privs_ok) {
                                ACE_DEBUG((LM_CRITICAL, ACE_TEXT("One or more required process privileges are missing, see above. These must be assigned for your Brutus Server to work.")));
                                BRUTUS_LOG_CRITICAL("One or more required process privileges are missing, see above.\n             These must be assigned for your Brutus Server to work.");
                        }
		};
  
	virtual	CORBA::Boolean setUser(const char *LifeLineIOR,
                                       const char *UserName,
                                       const char *Domain,
                                       const char *Password,
                                       const char *MAPIProfileName,
                                       const char *MAPIProfilePassword,
                                       ::CORBA::ULong MaxNotificationDelay,
                                       ::CORBA::ULong Flags
                                       ACE_ENV_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));
  
	virtual void getUser(CORBA::String_out LifeLineIOR,
                             CORBA::String_out MAPIProfileName,
			     CORBA::String_out UserName,
			     CORBA::String_out Domain,
			     CORBA::String_out Password,
			     CORBA::String_out MAPIProfilePassword,
			     ::CORBA::ULong_out MaxNotificationDelay,
			     ::CORBA::ULong_out Flags
			     ACE_ENV_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));
  
	virtual	void setSession(const char *MAPIProfileName,
				const char *SessionIOR,
                                ::CORBA::Boolean Sharable
				ACE_ENV_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));
  
	virtual	void setSpawnStatus(const char *MAPIProfileName,
				    ::BRUTUS::ProxyManager::SpawnStatus Status
				    ACE_ENV_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));

	virtual	::BRUTUS::ProxyManager::SpawnStatus getSession(const char *MAPIProfileName,
							       CORBA::String_out SessionIOR
							       ACE_ENV_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));
  
	virtual CORBA::ULong getProxyCount(ACE_ENV_SINGLE_ARG_DECL)
		ACE_THROW_SPEC((CORBA::SystemException));

	void removeProxy(pid_t pid);

private:

	class User {
	public:
		User(const char *LifeLineIOR,
		     const char *Name,
		     const char *Domain,
		     const char *Password,
		     const char *MAPI_Pwd,
                     const unsigned long MaxNotificationDelay,
		     const ::BRUTUS::BDEFINE ulFlags)
			: status(::BRUTUS::ProxyManager::spawn_unknown),
                          life_line_ior(strdup(LifeLineIOR)),
                          name(strdup(Name)),
                          domain(strdup(Domain)),
                          password(strdup(Password)),
                          mapi_pwd(strdup(MAPI_Pwd)),
                          max_notification_delay(MaxNotificationDelay),
                          flags(ulFlags),
                          ior(NULL),
                          taken(false),
                          sharable((::CORBA::Boolean)1),
                          primary_token(NULL)
			{ 
			};

		~User(void)
			{
                                CloseHandle(primary_token);
				free(life_line_ior);
				free(name);
				free(domain);
				free(password);
				free(mapi_pwd);
                                free(ior);
			};

                bool create_token(void)
                        {
                                if (primary_token)
                                        return true;

                                if (!LogonUser(name, 
                                               domain, 
                                               password, 
                                               LOGON32_LOGON_INTERACTIVE, 
                                               LOGON32_PROVIDER_DEFAULT, 
                                               &primary_token)) {
                                        char msg[128] = {0};
                                        sprintf(msg, "LogonUser(%s/%s) failed", name, domain); 
                                        BRUTUS_LOG_GETLASTERROR(msg);

                                        primary_token = NULL;
                                        return false;
                                }

                                return true;
                        };

		::BRUTUS::ProxyManager::SpawnStatus status;
		char *life_line_ior;
		char *name;
		char *domain;
		char *password;
		char *mapi_pwd;
                unsigned long max_notification_delay;
                ::BRUTUS::BDEFINE flags;
		char *ior;
		bool taken; // is this being served by a proxy process?
                ::CORBA::Boolean sharable;
                HANDLE primary_token;
	};
        typedef std::map<std::string, User*>::iterator SLit;

	class BRUTUS_Process {
        public:
                BRUTUS_Process(void)
                        : spawned_(false),
                          cmd_line_(NULL)
                        { 
                        };

                ~BRUTUS_Process(void) 
                        {
                                free(cmd_line_);
                                try {
                                        CloseHandle(process_info_.hProcess);
				}
				catch (...) {
				}
                        };

                HANDLE gethandle(void)
                        {
                                if (!spawned_)
                                        return NULL;

                                return process_info_.hProcess;
                        }

                void kill(void)
                        {
                                TerminateProcess(process_info_.hProcess, 0);
                        }

                pid_t getpid(void)
                        {
                                if (!spawned_)
                                        return -1;

                                return process_info_.dwProcessId;
                        };

                bool running (void)
                        {
                                if (!spawned_)
                                        return false;

                                DWORD code;
                                BOOL result = ::GetExitCodeProcess(process_info_.hProcess,
                                                                   &code);
                                return (result && (STILL_ACTIVE == code));
                        };
                
                bool prepare(const char *CommandLine) {
                        cmd_line_ = strdup(CommandLine);
                        return true;
                };

                pid_t spawn(HANDLE PrimaryToken)
                        {
                                try {
                                        STARTUPINFO startup_info;
                                        ZeroMemory(&startup_info, sizeof(STARTUPINFO));
                                        startup_info.cb = sizeof(STARTUPINFO);
                                        startup_info.lpDesktop = "";

                                        BOOL result = CreateProcessAsUser(PrimaryToken,
                                                                          NULL,
                                                                          cmd_line_,
                                                                          NULL,
                                                                          NULL,
                                                                          FALSE,
                                                                          CREATE_NEW_PROCESS_GROUP | CREATE_NO_WINDOW | BELOW_NORMAL_PRIORITY_CLASS,
                                                                          NULL,
                                                                          NULL,
                                                                          &startup_info,
                                                                          &process_info_);
                                        if (result) {
                                                spawned_ = true;
                                                try {
                                                        CloseHandle(process_info_.hThread);
                                                }
                                                catch (...) {
                                                }
                                                return this->getpid();
                                        } else {
                                                BRUTUS_LOG_GETLASTERROR("CreateProcessAsUser failed");
                                                return -1;
                                        }
                                }
                                catch (...) {
                                        return -1;
                                }
                        };

        protected:
                bool spawned_;
                char *cmd_line_;
                PROCESS_INFORMATION process_info_;
        };
	class ProxyProcess : public BRUTUS_Process {
	public:
		ProxyProcess(void)
                        : BRUTUS_Process(),
                          sharable(true),
                          handle_(NULL),
                          mapi_profile(NULL),
                          ior_string(NULL)
			{
                                this->prepare(PROXY_EXE);
			};

		~ProxyProcess(void)
			{
				try {
                                        if (this->running())
        					this->kill();
				}
				catch (...) {
				}
                                try {
                                        CloseHandle(handle_);
				}
				catch (...) {
				}
                                if (mapi_profile)
                                        free(mapi_profile);
                                if (ior_string)
                                        free(ior_string);
			};

                void set_exit_thread_handle(HANDLE thread_handle)
                        {
                                handle_ = thread_handle;
                        };

                bool set_mapi_profile(const char *MAPIProfile)
                        {
                                if (mapi_profile)
                                        free(mapi_profile);
                                mapi_profile = strdup(MAPIProfile);

                                return (mapi_profile ? true : false);
                        };

                const char *get_mapi_profile(void)
                        {
                                return mapi_profile;
                        };

                bool set_session_ior_string(const char *SessionIOR)
                        {
                                if (ior_string)
                                        free(ior_string);
                                ior_string = strdup(SessionIOR);

                                return (ior_string ? true : false);
                        };

                const char *get_session_ior_string(void)
                        {
                                return ior_string;
                        };
        public:
                bool sharable;
                
        private:
                HANDLE handle_;
                char *mapi_profile;
                char *ior_string;
	};

	virtual ~BRUTUS_ProxyManager_i(void)
		{ 
			{
				ACE_Write_Guard<ACE_RW_Mutex> guard(spawn_mutex_);

				// this loop should be empty as the spawn list only holds spawns in progress
				for (SLit i = spawn_list_.begin(); i != spawn_list_.end(); i++) 
					delete i->second;
				spawn_list_.clear();
			}

			{
				ACE_Write_Guard<ACE_RW_Mutex> guard(proxy_mutex_);
				// send SIGINT to proxy processes
                                unsigned long n;
				for (n = 0; n < proxy_list_.size(); n++) 
					proxy_list_[n]->kill();

				// give the proxies 10 seconds to die gracefully
                                Sleep(10000);

				// now shoot any survivors
				for (n = 0; n < proxy_list_.size(); n++) 
					delete proxy_list_[n];

                                proxy_list_.resize(0);
			}
		};

	std::map<std::string, User*> spawn_list_; // temporary "while spawning" mapping of MAPIProfileName <==> ior/retval
	std::vector<ProxyProcess*> proxy_list_; // list of spawned processes

	ACE_RW_Mutex spawn_mutex_; // access to spawn_list_ and elements
	ACE_RW_Mutex proxy_mutex_; // access to proxy_list_ and elements
};


#endif /* __PROXY_MANAGERS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
