/*
 *    Brutus source file implementing a logging class.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <NT_Service/logging.h>
#include <NT_Service/corbaloc.h>
#include <NT_Service/config.h>
#include <NT_Service/defines.h>
#include <idl/BrutusSessionC.h>

#include <winsock2.h>
#include <process.h>


Log::Log(void)
        : orb_(CORBA::ORB::_nil()),
          log_server_(BRUTUS::LogServer::_nil()),
          hostname_(NULL),
          pid_(-1),
	  no_loging_(false)
{
        // configuration helper class
        Config *conf = new Config(SERVER_ORB_STACK); // stack name
        if (!conf) 
                throw 0;

        // load configuration file
        int retv = conf->load(BRUTUS_CONF);
        if (retv)
                throw 0;

	// get log server
        char *server = conf->get_value("LOG_SERVER");
	if (server && !strcmp(server, "none")) {
		free(server);
		no_loging_ = true;
		return;
	}

	try {
		int argc = 0;
		orb_ = CORBA::ORB_init(argc, NULL);
		if (CORBA::is_nil(orb_.in()))
			throw 0;
	}
	catch (...) {
		throw 0;
	}

        // load port configuration
        unsigned short port;
        {
                // get endpoint port numbers
                char *orb_cmd_line_option = conf->get_value(SERVER_ORB_STACK);
                while (orb_cmd_line_option) {
                        if (!strcmp(orb_cmd_line_option, "-ORBListenEndpoints"))
                                break;

                        free(orb_cmd_line_option);
                        orb_cmd_line_option = conf->get_value(SERVER_ORB_STACK);
                        if (!orb_cmd_line_option)
                                throw 0;
                }
                free(orb_cmd_line_option);
                orb_cmd_line_option = conf->get_value(SERVER_ORB_STACK);
                unsigned short iiop_port = 0;
                unsigned short ssl_port = 0; // 0 if not using SSL
                if (!get_ports(orb_cmd_line_option, iiop_port, ssl_port))
                        throw 0;
                free(orb_cmd_line_option);
                port = ssl_port ? ssl_port : iiop_port;
        }
        delete conf;
        conf = NULL;

        // locate log server
        if ((!server) || !strcmp(server, "localhost")) {
                try {
                        CORBA::Object_var obj = CORBA::Object::_nil();                
                        obj = orb_->string_to_object(LOG_SERVER_IOR);
                        log_server_ = ::BRUTUS::LogServer::_narrow(obj.in());

                }
                catch (...) {
                        throw 0;
                }
                if (CORBA::is_nil(log_server_.in()))
                        throw 0;
	} else {
                char *server_loc = corbaloc_str(::BRUTUS::BrutusSession::IOR_TABLE_KEY, server, port);

                try {
                        CORBA::Object_var obj = CORBA::Object::_nil();                
                        obj = orb_->string_to_object(server_loc);
                        log_server_ = ::BRUTUS::LogServer::_narrow(obj.in());

                }
                catch (...) {
                        throw 0;
                }
                if (CORBA::is_nil(log_server_.in()))
                        throw 0;

                free(server_loc);
        }
        free(server);

	char host[512]= { 0 };
	if (gethostname(host, 512))
		throw 0;
	hostname_ = strdup(host);	

        // get process ID
        pid_ = _getpid();
};


Log::~Log(void)
{
	if (no_loging_) 
		return;

	orb_->shutdown(1);
	orb_->destroy();

        free(hostname_);
};



void Log::add_entry(const BRUTUS::LogServer::log_severity Severity,
                    const char *File,
                    const unsigned long Line,
                    const char *Entry)
{
	if (no_loging_)
		return;

	{
		BRUTUS::LogServer::LogEntry entry;

		entry.mark = CORBA::string_dup("From a Log:: class instance");
		entry.hostname = CORBA::string_dup(hostname_);
		entry.process_id = pid_;
		entry.severity = Severity;
		entry.line = Line;
		if (!File)
			entry.file = CORBA::string_dup("");
		else
			entry.file = CORBA::string_dup(File);
		if (!Entry)
			entry.message = CORBA::string_dup("");
		else
			entry.message = CORBA::string_dup(Entry);

		try {
			log_server_->log(entry);
		}
		catch (...) {
		}
	}
}

unsigned long Log::add_leading_entry(const BRUTUS::LogServer::log_severity Severity,
                                     const char *File,
                                     const unsigned long Line,
                                     const char *Entry)
{
	if (no_loging_)
		return 0;

	{
		BRUTUS::LogServer::LogEntry entry;

		entry.mark = CORBA::string_dup("From a Log:: class instance");
		entry.hostname = CORBA::string_dup(hostname_);
		entry.process_id = pid_;
		entry.severity = Severity;
		entry.line = Line;
		if (!File)
			entry.file = CORBA::string_dup("");
		else
			entry.file = CORBA::string_dup(File);
		if (!Entry)
			entry.message = CORBA::string_dup("");
		else
			entry.message = CORBA::string_dup(Entry);
        
		unsigned long retv = 0;
		try {
			retv = log_server_->log_leader(entry);
		}
		catch (...) {
		}

		return retv;
	}
}


void Log::add_sub_entry(const BRUTUS::LogServer::log_severity Severity,
                        const char *File,
                        const unsigned long Line,
                        const char *Entry,
                        const unsigned long LeadingEntry)
{
	if (no_loging_)
		return;

	{
		BRUTUS::LogServer::LogEntry entry;

		entry.mark = CORBA::string_dup("From a Log:: class instance");
		entry.hostname = CORBA::string_dup(hostname_);
		entry.process_id = pid_;
		entry.severity = Severity;
		entry.line = Line;
		if (!File)
			entry.file = CORBA::string_dup("");
		else
			entry.file = CORBA::string_dup(File);
		if (!Entry)
			entry.message = CORBA::string_dup("");
		else
			entry.message = CORBA::string_dup(Entry);
        
		try {
			log_server_->log_follower(LeadingEntry, entry);
		}
		catch (...) {
		}
	}
}

