/*
 *    Brutus header file for a logging class.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef _LOGGING_H_
#define _LOGGING_H_

#include <ace/Singleton.h>
#include <ace/Synch.h>
#include <idl/log_serverC.h>

class Log {
public:
        Log(void);

        ~Log(void);

        void add_entry(const BRUTUS::LogServer::log_severity Severity,
                       const char *File,
                       const unsigned long Line,
                       const char *Entry);

        unsigned long add_leading_entry(const BRUTUS::LogServer::log_severity Severity,
                                        const char *File,
                                        const unsigned long Line,
                                        const char *Entry);

        void add_sub_entry(const BRUTUS::LogServer::log_severity Severity,
                           const char *File,
                           const unsigned long Line,
                           const char *Entry,
                           const unsigned long LeadingEntry);

private:
        CORBA::ORB_var orb_;
	BRUTUS::LogServer_var log_server_;
	char *hostname_;
	int pid_;
        bool no_loging_;
};

// Define a singleton class as a way to insure that there's only one
// Log instance in the program, and to protect against access from
// multiple threads.  The first reference to it at runtime creates it,
// and the ACE_Object_Manager deletes it at run-down.
typedef ACE_Singleton<Log, ACE_Recursive_Thread_Mutex> Log_Singleton;

#endif // _LOGGING_H_

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
