/*
 *    Brutus header file for the Brutus NT Service.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


//**************************************************************************
//
// BASED ON ACE SAMPLE CODE BY:
//    Gonzalo Diethelm <gonzo@cs.wustl.edu> 
//    and Steve Huston <shuston@riverace.com>
//
//**************************************************************************


#include <ace/Event_Handler.h>
#include <ace/NT_Service.h>
#include <ace/Singleton.h>
#include <ace/Synch.h>

#ifndef NTSVC_H_
#define NTSVC_H_

class Service : public ACE_NT_Service
{
public:
        Service(void);

        void SetOptions(const bool debug,
			const bool ProfilesOnDisk,
			const bool ProcessPerSession,
			const bool OnlyAdministrator,
			const bool Authenticate,
                        const unsigned long LifeCheckDelay);

        // We override <handle_control> because it handles stop requests
        // privately.
        virtual void handle_control(DWORD control_code);

        // We override <handle_exception> so a 'stop' control code can pop
        // the reactor off of its wait.
        virtual int handle_exception(ACE_HANDLE h);

        // This is a virtual method inherited from ACE_NT_Service.
        virtual int svc(void);

private:
        typedef ACE_NT_Service inherited;

	int stop_;

	bool debug_;
	bool profiles_on_disk_;
	bool process_per_session_;
	bool only_administrator_;
	bool authenticate_;
        unsigned long life_check_delay_;
};

// Define a singleton class as a way to insure that there's only one
// Service instance in the program, and to protect against access from
// multiple threads.  The first reference to it at runtime creates it,
// and the ACE_Object_Manager deletes it at run-down.
typedef ACE_Singleton<Service, ACE_Mutex> SERVICE;


#endif /* #ifndef NTSVC_H_ */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
