/*
 *    Brutus header file for POA utility functions.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef _POA_UTILS_H_
#define _POA_UTILS_H_

#include <NT_Service/ServantLib/ServantLib.h>
#include <vector>
#include <ace/Synch.h>
#include <tao/PortableServer/PortableServer.h>


extern SERVANTLIB_API PortableServer::POA_ptr create_poa(const char *Name, 
							 PortableServer::POAManager_ptr PoaManager,
							 PortableServer::POA_ptr Poa);

class SERVANTLIB_API SessionRegistry {
public:
        SessionRegistry() 
                { 
                        managed_entries_.resize(0);
                        blocked_users_.resize(0);
                        last_session_id = 0;
                };

        ~SessionRegistry();

        // will return a session id which must be used for reference later
        unsigned long add_entry(PortableServer::POA_ptr Poa, 
                                const char *ProfileName);

        void sign_out(unsigned long SessionID,
                      CORBA::Boolean wait_for_completion);

        void sign_out(const char *ProfileName,
                      CORBA::Boolean wait_for_completion);
 
        void block(const char *ProfileName);

        void unblock(const char *ProfileName);
        
        bool blocked(const char *ProfileName);

        char* get_profile_name(unsigned long SessionID);

private:

        unsigned long last_session_id;

        typedef struct SessionEntry_ {
                PortableServer::POA_var session_poa;
                char *profile_name;
                unsigned long id;
        } SessionEntry;
                
        std::vector<SessionEntry*> managed_entries_;
        ACE_RW_Mutex entry_mutex_;

        std::vector<char*> blocked_users_;
        ACE_RW_Mutex block_mutex_;

        std::string msg;
};


#endif // _POA_UTILS_H_

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
