/*
 *    Brutus source file for the BrutusSession servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <NT_Service/servant_impl/BrutusSessionS_impl.h>
#include <NT_Service/defines.h>


BRUTUS_BrutusSession_i::BRUTUS_BrutusSession_i(BRUTUS::ProxyManager_ptr ProxyManager,
                                               CORBA::ORB_ptr InternalORB)
        : proxy_manager_(BRUTUS::ProxyManager::_duplicate(ProxyManager)),
          log_server_(BRUTUS::LogServer::_nil())
{ 
        if (CORBA::is_nil(InternalORB))
                throw 0;

        if (CORBA::is_nil(proxy_manager_.in()))
                throw 0;

        try {
                CORBA::Object_var obj = CORBA::Object::_nil();                
                obj = InternalORB->string_to_object(LOG_SERVER_IOR);
                log_server_ = ::BRUTUS::LogServer::_narrow(obj.in());

        }
        catch (...) {
                throw 0;
        }
        if (CORBA::is_nil(log_server_.in()))
                throw 0;
}

CORBA::ULong BRUTUS_BrutusSession_i::getProxyCount(ACE_ENV_SINGLE_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        return proxy_manager_->getProxyCount();
}

void BRUTUS_BrutusSession_i::log(const ::BRUTUS::LogServer::LogEntry &Entry
                                 ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        log_server_->log(Entry);
}

CORBA::ULong BRUTUS_BrutusSession_i::log_leader(const ::BRUTUS::LogServer::LogEntry & Entry
                                                ACE_ENV_ARG_DECL )
        ACE_THROW_SPEC((CORBA::SystemException))
{
        return log_server_->log_leader(Entry);
}

void BRUTUS_BrutusSession_i::log_follower(::CORBA::ULong leader,
                                          const ::BRUTUS::LogServer::LogEntry & Entry
                                          ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        log_server_->log_follower(leader, Entry);
}

