/*
 *    Brutus header file for the IABLogon servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __IABLOGONS_IMPL_H_
#define __IABLOGONS_IMPL_H_

#include <idl/IABLogonS.h>
#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>
#include <mapispi.h>


#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_IABLogon_i : public virtual POA_BRUTUS::IABLogon,
                                         public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_IABLogon_i(LPABLOGON ABLogon,
                          LPMAPISESSION MAPISession,
                          PortableServer::POA_ptr Poa);
  
  
        virtual ::BRUTUS::BRESULT GetLastError (
                BRUTUS::BRESULT hResult,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::MAPIERROR_out lppMAPIError
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT Logoff(BRUTUS::BDEFINE ulFlags
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenEntry (
                const BRUTUS::ENTRYID & lpEntryID,
                const char * lpInterface,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::BDEFINE_out lpulObjType,
                BRUTUS::IUnknown_out lppUnk
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT CompareEntryIDs (
                const BRUTUS::ENTRYID & lpEntryID1,
                const BRUTUS::ENTRYID & lpEntryID2,
                BRUTUS::BDEFINE ulFlags,
                CORBA::Boolean_out lpulResult
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT Advise (
                const BRUTUS::ENTRYID & lpEntryID,
                CORBA::ULong ulEventMask,
                BRUTUS::IMAPIAdviseSink_ptr lpAdviseSink,
                CORBA::ULong_out lpulConnection
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT Unadvise (
                CORBA::ULong ulConnection
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT OpenStatusEntry (
                const char * lpInterface,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::BDEFINE_out lpulObjType,
                BRUTUS::IMAPIStatus_out lppEntry
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT OpenTemplateID (
                const BRUTUS::ENTRYID & lpTemplateID,
                BRUTUS::BDEFINE ulTemplateFlags,
                BRUTUS::IMAPIProp_ptr lpMAPIPropData,
                const char * lpInterface,
                BRUTUS::IMAPIProp_out lppMAPIPropNew,
                BRUTUS::IMAPIProp_ptr lpMAPIPropSibling
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT GetOneOffTable (
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::IMAPITable_out lppTable
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT PrepareRecips (
                BRUTUS::BDEFINE ulBDEFINE,
                const BRUTUS::SPropTagArray & lpPropTagArray,
                const BRUTUS::ADRLIST & lpRecipList
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        // Skeleton constructor 
        BRUTUS_IABLogon_i(void)
                { };

        
        virtual ~BRUTUS_IABLogon_i(void)
                { 
                        try {
                                ab_logon_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };


        LPABLOGON ab_logon_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __IABLOGONS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
