/*
 *    Brutus source file for the IExchangeImportContentsChanges servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <NT_Service/servant_impl/IExchangeImportContentsChangesS_impl.h>

#include <NT_Service/obj_utils.h>
#include <NT_Service/conv_utils.h>
#include <NT_Service/istream_utils.h>

#include <NT_Service/templates.i>

#include <NT_Service/servant_impl/IMessageS_impl.h>

static inline ULONG native_flags(const BRUTUS::BDEFINE Flags) 
{
        BRUTUS::BDEFINE flags = Flags;
        ULONG retval = 0;

	if (!Flags)
		return 0;         

        if (flags & BRUTUS::BRUTUS_SYNC_UNICODE) {
                retval |= SYNC_UNICODE;
                FLAGS_OFF(BRUTUS::BDEFINE, flags, BRUTUS::BRUTUS_SYNC_UNICODE);
        }
        if (flags & BRUTUS::BRUTUS_SYNC_ASSOCIATED) {
                retval |= SYNC_ASSOCIATED;
                FLAGS_OFF(BRUTUS::BDEFINE, flags, BRUTUS::BRUTUS_SYNC_ASSOCIATED);
        }

        if (flags) {
                char msg[128] = {0};
                sprintf(msg, "Unknown flag(s) from BRUTUS : %X", flags); 
                BRUTUS_LOG_BUG(msg);
        }

        return retval;
}

BRUTUS_IExchangeImportContentsChanges_i::BRUTUS_IExchangeImportContentsChanges_i(
        LPEXCHANGEIMPORTCONTENTSCHANGES ExchangeImportContentsChanges,
        LPMAPISESSION MAPISession,
        PortableServer::POA_ptr Poa)
        : exchange_import_contents_changes_(ExchangeImportContentsChanges), 
          mapi_session_(MAPISession),
          poa_(PortableServer::POA::_duplicate(Poa))
{
        mapi_session_->AddRef();
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::GetLastError(BRUTUS::BRESULT ReturnCode,
                                                                        BRUTUS::BDEFINE ulFlags,
                                                                        BRUTUS::MAPIERROR_out lppMAPIError
                                                                        ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::GetLastError()");

        BRUTUS::MAPIERROR_var error;
        try {
                error = new BRUTUS::MAPIERROR;
        }
        catch (std::bad_alloc &) {
                BRUTUS_LOG_SUB_ERR("No memory");
                throw CORBA::NO_MEMORY();
        }
        error->ulVersion = (CORBA::ULong)0;
        error->ulLowLevelError = (CORBA::ULong)0;
        error->ulContext = (CORBA::ULong)0;

        HRESULT hr;
        if (!bresult_to_hresult(ReturnCode, hr)) {
                lppMAPIError = error._retn();
                BRUTUS_LOG_SUB_BUG("Could not convert BRESULT into HRESULT");
                return BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
        }

        LPMAPIERROR mapi_error = NULL;
        ULONG flags = native_flags(ulFlags);
        
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->GetLastError(hr, flags, &mapi_error);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                MAPIFreeBuffer(mapi_error);
                lppMAPIError = error._retn();
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        if (!mapi_error) 
                BRUTUS_LOG_SUB_INF("No applicable error information from MAPI");

        mapierror_mapi_to_brutus(mapi_error, error, true);

        lppMAPIError = error._retn();

        BRUTUS_LOG_SUB_INF("Leaving Brutus_IExchangeImportContentsChanges_i::GetLastError");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::Config(BRUTUS::IStream_ptr lpStream,
                                                                  BRUTUS::BDEFINE ulFlags
                                                                  ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::Config()");

        ULONG flags = native_flags(ulFlags);

        LPSTREAM mapi_stream = NULL;
        if (!CORBA::is_nil(lpStream)) {
                try {
                        mapi_stream = new CStream(lpStream);
                }
                catch (...) {
                        BRUTUS_LOG_SUB_ERR("No memory");
                        throw CORBA::NO_MEMORY();
                }
        }

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->Config(mapi_stream, 
                                                               flags);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::Config()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::UpdateState(BRUTUS::IStream_ptr lpStream
                                                                       ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::UpdateState()");

        LPSTREAM mapi_stream = NULL;
        if (!CORBA::is_nil(lpStream)) {
                try {
                        mapi_stream = new CStream(lpStream);
                }
                catch (...) {
                        BRUTUS_LOG_SUB_ERR("No memory");
                        throw CORBA::NO_MEMORY();
                }
        }

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->UpdateState(mapi_stream);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::UpdateState()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::ImportMessageChange(const BRUTUS::seq_SPropValue & ppvalChanges,
                                                                               BRUTUS::BDEFINE ulFlags,
                                                                               BRUTUS::IMessage_out lppMessage
                                                                               ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::ImportMessageChange()");

        lppMessage = BRUTUS::IMessage::_nil();

        unsigned long count = 0;
        SPropValue *mapi_props = NULL;
        if (!spropvalue_array_brutus_to_mapi(&ppvalChanges, 0, count, mapi_props)) {
                MAPIFreeBuffer(mapi_props);
                BRUTUS_LOG_SUB_ERR("Could not convert Brutus SPropValue to MAPI SPropValue");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        ULONG flags = native_flags(ulFlags);
        LPMESSAGE mapi_message = NULL;

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->ImportMessageChange(count, 
                                                                            mapi_props,
                                                                            flags,
                                                                            &mapi_message);
        }
        MAPIFreeBuffer(mapi_props);
        
        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        if (hr == S_OK)
                lppMessage = create_object<BRUTUS_IMessage_i, BRUTUS::IMessage, LPMESSAGE>
                        (mapi_message, poa_.in(), mapi_session_);

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::ImportMessageChange()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::ImportMessageDeletion(BRUTUS::BDEFINE ulFlags,
                                                                                 const BRUTUS::SBinaryArray & lpSrcEntryList
                                                                                 ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;  
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::ImportMessageDeletion()");

        unsigned long flags = native_flags(ulFlags);

        SBinaryArray *mapi_bin_array = NULL;
        if (!sbinary_array_brutus_to_mapi(&lpSrcEntryList, 0, mapi_bin_array)) {
                BRUTUS_LOG_SUB_BUG("Conversion error");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->ImportMessageDeletion(flags, mapi_bin_array);
        }
        MAPIFreeBuffer(mapi_bin_array);

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        
        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::ImportMessageDeletion()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::ImportPerUserReadStateChange(const BRUTUS::seq_READSTATE & lpReadState
                                                                                        ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;  
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::ImportPerUserReadStateChange()");

        LPREADSTATE mapi_read_states = NULL;
        if (!readstate_array_brutus_to_mapi(&lpReadState, NULL, mapi_read_states)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                throw CORBA::NO_MEMORY();
        }

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->ImportPerUserReadStateChange(lpReadState.length(), mapi_read_states);
        }
        MAPIFreeBuffer(mapi_read_states);

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        
        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::ImportPerUserReadStateChange()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::ImportMessageMove(const BRUTUS::seq_octet & pbSourceKeySrcFolder,
                                                                             const BRUTUS::seq_octet & pbSourceKeySrcMessage,
                                                                             const BRUTUS::seq_octet & pbPCLMessage,
                                                                             const BRUTUS::seq_octet & pbSourceKeyDestMessage,
                                                                             const BRUTUS::seq_octet & pbChangeNumDestMessage
                                                                             ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;  
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::ImportMessageMove()");

        void *mapi_pbSourceKeySrcFolder = NULL;
        if (!seq_octet_brutus_to_mapi(pbSourceKeySrcFolder, NULL, mapi_pbSourceKeySrcFolder)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                throw CORBA::NO_MEMORY();
        }
        void *mapi_pbSourceKeySrcMessage = NULL;
        if (!seq_octet_brutus_to_mapi(pbSourceKeySrcMessage, mapi_pbSourceKeySrcFolder, mapi_pbSourceKeySrcMessage)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                MAPIFreeBuffer(mapi_pbSourceKeySrcFolder);
                throw CORBA::NO_MEMORY();
        }
        void *mapi_pbPCLMessage = NULL;
        if (!seq_octet_brutus_to_mapi(pbPCLMessage, mapi_pbSourceKeySrcFolder, mapi_pbPCLMessage)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                MAPIFreeBuffer(mapi_pbSourceKeySrcFolder);
                throw CORBA::NO_MEMORY();
        }
        void *mapi_pbSourceKeyDestMessage = NULL;
        if (!seq_octet_brutus_to_mapi(pbSourceKeyDestMessage, mapi_pbSourceKeySrcFolder, mapi_pbSourceKeyDestMessage)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                MAPIFreeBuffer(mapi_pbSourceKeySrcFolder);
                throw CORBA::NO_MEMORY();
        }
        void *mapi_pbChangeNumDestMessage = NULL;
        if (!seq_octet_brutus_to_mapi(pbChangeNumDestMessage, mapi_pbSourceKeySrcFolder, mapi_pbChangeNumDestMessage)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                MAPIFreeBuffer(mapi_pbSourceKeySrcFolder);
                throw CORBA::NO_MEMORY();
        }

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->ImportMessageMove(pbSourceKeySrcFolder.length(),
                                                                          (BYTE*)mapi_pbSourceKeySrcFolder,
                                                                          pbSourceKeySrcMessage.length(),
                                                                          (BYTE*)mapi_pbSourceKeySrcMessage,
                                                                          pbPCLMessage.length(),
                                                                          (BYTE*)mapi_pbPCLMessage,
                                                                          pbSourceKeyDestMessage.length(),
                                                                          (BYTE*)mapi_pbSourceKeyDestMessage,
                                                                          pbChangeNumDestMessage.length(),
                                                                          (BYTE*)mapi_pbChangeNumDestMessage);
        }
        MAPIFreeBuffer(mapi_pbSourceKeySrcFolder);

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        
        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::ImportMessageMove()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeImportContentsChanges_i::QueryInterface(const char *iid,
                                                                          BRUTUS::IUnknown_out ppvObject
                                                                          ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        ppvObject = BRUTUS::IUnknown::_nil();

        GUID mapi_iid;
        if (!guid_brutus_to_mapi_no_alloc(iid, &mapi_iid)) {
                BRUTUS_LOG_SUB_BUG("Could not convert BRUTUS::GUID into MAPI GUID");
                return BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
        }

        HRESULT hr;
        void *object = NULL;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_import_contents_changes_->QueryInterface(mapi_iid, &object);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        if (hr == S_OK) {
                if (!create_brutus_object(iid, (LPUNKNOWN)object, poa_, ppvObject, mapi_session_)) {
                        BRUTUS_LOG_SUB_BUG("Could not create brutus object.");
                        ((LPUNKNOWN)object)->Release();
                        return BRUTUS::BRUTUS_INTERNAL_ERROR;
                }
        }

        return br;
}

void BRUTUS_IExchangeImportContentsChanges_i::Destroy(CORBA::ULong InstanceID
						      ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeImportContentsChanges_i::Destroy()");

        PortableServer::ObjectId_var oid;
        oid = poa_->servant_to_id(this);

        poa_->deactivate_object(oid);

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeImportContentsChanges_i::Destroy()");
}

