/*
 *    Brutus header file for the IExchangeManageStore servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __IEXCHANGEMANAGESTORES_IMPL_H_
#define __IEXCHANGEMANAGESTORES_IMPL_H_

#include <idl/IExchangeManageStoreS.h>

#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>
#include <NT_Service/edkinc/edkmdb.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_IExchangeManageStore_i : public virtual POA_BRUTUS::IExchangeManageStore, 
                                                     public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_IExchangeManageStore_i(LPEXCHANGEMANAGESTORE ExchangeManageStore,
                                      LPMAPISESSION MAPISession,
                                      PortableServer::POA_ptr Poa);
  
        virtual ::BRUTUS::BRESULT CreateStoreEntryID(const char * lpszMsgStoreDN,
                                                     const char * lpszMailboxDN,
                                                     BRUTUS::BDEFINE ulFlags,
                                                     BRUTUS::ENTRYID_out lppEntryID
                                                     ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT EntryIDFromSourceKey(const BRUTUS::seq_octet & lpFolderSourceKey,
                                                       const BRUTUS::seq_octet & lpMessageSourceKey,
                                                       BRUTUS::ENTRYID_out lppEntryID
                                                       ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT GetRights(const BRUTUS::ENTRYID & lpUserEntryID,
                                            const BRUTUS::ENTRYID & lpEntryID,
                                            CORBA::ULong_out lpulRights
                                            ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT GetMailboxTable(const char * lpszServerName,
                                                  BRUTUS::IMAPITable_out lppTable,
                                                  BRUTUS::BDEFINE ulFlags
                                                  ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT GetPublicFolderTable(const char * lpszServerName,
                                                       BRUTUS::IMAPITable_out lppTable,
                                                       BRUTUS::BDEFINE ulFlags
                                                       ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_IExchangeManageStore_i(void);
  
        virtual ~BRUTUS_IExchangeManageStore_i(void)
                {
                        try {
                                exchange_manage_store_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        LPEXCHANGEMANAGESTORE exchange_manage_store_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __IEXCHANGEMANAGESTORES_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
