/*
 *    Brutus source file for the IExchangeModifyTable servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include <NT_Service/servant_impl/IExchangeModifyTableS_impl.h>

#include <NT_Service/obj_utils.h>
#include <NT_Service/conv_utils.h>

#include <NT_Service/servant_impl/IMAPITableS_impl.h>

#include <NT_Service/templates.i>

static inline ULONG native_flags(const BRUTUS::BDEFINE Flags) 
{
        BRUTUS::BDEFINE flags = Flags;
        ULONG retval = 0;

	if (!Flags)
		return 0;         

        if (flags & BRUTUS::BRUTUS_ROWLIST_REPLACE) {
                retval |= ROWLIST_REPLACE;
                FLAGS_OFF(BRUTUS::BDEFINE, flags, BRUTUS::BRUTUS_ROWLIST_REPLACE);
        }

        if (flags) {
                char msg[128] = {0};
                sprintf(msg, "Unknown flag(s) from BRUTUS : %X", flags); 
                BRUTUS_LOG_BUG(msg);
        }

        return retval;
}

BRUTUS_IExchangeModifyTable_i::BRUTUS_IExchangeModifyTable_i(LPEXCHANGEMODIFYTABLE ExchangeModifyTable,
                                                             LPMAPISESSION MAPISession,
                                                             PortableServer::POA_ptr Poa)
        : exchange_modify_table_(ExchangeModifyTable), 
          mapi_session_(MAPISession),
          poa_(PortableServer::POA::_duplicate(Poa))
{
        mapi_session_->AddRef();
}


::BRUTUS::BRESULT BRUTUS_IExchangeModifyTable_i::GetLastError(BRUTUS::BRESULT ReturnCode,
                                                              BRUTUS::BDEFINE ulFlags,
                                                              BRUTUS::MAPIERROR_out lppMAPIError
                                                              ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeModifyTable_i::GetLastError()");

        BRUTUS::MAPIERROR_var error;
        try {
                error = new BRUTUS::MAPIERROR;
        }
        catch (std::bad_alloc &) {
                BRUTUS_LOG_SUB_ERR("No memory");
                throw CORBA::NO_MEMORY();
        }
        error->ulVersion = (CORBA::ULong)0;
        error->ulLowLevelError = (CORBA::ULong)0;
        error->ulContext = (CORBA::ULong)0;

        HRESULT hr;
        if (!bresult_to_hresult(ReturnCode, hr)) {
                lppMAPIError = error._retn();
                BRUTUS_LOG_SUB_BUG("Could not convert BRESULT into HRESULT");
                return BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
        }

        LPMAPIERROR mapi_error = NULL;
        unsigned long flags = native_flags(ulFlags);
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_modify_table_->GetLastError(hr, flags, &mapi_error);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                MAPIFreeBuffer(mapi_error);
                lppMAPIError = error._retn();
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        if (!mapi_error) 
                BRUTUS_LOG_SUB_INF("No applicable error information from MAPI");

        mapierror_mapi_to_brutus(mapi_error, error, true);

        lppMAPIError = error._retn();

        BRUTUS_LOG_SUB_INF("Leaving Brutus_IExchangeModifyTable_i::GetLastError");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeModifyTable_i::GetTable(BRUTUS::BDEFINE ulFlags,
                                                          BRUTUS::IMAPITable_out lppTable
                                                          ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;  
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeModifyTable_i::GetTable()");

        lppTable = BRUTUS::IMAPITable::_nil();
        LPMAPITABLE mapi_table = NULL;

        unsigned long flags = native_flags(ulFlags);

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_modify_table_->GetTable(flags,
                                                      &mapi_table);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        
        lppTable = create_object<BRUTUS_IMAPITable_i, BRUTUS::IMAPITable, LPMAPITABLE>
                (mapi_table, poa_.in(), mapi_session_);

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeModifyTable_i::GetTable()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeModifyTable_i::ModifyTable(BRUTUS::BDEFINE ulFlags,
                                                             const BRUTUS::ROWLIST & lpMods
                                                             ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;  
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeModifyTable_i::ModifyTable()");

        LPROWLIST mapi_rowlist = NULL;
        if (!rowentry_array_brutus_to_mapi(&lpMods, NULL, mapi_rowlist)) {
                BRUTUS_LOG_SUB_ERR("No memory");
                throw CORBA::NO_MEMORY();
        }

        unsigned long flags = native_flags(ulFlags);

        HRESULT hr;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_modify_table_->ModifyTable(flags,
                                                         mapi_rowlist);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }
        
        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IExchangeModifyTable_i::ModifyTable()");
        return br;
}

::BRUTUS::BRESULT BRUTUS_IExchangeModifyTable_i::QueryInterface(const char *iid,
                                                                BRUTUS::IUnknown_out ppvObject
                                                                ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        ppvObject = BRUTUS::IUnknown::_nil();

        GUID mapi_iid;
        if (!guid_brutus_to_mapi_no_alloc(iid, &mapi_iid)) {
                BRUTUS_LOG_SUB_BUG("Could not convert BRUTUS::GUID into MAPI GUID");
                return BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
        }

        HRESULT hr;
        void *object = NULL;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = exchange_modify_table_->QueryInterface(mapi_iid, &object);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        if (hr == S_OK) {
                if (!create_brutus_object(iid, (LPUNKNOWN)object, poa_, ppvObject, mapi_session_)) {
                        BRUTUS_LOG_SUB_BUG("Could not create brutus object.");
                        ((LPUNKNOWN)object)->Release();
                        return BRUTUS::BRUTUS_INTERNAL_ERROR;
                }
        }

        return br;
}

void BRUTUS_IExchangeModifyTable_i::Destroy(CORBA::ULong InstanceID
					    ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeModifyTable_i::Destroy()");

        PortableServer::ObjectId_var oid;
        oid = poa_->servant_to_id(this);

        poa_->deactivate_object(oid);

        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IExchangeModifyTable_i::Destroy()");
}

