/*
 *    Brutus source file for the IMAPIControl servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <NT_Service/servant_impl/IMAPIControlS_impl.h>

#include <NT_Service/obj_utils.h>
#include <NT_Service/conv_utils.h>

  
BRUTUS_IMAPIControl_i::BRUTUS_IMAPIControl_i(LPMAPICONTROL MAPIControl,
                                             LPMAPISESSION MAPISession,
                                             PortableServer::POA_ptr Poa)
        : mapi_control_(MAPIControl), 
          mapi_session_(MAPISession),
          poa_(PortableServer::POA::_duplicate(Poa))
{
        mapi_session_->AddRef();
}
 

::BRUTUS::BRESULT BRUTUS_IMAPIControl_i::GetLastError (
        BRUTUS::BRESULT bResult,
        BRUTUS::BDEFINE ulFlags,
        BRUTUS::MAPIERROR_out lppMAPIError
        ACE_ENV_ARG_DECL
        )
        ACE_THROW_SPEC ((
                                CORBA::SystemException
                                ))
{
        // Add your implementation here
        return BRUTUS::BRUTUS_MAPI_E_NO_SUPPORT;
}

::BRUTUS::BRESULT BRUTUS_IMAPIControl_i::Activate (
        BRUTUS::BDEFINE ulFlags
        ACE_ENV_ARG_DECL
        )
        ACE_THROW_SPEC ((
                                CORBA::SystemException
                                ))
{
        // Add your implementation here
        return BRUTUS::BRUTUS_MAPI_E_NO_SUPPORT;
}

::BRUTUS::BRESULT BRUTUS_IMAPIControl_i::GetState (
        BRUTUS::BDEFINE ulFlags,
        BRUTUS::BDEFINE_out lpulState
        ACE_ENV_ARG_DECL
        )
        ACE_THROW_SPEC ((
                                CORBA::SystemException
                                ))
{
        // Add your implementation here
        return BRUTUS::BRUTUS_MAPI_E_NO_SUPPORT;
}


::BRUTUS::BRESULT BRUTUS_IMAPIControl_i::QueryInterface(const char *iid,
                                                        BRUTUS::IUnknown_out ppvObject
                                                        ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        ppvObject = BRUTUS::IUnknown::_nil();

        GUID mapi_iid;
        if (!guid_brutus_to_mapi_no_alloc(iid, &mapi_iid)) {
                BRUTUS_LOG_SUB_BUG("Could not convert BRUTUS::GUID into MAPI GUID");
                return BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
        }

        HRESULT hr;
        void *object = NULL;
        {
                ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
                hr = mapi_control_->QueryInterface(mapi_iid, &object);
        }

        BRUTUS::BRESULT br;
        if (!hresult_to_bresult(hr, br)) {
                BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
                return BRUTUS::BRUTUS_INTERNAL_ERROR;
        }

        if (hr == S_OK) {
                if (!create_brutus_object(iid, (LPUNKNOWN)object, poa_, ppvObject, mapi_session_)) {
                        BRUTUS_LOG_SUB_BUG("Could not create brutus object.");
                        ((LPUNKNOWN)object)->Release();
                        return BRUTUS::BRUTUS_INTERNAL_ERROR;
                }
        }

        return br;
}

void BRUTUS_IMAPIControl_i::Destroy(CORBA::ULong InstanceID
				    ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        BRUTUS_LOG_REF;
        BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPIControl_i::Destroy()");

        PortableServer::ObjectId_var oid;
        oid = poa_->servant_to_id(this);

        poa_->deactivate_object(oid);

        BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPIControl_i::Destroy()");
}
  
