/*
 *    Brutus header file for the IMAPIFormInfo servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __IMAPIFORMINFOS_IMPL_H_
#define __IMAPIFORMINFOS_IMPL_H_

#include <idl/IMAPIFormInfoS.h>

#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>
#include <mapiform.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_IMAPIFormInfo_i : public virtual POA_BRUTUS::IMAPIFormInfo,
                                              public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_IMAPIFormInfo_i(LPMAPIFORMINFO FormInfo,
                               LPMAPISESSION MAPISession,
                               PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::BRESULT GetLastError (
                BRUTUS::BRESULT bResult,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::MAPIERROR_out lppMAPIError
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT SaveChanges (
                BRUTUS::BDEFINE ulFlags
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT GetProps (
                const BRUTUS::SPropTagArray & lpPropTagArray,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::seq_SPropValue_out lppPropArray
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT GetPropList (
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::SPropTagArray_out lppPropTagArray
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT OpenProperty (
                CORBA::ULong ulPropTag,
                const char * lpiid,
                CORBA::ULong ulInterfaceOptions,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::IUnknown_out lppUnk
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT SetProps (
                const BRUTUS::seq_SPropValue & lpPropArray,
                CORBA::Boolean ProblemInfo,
                BRUTUS::SPropProblemArray_out lppProblems
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT DeleteProps (
                const BRUTUS::SPropTagArray & lpPropTagArray,
                CORBA::Boolean ProblemInfo,
                BRUTUS::SPropProblemArray_out lppProblems
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT CopyTo (
                const BRUTUS::seq_GUID & rgiidExclude,
                const BRUTUS::SPropTagArray & lpExcludeProps,
                BRUTUS::IMAPIProgress_ptr lpProgress,
                const char * lpInterface,
                const BRUTUS::ENTRYID & lpDestObj,
                BRUTUS::BDEFINE ulFlags,
                CORBA::Boolean ProblemInfo,
                BRUTUS::SPropProblemArray_out lppProblems
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT CopyProps (
                const BRUTUS::SPropTagArray & lpIncludeProps,
                BRUTUS::IMAPIProgress_ptr lpProgress,
                const char * lpInterface,
                const BRUTUS::ENTRYID & lpDestObj,
                BRUTUS::BDEFINE ulFlags,
                CORBA::Boolean ProblemInfo,
                BRUTUS::SPropProblemArray_out lppProblems
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT GetNamesFromIDs (
                BRUTUS::SPropTagArray & lppPropTags,
                const char * lpPropSetGuid,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::seq_MAPINAMEID_out lpppPropNames
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

        virtual ::BRUTUS::BRESULT GetIDsFromNames (
                const BRUTUS::seq_MAPINAMEID & lppPropNames,
                BRUTUS::BDEFINE ulFlags,
                BRUTUS::SPropTagArray_out lppPropTags
                ACE_ENV_ARG_DECL
                )
                ACE_THROW_SPEC ((
                                        CORBA::SystemException
                                        ));

  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC ((CORBA::SystemException));
  
private:
        BRUTUS_IMAPIFormInfo_i (void)
                { };

        virtual ~BRUTUS_IMAPIFormInfo_i(void)
                { 
                        try {
                                form_info_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        LPMAPIFORMINFO form_info_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_ptr poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __IMAPIFORMINFOS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
