/*
 *    Brutus source file for the IMAPISession servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#include <NT_Service/servant_impl/IMAPISessionS_impl.h>

#include <NT_Service/obj_utils.h>
#include <NT_Service/conv_utils.h>
#include <NT_Service/istream_utils.h>
#include <NT_Service/edkguid.h>
#include <NT_Service/guid.h>

#include <NT_Service/servant_impl/IMAPITableS_impl.h>
#include <NT_Service/servant_impl/IMsgStoreS_impl.h>
#include <NT_Service/servant_impl/IAddrBookS_impl.h>
#include <NT_Service/servant_impl/IProfSectS_impl.h>
#include <NT_Service/servant_impl/IMAPIFolderS_impl.h>
#include <NT_Service/servant_impl/IABContainerS_impl.h>
#include <NT_Service/servant_impl/IMessageS_impl.h>
#include <NT_Service/servant_impl/IAttachS_impl.h>
#include <NT_Service/servant_impl/IDistListS_impl.h>
#include <NT_Service/servant_impl/IMAPIStatusS_impl.h>
#include <NT_Service/servant_impl/IMailUserS_impl.h>
#include <NT_Service/servant_impl/IMsgServiceAdminS_impl.h>
#include <NT_Service/servant_impl/ITnefS_impl.h>
#include <NT_Service/servant_impl/IProfAdminS_impl.h>
#include <NT_Service/servant_impl/IPropDataS_impl.h>

#include <NT_Service/templates.i>

#include <lm.h>

#include <mapiutil.h>


static inline unsigned long native_message_store_flags(const ::BRUTUS::BDEFINE Flags)
{
	if (Flags & ::BRUTUS::BRUTUS_MAPI_DEFAULT_STORE)
		return MAPI_DEFAULT_STORE;

	if (Flags & ::BRUTUS::BRUTUS_MAPI_SIMPLE_STORE_TEMPORARY)
		return MAPI_SIMPLE_STORE_TEMPORARY;

	if (Flags & ::BRUTUS::BRUTUS_MAPI_SIMPLE_STORE_PERMANENT)
		return MAPI_SIMPLE_STORE_PERMANENT;

	if (Flags & ::BRUTUS::BRUTUS_MAPI_PRIMARY_STORE)
		return MAPI_PRIMARY_STORE;

	if (Flags & ::BRUTUS::BRUTUS_MAPI_SECONDARY_STORE)
		return MAPI_SECONDARY_STORE;

	{
		char msg[128] = {0};
		sprintf(msg, "Unknown message store flag from BRUTUS : %X", Flags);
		BRUTUS_LOG_BUG(msg);

		return 0;
	}
}

static inline ULONG native_flags(const ::BRUTUS::BDEFINE ulFlags)
{
	::BRUTUS::BDEFINE flags = ulFlags;
	ULONG retval = 0;

	if (!ulFlags)
		return 0;

	if (flags & ::BRUTUS::BRUTUS_MAPI_UNICODE) {
		retval |= MAPI_UNICODE;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MAPI_UNICODE);
	}
	if (flags & ::BRUTUS::BRUTUS_MAPI_BEST_ACCESS) {
		retval |= MAPI_BEST_ACCESS;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MAPI_BEST_ACCESS);
	}
	if (flags & ::BRUTUS::BRUTUS_MAPI_DEFERRED_ERRORS) {
		retval |= MAPI_DEFERRED_ERRORS;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MAPI_DEFERRED_ERRORS);
	}
	if (flags & ::BRUTUS::BRUTUS_MDB_NO_MAIL) {
		retval |= MDB_NO_MAIL;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MDB_NO_MAIL);
	}
	if (flags & ::BRUTUS::BRUTUS_MDB_TEMPORARY) {
		retval |= MDB_TEMPORARY;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MDB_TEMPORARY);
	}
	if (flags & ::BRUTUS::BRUTUS_MDB_WRITE) {
		retval |= MDB_WRITE;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MDB_WRITE);
	}
	if (flags & ::BRUTUS::BRUTUS_MAPI_MODIFY) {
		retval |= MAPI_MODIFY;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_MAPI_MODIFY);
	}
	if (flags & ::BRUTUS::BRUTUS_OPENSTORE_HOME_LOGON) {
		retval |= OPENSTORE_HOME_LOGON;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_OPENSTORE_HOME_LOGON);
	}
	if (flags & ::BRUTUS::BRUTUS_OPENSTORE_OVERRIDE_HOME_MDB) {
		retval |= OPENSTORE_OVERRIDE_HOME_MDB;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_OPENSTORE_OVERRIDE_HOME_MDB);
	}
	if (flags & ::BRUTUS::BRUTUS_OPENSTORE_PUBLIC) {
		retval |= OPENSTORE_PUBLIC;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_OPENSTORE_PUBLIC);
	}
	if (flags & ::BRUTUS::BRUTUS_OPENSTORE_TAKE_OWNERSHIP) {
		retval |= OPENSTORE_TAKE_OWNERSHIP;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_OPENSTORE_TAKE_OWNERSHIP);
	}
	if (flags & ::BRUTUS::BRUTUS_OPENSTORE_USE_ADMIN_PRIVILEGE) {
		retval |= OPENSTORE_USE_ADMIN_PRIVILEGE;
		FLAGS_OFF(::BRUTUS::BDEFINE, flags, ::BRUTUS::BRUTUS_OPENSTORE_USE_ADMIN_PRIVILEGE);
	}

	if (flags) {
		char msg[128] = {0};
		sprintf(msg, "Unknown flag(s) from BRUTUS : %X", flags);
		BRUTUS_LOG_BUG(msg);
	}

	return retval;
}

static inline ULONG native_eventmask(const ::BRUTUS::BDEFINE EventMask)
{
	::BRUTUS::BDEFINE event_mask = EventMask;
	ULONG retval = 0;

	if (!EventMask)
		return 0;

	if (event_mask & ::BRUTUS::BRUTUS_fnevCriticalError) {
		retval |= fnevCriticalError;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevCriticalError);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevNewMail) {
		retval |= fnevNewMail;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevNewMail);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevObjectCreated) {
		retval |= fnevObjectCreated;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevObjectCreated);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevObjectDeleted) {
		retval |= fnevObjectDeleted;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevObjectDeleted);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevObjectModified) {
		retval |= fnevObjectModified;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevObjectModified);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevObjectMoved) {
		retval |= fnevObjectMoved;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevObjectMoved);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevObjectCopied) {
		retval |= fnevObjectCopied;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevObjectCopied);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevSearchComplete) {
		retval |= fnevSearchComplete;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevSearchComplete);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevTableModified) {
		retval |= fnevTableModified;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevTableModified);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevStatusObjectModified) {
		retval |= fnevStatusObjectModified;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevStatusObjectModified);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevReservedForMapi) {
		retval |= fnevReservedForMapi;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevReservedForMapi);
	}

	if (event_mask & ::BRUTUS::BRUTUS_fnevExtended) {
		retval |= fnevExtended;
		FLAGS_OFF(::BRUTUS::BDEFINE, event_mask, ::BRUTUS::BRUTUS_fnevExtended);
	}

	if (event_mask) {
		char msg[128] = {0};
		sprintf(msg, "Unknown event mask flag(s) from BRUTUS : %X", event_mask);
		BRUTUS_LOG_BUG(msg);
	}


	return retval;
}


static inline HRESULT get_default_message_store(LPMAPISESSION Session,
						LPMDB *DefMsgStore)
{
	ULONG cRows = 0;
	LPSRowSet lpRows = NULL;
	LPENTRYID entry_id = NULL;
	ULONG entry_id_size = 0;
	ULONG i;

	// Columns to return from the table
	SizedSPropTagArray(2, prop_tags) = {2, {PR_DEFAULT_STORE, PR_ENTRYID} };

	// Get list of available message stores
	HRESULT hr;
	LPMAPITABLE mapi_table = NULL;
	hr = Session->GetMsgStoresTable(0, &mapi_table);
	if (S_OK != hr)
		goto exit;

	// Get row count
	hr = mapi_table->GetRowCount(0, &cRows);
	if ((S_OK != hr) || !cRows)
		goto exit;

	// Select columns to return from the table
	hr = mapi_table->SetColumns((LPSPropTagArray)&prop_tags, 0);
	if (S_OK != hr)
		goto exit;

	// Go to the beginning of the recipient table for the envelope
	hr = mapi_table->SeekRow(BOOKMARK_BEGINNING, 0, NULL);
	if (S_OK != hr)
		goto exit;

	// Read all the rows of the table
	hr = mapi_table->QueryRows(cRows, 0, &lpRows);
	if ((S_OK == hr) && (lpRows != NULL) && (lpRows->cRows == 0))
		goto exit;

	mapi_table->Release();
	mapi_table = NULL;

	for (i = 0; i < cRows; i++) {
		// check PR_DEFAULT_STORE boolean
		if (lpRows->aRow[i].lpProps[0].Value.b == TRUE) {
			entry_id = (LPENTRYID)lpRows->aRow[i].lpProps[1].Value.bin.lpb;
			entry_id_size = lpRows->aRow[i].lpProps[1].Value.bin.cb;
			break;
		}
	}

	hr = Session->OpenMsgStore(0,
				   entry_id_size,
				   entry_id,
				   NULL,
				   MAPI_BEST_ACCESS | MDB_NO_DIALOG,
				   DefMsgStore);
exit:
	FreeProws(lpRows);

	if (mapi_table)
		mapi_table->Release();

	return hr;
}

struct thread_args {
	unsigned long delay;
	HANDLE stop_handle;
};
DWORD WINAPI DispatchThreadProc(LPVOID lpParameter)
{
	struct thread_args *args = (struct thread_args*)lpParameter;
	unsigned long delay = 0;
	if (args->delay > 86400) // 24 hours
		args->delay = 86400;

	delay = args->delay * 1000;
	do {
		if (S_OK != HrDispatchNotifications(0))
			BRUTUS_LOG_BUG("HrDispatchNotifications(0) returned an error");

		if (WAIT_OBJECT_0 == WaitForSingleObject(args->stop_handle, delay))
			break;
	} while (true);

	HeapFree(GetProcessHeap(), 0, lpParameter);

	return 0;
}


struct life_line_thread_args {
	unsigned long delay;
	BRUTUS_IMAPISession_i *session;
	HANDLE client_handle;
};
DWORD WINAPI LifeLineThreadProc(LPVOID lpParameter)
{
	struct life_line_thread_args *args = (struct life_line_thread_args*)lpParameter;
	BRUTUS_IMAPISession_i *session = args->session;
	unsigned long delay = 0;

	if (!args->delay) // minimum supervising
		args->delay = 86400;
	else if (args->delay > 86400) // (24 hours) we won't wait for eternity
		args->delay = 86400;

	delay = args->delay * 1000; // to miliseconds - 0x7FFFFFFF is the maximum number which is not equal to infinity

	do {
		session->checkLifelines();

		if (WAIT_OBJECT_0 == WaitForSingleObject(args->client_handle, delay))
			break;
	} while (true);

	CloseHandle(args->client_handle);
	HeapFree(GetProcessHeap(), 0, lpParameter);

	return 0;
}

BRUTUS_IMAPISession_i::BRUTUS_IMAPISession_i(LPMAPISESSION MAPISession,
					     CORBA::ULong MaxNotifyDelay,
					     PortableServer::POA_ptr Poa,
					     SessionRegistry *Registry,
					     unsigned long RegistryID,
					     HANDLE &Stop,
					     ::BRUTUS::BrutusCheck_ptr LifeLine,
					     const unsigned long LifeCheckDelay)
	: mapi_session_(MAPISession),
	  notify_delay_((unsigned long)MaxNotifyDelay),
	  poa_(PortableServer::POA::_duplicate(Poa)),
	  registry_(Registry),
	  id_(RegistryID),
	  ref_count_(1),
	  stop_(Stop),
	  life_line_counter_(0)
{
	advise_sinks_.Clear();

	{ // add the first lifeline
		ACE_Write_Guard<ACE_RW_Mutex> guard(life_check_mutex_);
		life_lines_.clear();
		life_lines_.push_back(new LifeCheck(LifeLine, life_line_counter_));
	}
	/*******************************************************************/
	/* Create a thread to force MAPI notification at regular intervals */
	/*******************************************************************/

	HANDLE thread = NULL;
	if ((notify_delay_) && stop_) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);

		// launch thread to dispatch notifications forcefully
		struct thread_args *args = NULL;
		try {
			args = (struct thread_args*)HeapAlloc(GetProcessHeap(),
							      0,
							      sizeof(struct thread_args));
		}
		catch (...) {
			throw CORBA::NO_MEMORY();
		}
		if (!args) {
			throw CORBA::NO_MEMORY();
		}

		args->delay = notify_delay_;
		args->stop_handle = stop_;
		thread = CreateThread(NULL,
				      0,
				      DispatchThreadProc,
				      (void*)args,
				      CREATE_SUSPENDED,
				      NULL);
		if (!thread)
			throw CORBA::NO_MEMORY();
		SetThreadPriority(thread, THREAD_PRIORITY_LOWEST);
	}
	if (thread)
		ResumeThread(thread);

	/*********************************************/
	/* Create a thread to check for dead clients */
	/*********************************************/

	client_dead_handle_ = CreateEvent(NULL, true, false, NULL);
	if (!client_dead_handle_)
		throw CORBA::NO_MEMORY();

	HANDLE life_thread = NULL;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);

		// launch thread to dispatch notifications forcefully
		struct life_line_thread_args *args = NULL;
		try {
			args = (struct life_line_thread_args*)HeapAlloc(GetProcessHeap(),
									0,
									sizeof(struct life_line_thread_args));
		}
		catch (...) {
			throw CORBA::NO_MEMORY();
		}
		if (!args) {
			throw CORBA::NO_MEMORY();
		}
		args->delay = LifeCheckDelay;
		args->session = this;
		args->client_handle = client_dead_handle_;

		life_thread = CreateThread(NULL,
					   0,
					   LifeLineThreadProc,
					   (void*)args,
					   CREATE_SUSPENDED,
					   NULL);
		if (!life_thread)
			throw CORBA::NO_MEMORY();
		SetThreadPriority(life_thread, THREAD_PRIORITY_LOWEST);
	}
	if (life_thread)
		ResumeThread(life_thread);
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetLastError(::BRUTUS::BRESULT ReturnCode,
						      ::BRUTUS::BDEFINE ulFlags,
						      ::BRUTUS::MAPIERROR_out lppMAPIError
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetLastError()");

	::BRUTUS::MAPIERROR_var error;
	try {
		error = new ::BRUTUS::MAPIERROR;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	error->ulVersion = (CORBA::ULong)0;
	error->ulLowLevelError = (CORBA::ULong)0;
	error->ulContext = (CORBA::ULong)0;

	LPMAPIERROR mapi_error = NULL;
	unsigned long flags = native_flags(ulFlags);

	HRESULT hr;
	::BRUTUS::BRESULT br;
	if (!bresult_to_hresult(ReturnCode, hr)) {
		lppMAPIError = error._retn();
		BRUTUS_LOG_SUB_BUG("Could not convert BRESULT into HRESULT");
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		BRUTUS_SUB_EXIT;
	}

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetLastError(hr, flags, &mapi_error);
	}

	if (!hresult_to_bresult(hr, br)) {
		MAPIFreeBuffer(mapi_error);
		lppMAPIError = error._retn();
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}
	if (!mapi_error)
		BRUTUS_LOG_SUB_INF("No applicable error information from MAPI");

	mapierror_mapi_to_brutus(mapi_error, error, true);

	lppMAPIError = error._retn();

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetLastError()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetMsgStoresTable(::BRUTUS::BDEFINE ulFlags,
							   ::BRUTUS::IMAPITable_out lppTable
							   ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetMsgStoresTable()");

	lppTable = ::BRUTUS::IMAPITable::_nil();
	LPMAPITABLE mapi_table = NULL;
	unsigned long flags = native_flags(ulFlags);

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetMsgStoresTable(flags, &mapi_table);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppTable = create_object<BRUTUS_IMAPITable_i, ::BRUTUS::IMAPITable, LPMAPITABLE>
			(mapi_table, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetMsgStoresTable()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenMsgStore(const ::BRUTUS::ENTRYID &lpEntryID,
						      const char *lpInterface,
						      ::BRUTUS::BDEFINE ulFlags,
						      ::BRUTUS::IMsgStore_out lppMDB
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenMsgStore()");

	lppMDB = ::BRUTUS::IMsgStore::_nil();

	unsigned long flags = native_flags(ulFlags);
	flags |= MDB_NO_DIALOG;

	GUID* i_id;
	if (!guid_brutus_to_mapi(lpInterface, NULL, i_id)) {
		BRUTUS_LOG_SUB_BUG("Could not convert ::BRUTUS::GUID into MAPI GUID");
		return ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
	}

	ENTRYID *entry_id;
	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&lpEntryID, 0, entry_id_size, entry_id)) {
		MAPIFreeBuffer(i_id);
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	LPMDB msg_store = NULL;
	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenMsgStore(0,
						 entry_id_size,
						 entry_id,
						 i_id,
						 flags,
						 &msg_store);
	}
	MAPIFreeBuffer(entry_id);
	MAPIFreeBuffer(i_id);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppMDB = create_object<BRUTUS_IMsgStore_i, ::BRUTUS::IMsgStore, LPMDB>
			(msg_store, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenMsgStore()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenAddressBook(const char *lpInterface,
							 ::BRUTUS::BDEFINE ulFlags,
							 ::BRUTUS::IAddrBook_out lppAdrBook
							 ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenAddressBook()");

	lppAdrBook = ::BRUTUS::IAddrBook::_nil();

	unsigned long flags = native_flags(ulFlags);
	flags |= AB_NO_DIALOG;

	GUID *i_id;
	if (!guid_brutus_to_mapi(lpInterface, NULL, i_id)) {
		BRUTUS_LOG_SUB_BUG("Could not convert ::BRUTUS::GUID into MAPI GUID");
		return ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
	}

	LPADRBOOK addr_book = NULL;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenAddressBook(0,
						    i_id,
						    flags,
						    &addr_book);
	}
	MAPIFreeBuffer(i_id);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppAdrBook = create_object<BRUTUS_IAddrBook_i, ::BRUTUS::IAddrBook, LPADRBOOK>
			(addr_book, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenAddressBook()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenProfileSection(const char *lpUID,
							    const char *lpInterface,
							    ::BRUTUS::BDEFINE ulFlags,
							    ::BRUTUS::IProfSect_out lppProfSect
							    ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenProfileSection()");

	lppProfSect = ::BRUTUS::IProfSect::_nil();

	MAPIUID u_id;
	mapiuid_brutus_to_mapi_no_alloc(lpUID, u_id);


	GUID *i_id;
	if (!guid_brutus_to_mapi(lpInterface, NULL, i_id)) {
		BRUTUS_LOG_SUB_BUG("Could not convert ::BRUTUS::GUID into MAPI GUID");
		return ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
	}

	unsigned long flags = native_flags(ulFlags);
	LPPROFSECT prof_sect = NULL;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenProfileSection(&u_id,
						       i_id,
						       flags,
						       &prof_sect);
	}
	MAPIFreeBuffer(i_id);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppProfSect = create_object<BRUTUS_IProfSect_i, ::BRUTUS::IProfSect, LPPROFSECT>
			(prof_sect, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenProfileSection()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetStatusTable(::BRUTUS::BDEFINE ulFlags,
							::BRUTUS::IMAPITable_out lppTable
							ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetStatusTable()");

	lppTable = ::BRUTUS::IMAPITable::_nil();

	LPMAPITABLE mapi_table = NULL;
	unsigned long flags = native_flags(ulFlags);

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetStatusTable(flags,
						   &mapi_table);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppTable = create_object<BRUTUS_IMAPITable_i, ::BRUTUS::IMAPITable, LPMAPITABLE>
			(mapi_table, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetStatusTable()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenEntry(const ::BRUTUS::ENTRYID &lpEntryID,
						   const char *lpInterface,
						   ::BRUTUS::BDEFINE ulFlags,
						   ::BRUTUS::BDEFINE_out lpulObjType,
						   ::BRUTUS::IUnknown_out lppUnk
						   ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenEntry()");

	lppUnk = ::BRUTUS::IUnknown::_nil();
	::BRUTUS::IUnknown_var obj = ::BRUTUS::IUnknown::_nil();

	unsigned long flags = native_flags(ulFlags);

	GUID *i_id;
	if (!guid_brutus_to_mapi(lpInterface, NULL, i_id)) {
		BRUTUS_LOG_SUB_BUG("Could not convert ::BRUTUS::GUID into MAPI GUID");
		return ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
	}

	unsigned long obj_type;
	LPUNKNOWN unk_object = NULL;
	ENTRYID *entry_id = NULL;

	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&lpEntryID, 0, entry_id_size, entry_id)) {
		MAPIFreeBuffer(i_id);
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenEntry(entry_id_size,
					      entry_id,
					      i_id,
					      flags,
					      &obj_type,
					      &unk_object);
	}
	MAPIFreeBuffer(entry_id);
	MAPIFreeBuffer(i_id);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("BRUTUS_IMAPISession_i::OpenEntry() failed");
		BRUTUS_SUB_EXIT;
	}

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		if (object_type_mapi_to_brutus(obj_type, lpulObjType)) {
			if (!create_brutus_object(obj_type, unk_object, poa_.in(), obj.out(), mapi_session_)) {
				if (unk_object)
					unk_object->Release();
				BRUTUS_LOG_SUB_BUG("Could not create brutus object");
				br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
				BRUTUS_SUB_EXIT;
			}
		} else {
			if (!create_brutus_object(lpInterface, unk_object, poa_, obj.out(), mapi_session_)) {
				if (unk_object)
					unk_object->Release();
				BRUTUS_LOG_SUB_BUG("Could not create brutus object");
				br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
				BRUTUS_SUB_EXIT;
			}
		}
	}
	lppUnk = obj._retn();

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenEntry()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::CompareEntryIDs(const ::BRUTUS::ENTRYID &lpEntryID1,
							 const ::BRUTUS::ENTRYID &lpEntryID2,
							 ::BRUTUS::BDEFINE ulFlags,
							 CORBA::Boolean_out lpulResult
							 ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::CompareEntryIDs()");

	ENTRYID *entry_id_1;
	ENTRYID *entry_id_2;

	unsigned long result = 0;
	unsigned long flags = native_flags(ulFlags);

	unsigned long entry_id_1_size;
	if (!entryid_brutus_to_mapi(&lpEntryID1, 0, entry_id_1_size, entry_id_1)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	unsigned long entry_id_2_size;
	if (!entryid_brutus_to_mapi(&lpEntryID2, 0, entry_id_2_size, entry_id_2)) {
		MAPIFreeBuffer(entry_id_1);
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->CompareEntryIDs(entry_id_1_size,
						    entry_id_1,
						    entry_id_2_size,
						    entry_id_2,
						    flags,
						    &result);
	}
	MAPIFreeBuffer(entry_id_1);
	MAPIFreeBuffer(entry_id_2);

	if (result == TRUE)
		lpulResult = (::CORBA::Boolean)1;
	else
		lpulResult = (::CORBA::Boolean)0;

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::CompareEntryIDs()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::Advise(const ::BRUTUS::ENTRYID &lpEntryID,
						::BRUTUS::BDEFINE ulEventMask,
						::BRUTUS::IMAPIAdviseSink_ptr lpAdviseSink,
						CORBA::ULong_out lpulConnection
						ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::Advise()");

	unsigned long event_mask = native_eventmask(ulEventMask);

	lpulConnection = (CORBA::ULong)0;

	HRESULT hr = S_OK;
	::BRUTUS::BRESULT br;
	if (CORBA::is_nil(lpAdviseSink)) {
		BRUTUS_LOG_SUB_BUG("lpAdviseSink is NIL");
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		BRUTUS_SUB_EXIT;
	}

	ENTRYID *entry_id;
	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&lpEntryID, 0, entry_id_size, entry_id)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	CMAPIAdviseSink *sink;
	try {
		sink = new CMAPIAdviseSink(lpAdviseSink, mapi_session_, poa_.in());
	}
	catch (std::bad_alloc &) {
		MAPIFreeBuffer(entry_id);
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		unsigned long adv_con; // the compiler complains otherwise..
		hr = mapi_session_->Advise(entry_id_size,
					   entry_id,
					   event_mask,
					   sink,
					   &adv_con);

		lpulConnection = (CORBA::ULong)adv_con; // see above..
	}
	MAPIFreeBuffer(entry_id);
	sink->Release(); // MAPI::UnAdvise will call Release()

	if (S_OK == hr)
		advise_sinks_.AddSink(lpulConnection);

	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::Advise()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::Unadvise(CORBA::ULong ulConnection
						  ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::Unadvise()");

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->Unadvise(ulConnection);
	}
	advise_sinks_.RemoveSink(ulConnection);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::Unadvise()");
	return br;
}


::BRUTUS::BRESULT BRUTUS_IMAPISession_i::QueryDefaultMessageOpt(const char * lpszAdrType,
								::BRUTUS::BDEFINE ulFlags,
								::BRUTUS::seq_SPropValue_out lppOptions
								ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::QueryDefaultMessageOpt()");

	::BRUTUS::seq_SPropValue_var out_values;
	try {
		out_values = new ::BRUTUS::seq_SPropValue;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	out_values->length(0);

	unsigned long count = 0;
	LPSPropValue values = NULL;
	unsigned long flags = native_flags(ulFlags);

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->QueryDefaultMessageOpt((char*)lpszAdrType,
							   flags,
							   &count,
							   &values);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (count && (S_OK == hr))
		spropvalue_array_mapi_to_brutus(count, values, out_values, mapi_session_, true, poa_);
	else if (values)
		MAPIFreeBuffer(values);

	lppOptions = out_values._retn();

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::QueryDefaultMessageOpt()");
	return br;
}



::BRUTUS::BRESULT BRUTUS_IMAPISession_i::EnumAdrTypes(::BRUTUS::BDEFINE ulFlags,
						      ::BRUTUS::seq_string_out lpppszAdrTypes
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::EnumAdrTypes()");

	::BRUTUS::seq_string_var out_adr_types;
	try {
		out_adr_types = new ::BRUTUS::seq_string;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	out_adr_types->length(0);

	unsigned long count = 0;
	unsigned long flags = native_flags(ulFlags);
	LPTSTR *adr_types = NULL;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->EnumAdrTypes(flags,
						 &count,
						 &adr_types);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		MAPIFreeBuffer(adr_types);
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (count && (S_OK == hr)) {
		out_adr_types->length(count);
		for (CORBA::ULong i = 0; i < count; i++)
			out_adr_types[i] = CORBA::string_dup(adr_types[i]);
	}
	MAPIFreeBuffer(adr_types);
	lpppszAdrTypes = out_adr_types._retn();

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::EnumAdrTypes()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::QueryIdentity(::BRUTUS::ENTRYID_out lppEntryID
						       ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::QueryIdentity()");

	unsigned long count = 0;
	ENTRYID *entry_id = NULL;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->QueryIdentity(&count,
						  &entry_id);
	}

	::BRUTUS::ENTRYID_var out_entry_id;
	if (!entryid_mapi_to_brutus(count, entry_id, out_entry_id.out(), true)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	lppEntryID = out_entry_id._retn();

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::QueryIdentity()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::SetDefaultStore(::BRUTUS::BDEFINE ulFlags,
							 const ::BRUTUS::ENTRYID &lpEntryID
							 ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::SetDefaultStore()");

	ENTRYID *entry_id = NULL;
	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&lpEntryID, 0, entry_id_size, entry_id)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	unsigned long flags = native_message_store_flags(ulFlags);

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->SetDefaultStore(flags, entry_id_size, entry_id);
	}
	MAPIFreeBuffer(entry_id);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::SetDefaultStore()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::AdminServices(::BRUTUS::BDEFINE ulFlags,
						       ::BRUTUS::IMsgServiceAdmin_out lppServiceAdmin
						       ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))

{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::AdminServices()");

	lppServiceAdmin = ::BRUTUS::IMsgServiceAdmin::_nil();

	LPSERVICEADMIN mapi_service_admin = NULL;
	unsigned long flags = (unsigned long)ulFlags;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->AdminServices(flags,
						  &mapi_service_admin);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr ) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppServiceAdmin = create_object<BRUTUS_IMsgServiceAdmin_i, ::BRUTUS::IMsgServiceAdmin, LPSERVICEADMIN>
			(mapi_service_admin, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::AdminServices()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::QueryInterface(const char *iid,
							::BRUTUS::IUnknown_out ppvObject
							ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	ppvObject = ::BRUTUS::IUnknown::_nil();

	HRESULT hr = S_OK;
	::BRUTUS::BRESULT br;
	void *object = NULL;

	GUID mapi_iid;
	if (!guid_brutus_to_mapi_no_alloc(iid, &mapi_iid)) {
		BRUTUS_LOG_SUB_BUG("Could not convert ::BRUTUS::GUID into MAPI GUID");
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		BRUTUS_EXIT;
	}

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr =  mapi_session_->QueryInterface(mapi_iid, &object);
	}

	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		if (!create_brutus_object(iid, (LPUNKNOWN)object, poa_, ppvObject, mapi_session_)) {
			BRUTUS_LOG_SUB_BUG("Could not create brutus object.");
			((LPUNKNOWN)object)->Release();
			br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
			BRUTUS_EXIT;
		}
	}

exit:
	return br;
}

void BRUTUS_IMAPISession_i::Destroy(CORBA::ULong InstanceID
				    ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::Destroy()");

	ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(life_check_mutex_);

		this->remove_life_check(InstanceID);

		ref_count_--;
		if (0 < ref_count_)
			goto exit;
	}
	BRUTUS_LOG_SUB_INF("Continuing in BRUTUS_IMAPISession_i::Destroy()");

	// stop notification thread
	if (stop_)
		SetEvent(stop_);

	// stop dead client supervisor thread
	if (client_dead_handle_)
		SetEvent(client_dead_handle_);

	HRESULT hr = S_OK; // needed by logging macro
	char *profile = NULL;
	unsigned long n = 0;

	// should never happen
	if (!mapi_session_) {
		BRUTUS_LOG_SUB_CRITICAL("Thread synchronization bug");
		goto exit;
	}

	for (n = 0; n < advise_sinks_.length(); n++)
		mapi_session_->Unadvise(advise_sinks_[n]);

	advise_sinks_.Clear();

	mapi_session_->Logoff(0, 0, 0);
	mapi_session_->Release();
	mapi_session_ = NULL;

	if (id_) { // there might not be a registry if this is a proxy
		profile = registry_->get_profile_name(id_);
		registry_->sign_out(id_, 0);
		id_ = 0;
	}

	if (profile) {
		const char *const buf = " has logged out";
		char *msg = (char *)malloc(sizeof(char)*(strlen(buf) + strlen(profile) + 1));
		if (msg) {
			*msg = '\0';
			strcat(msg, profile);
			strcat(msg, buf);
			BRUTUS_LOG_SUB_PRIVATE(msg);
			free(msg);
		} else
			BRUTUS_LOG_SUB_ERR("No memory");

		free(profile);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::Destroy()");
}



////////////////////////////////////////////////////////////
//                                                        //
// The following functions are Brutus specific management //
// functions. They are *not* part of IMAPISession proper. //
//                                                        //
////////////////////////////////////////////////////////////

CORBA::ULong BRUTUS_IMAPISession_i::addLifeline(::BRUTUS::BrutusCheck_ptr LifeLine
						ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((::CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::addLifeline()");

	ACE_Write_Guard<ACE_RW_Mutex> guard(life_check_mutex_);

	life_lines_.push_back(new LifeCheck(LifeLine, life_line_counter_));
	ref_count_++;

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::addLifeline()");

	return life_line_counter_;
}

void BRUTUS_IMAPISession_i::ping(ACE_ENV_SINGLE_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
}

void BRUTUS_IMAPISession_i::checkLifelines(void)
{
	unsigned long n = 0;
	std::vector<CORBA::ULong> dead_ones;

	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(life_check_mutex_);

		std::vector<LifeCheck*>::iterator lit = life_lines_.begin();

		for (n = 0; n < life_lines_.size(); n++, lit++) {
			try {
				life_lines_.at(n)->life_line->ping();
			}
			catch (...) {
				dead_ones.push_back(life_lines_.at(n)->id);
				goto destroy;
			}
		}
	}

	return;

destroy:
	for (n = 0; n < dead_ones.size(); n++) {
		BRUTUS_LOG_BUG("BRUTUS_IMAPISession_i::checkLifelines() - Destroying dead session");
		this->Destroy(dead_ones[n]);
	}
}


///////////////////////////////////////////////////////////
//                                                       //
// The following functions are generic helper functions  //
// typically found in the Exchange SDK. They are *not*   //
// a part of IMAPISession proper.                        //
//                                                       //
///////////////////////////////////////////////////////////






::BRUTUS::BRESULT BRUTUS_IMAPISession_i::CreateIProp(::BRUTUS::IPropData_out lppPropData
						     ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::CreateIProp()");

	lppPropData = ::BRUTUS::IPropData::_nil();
	LPPROPDATA prop_data = NULL;

	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = ::CreateIProp(&IID_IMAPIPropData,
				   MAPIAllocateBuffer,
				   MAPIAllocateMore,
				   MAPIFreeBuffer,
				   0,
				   &prop_data);
	}
	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppPropData = create_object<BRUTUS_IPropData_i, ::BRUTUS::IPropData, LPPROPDATA>
			(prop_data, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::CreateIProp()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenTnefStreamEx(::BRUTUS::IStream_ptr lpStreamName,
							  const char * lpszStreamName,
							  ::BRUTUS::BDEFINE ulFlags,
							  ::BRUTUS::IMessage_ptr lpMessage,
							  CORBA::Short wKeyVal,
							  ::BRUTUS::IAddrBook_ptr lpAdressBook,
							  ::BRUTUS::ITnef_out lppTNEF
							  ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenTnefStreamEx()");

	lppTNEF = ::BRUTUS::ITnef::_nil();
	CORBA::ULongLong p = (CORBA::ULongLong)NULL;
	LPADRBOOK addr_book = NULL;
	LPMESSAGE message = NULL;

	CStream *brutus_stream;
	try {
		brutus_stream = new CStream(lpStreamName);
	}
	catch (...) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	::BRUTUS::BDEFINE brutus_flags = ulFlags;
	ULONG flags = 0;
	if (brutus_flags & ::BRUTUS::BRUTUS_TNEF_BEST_DATA) {
		flags |= TNEF_BEST_DATA;
		FLAGS_OFF(::BRUTUS::BDEFINE, brutus_flags, ::BRUTUS::BRUTUS_TNEF_BEST_DATA);
	}
	if (brutus_flags & ::BRUTUS::BRUTUS_TNEF_COMPATIBILITY) {
		flags |= TNEF_COMPATIBILITY;
		FLAGS_OFF(::BRUTUS::BDEFINE, brutus_flags, ::BRUTUS::BRUTUS_TNEF_COMPATIBILITY);
	}
	if (brutus_flags & ::BRUTUS::BRUTUS_TNEF_DECODE) {
		flags |= TNEF_DECODE;
		FLAGS_OFF(::BRUTUS::BDEFINE, brutus_flags, ::BRUTUS::BRUTUS_TNEF_DECODE);
	}
	if (brutus_flags & ::BRUTUS::BRUTUS_TNEF_ENCODE) {
		flags |= TNEF_ENCODE;
		FLAGS_OFF(::BRUTUS::BDEFINE, brutus_flags, ::BRUTUS::BRUTUS_TNEF_ENCODE);
	}
	if (brutus_flags & ::BRUTUS::BRUTUS_TNEF_PURE) {
		flags |= TNEF_PURE;
		FLAGS_OFF(::BRUTUS::BDEFINE, brutus_flags, ::BRUTUS::BRUTUS_TNEF_PURE);
	}
	if (brutus_flags) {
		char msg[128] = {0};
		sprintf(msg, "Unknown flag(s) in BRUTUS_IMAPISession_i::OpenTnefStreamEx() : %X", brutus_flags);
		BRUTUS_LOG_SUB_BUG(msg);
	}

	HRESULT hr = S_OK;
	::BRUTUS::BRESULT br;
	if (!CORBA::is_nil(lpMessage)) {
		if (::BRUTUS::BRUTUS_S_OK != lpMessage->GetLocalPointer(p)) {
			BRUTUS_LOG_SUB_ERR("Could not retrive local pointer from IMessage");
			brutus_stream->Release();
			br = ::BRUTUS::BRUTUS_UNKNOWN_ERROR;
			BRUTUS_SUB_EXIT;
		}
	}
	message = (LPMESSAGE)p;
	if (!valid_mapi_interface_pointer<IMessage>(message)) {
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		message = NULL;
		BRUTUS_SUB_EXIT;
	}

	p = (CORBA::ULongLong)NULL;

	if (!CORBA::is_nil(lpAdressBook)) {
		if (::BRUTUS::BRUTUS_S_OK != lpAdressBook->GetLocalPointer(p)) {
			BRUTUS_LOG_SUB_ERR("Could not retrive local pointer from IAddrBook");
			brutus_stream->Release();
			message->Release();
			br = ::BRUTUS::BRUTUS_UNKNOWN_ERROR;
			BRUTUS_SUB_EXIT;
		}
	}
	addr_book = (LPADRBOOK)p;
	if (!valid_mapi_interface_pointer<IAddrBook>(addr_book)) {
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		addr_book = NULL;
		BRUTUS_SUB_EXIT;
	}
	p = (CORBA::ULongLong)NULL;

	LPITNEF tnef;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = ::OpenTnefStreamEx(0,
					brutus_stream,
					(char*)lpszStreamName,
					flags,
					message,
					wKeyVal,
					addr_book,
					&tnef);
	}
	brutus_stream->Release();
	message->Release();
	addr_book->Release();

	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppTNEF = create_object<BRUTUS_ITnef_i, ::BRUTUS::ITnef, LPITNEF>
			(tnef, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenTnefStreamEx()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::MAPIAdminProfiles(::BRUTUS::BDEFINE ulFlags,
							   ::BRUTUS::IProfAdmin_out ProfAdmin
							   ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::MAPIAdminProfiles()");

	ProfAdmin = ::BRUTUS::IProfAdmin::_nil();

	// Get an IProfAdmin interface.
	LPPROFADMIN lpProfAdmin = NULL;
	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = ::MAPIAdminProfiles(0, &lpProfAdmin);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}
	if (S_OK == hr) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		ProfAdmin = create_object<BRUTUS_IProfAdmin_i, ::BRUTUS::IProfAdmin, LPPROFADMIN>
			(lpProfAdmin, poa_.in(), mapi_session_);
	}

exit:
	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::MAPIAdminProfiles()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetDefaultMsgStore(::BRUTUS::BDEFINE ulFlags,
							    ::BRUTUS::IMsgStore_out lpMDB
							    ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetDefaultMsgStore()");

	lpMDB = ::BRUTUS::IMsgStore::_nil();

	ULONG flags = 0;
	ULONG cRows = 0;
	LPSRowSet lpRows = NULL;
	LPENTRYID entry_id = NULL;
	ULONG entry_id_size = 0;
	ULONG i;
	LPMDB msg_store = NULL;

	// Columns to return from the table
	SizedSPropTagArray(2, prop_tags) = {2, {PR_DEFAULT_STORE, PR_ENTRYID} };

	// Get list of available message stores
	HRESULT hr;
	LPMAPITABLE mapi_table = NULL;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetMsgStoresTable(0, &mapi_table);
	}
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Get row count
	hr = mapi_table->GetRowCount(0, &cRows);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;
	if (!cRows) {
		hr = MAPI_E_NOT_FOUND;
		BRUTUS_SUB_EXIT;
	}

	// Select columns to return from the table
	hr = mapi_table->SetColumns((LPSPropTagArray)&prop_tags, 0);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Go to the beginning of the recipient table for the envelope
	hr = mapi_table->SeekRow(BOOKMARK_BEGINNING, 0, NULL);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Read all the rows of the table
	hr = mapi_table->QueryRows(cRows, 0, &lpRows);
	if ((S_OK == hr) && (lpRows != NULL) && (lpRows->cRows == 0)) {
		FreeProws(lpRows);
		hr = MAPI_E_NOT_FOUND;
		BRUTUS_SUB_EXIT;
	}
	mapi_table->Release();
	mapi_table = NULL;

	for (i = 0; i < cRows; i++) {
		// check PR_DEFAULT_STORE boolean
		if (lpRows->aRow[i].lpProps[0].Value.b == TRUE) {
			entry_id = (LPENTRYID)lpRows->aRow[i].lpProps[1].Value.bin.lpb;
			entry_id_size = lpRows->aRow[i].lpProps[1].Value.bin.cb;
			break;
		}
	}

	flags = native_flags(ulFlags);
	flags |= MDB_NO_DIALOG;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenMsgStore(0,
						 entry_id_size,
						 entry_id,
						 NULL,
						 flags,
						 &msg_store);
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lpMDB = create_object<BRUTUS_IMsgStore_i, ::BRUTUS::IMsgStore, LPMDB>
			(msg_store, poa_.in(), mapi_session_);
	}

exit:
	if (mapi_table)
		mapi_table->Release();

	if (lpRows)
		FreeProws(lpRows);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetDefaultMsgStore()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetPublicMsgStore(::BRUTUS::BDEFINE ulFlags,
							   ::BRUTUS::IMsgStore_out lpMDB
							   ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetPublicMsgStore()");

	lpMDB = ::BRUTUS::IMsgStore::_nil();

	ULONG flags = 0;
	ULONG cRows = 0;
	LPSRowSet lpRows = NULL;
	LPENTRYID entry_id = NULL;
	ULONG entry_id_size = 0;
	ULONG i;
	LPMDB msg_store = NULL;

	// Columns to return from the table
	SizedSPropTagArray(2, prop_tags) = {2, {PR_MDB_PROVIDER, PR_ENTRYID} };

	// Get list of available message stores
	HRESULT hr;
	LPMAPITABLE mapi_table = NULL;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetMsgStoresTable(0, &mapi_table);
	}
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Get row count
	hr = mapi_table->GetRowCount(0, &cRows);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;
	if (!cRows) {
		hr = MAPI_E_NOT_FOUND;
		BRUTUS_SUB_EXIT;
	}

	// Select columns to return from the table
	hr = mapi_table->SetColumns((LPSPropTagArray)&prop_tags, 0);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Go to the beginning of the recipient table for the envelope
	hr = mapi_table->SeekRow(BOOKMARK_BEGINNING, 0, NULL);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Read all the rows of the table
	hr = mapi_table->QueryRows(cRows, 0, &lpRows);
	if ((S_OK == hr) && (lpRows != NULL) && (lpRows->cRows == 0)) {
		FreeProws(lpRows);
		hr = MAPI_E_NOT_FOUND;
		BRUTUS_SUB_EXIT;
	}
	mapi_table->Release();
	mapi_table = NULL;

	for (i = 0; i < cRows; i++) {
		if (!memcmp(lpRows->aRow[i].lpProps[0].Value.bin.lpb, pbExchangeProviderPublicGuid, sizeof(MAPIUID))) {
			entry_id = (LPENTRYID)lpRows->aRow[i].lpProps[1].Value.bin.lpb;
			entry_id_size = lpRows->aRow[i].lpProps[1].Value.bin.cb;
			break;
		}
	}

	flags = native_flags(ulFlags);
	flags |= MDB_NO_DIALOG;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenMsgStore(0,
						 entry_id_size,
						 entry_id,
						 NULL,
						 flags,
						 &msg_store);
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lpMDB = create_object<BRUTUS_IMsgStore_i, ::BRUTUS::IMsgStore, LPMDB>
			(msg_store, poa_.in(), mapi_session_);
	}

exit:
	if (mapi_table)
		mapi_table->Release();

	if (lpRows)
		FreeProws(lpRows);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetPublicMsgStore()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::OpenStoreFromGuid(const char *lpGUID,
							   ::BRUTUS::BDEFINE ulFlags,
							   ::BRUTUS::IMsgStore_out lpMDB
							   ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::OpenStoreFromGuid()");

	lpMDB = ::BRUTUS::IMsgStore::_nil();

	HRESULT hr = NOERROR;
	ULONG flags = 0;
	LPMAPITABLE mapi_table = NULL;
	LPMDB mapi_store = NULL;
	LPSRowSet lpRows = NULL;
	BYTE mapi_lpbyte[sizeof(GUID)];

	SPropTagArray aptEntryID = {1, {PR_ENTRYID} };
	SPropValue SProp = {PR_MDB_PROVIDER, 0L, {0} };

	SRestriction SRestrict;
	SRestrict.rt = RES_PROPERTY;
	SRestrict.res.resProperty.relop = RELOP_EQ;
	SRestrict.res.resProperty.ulPropTag = PR_MDB_PROVIDER;
	SRestrict.res.resProperty.lpProp = &SProp;

	guid_brutus_to_lpbyte_no_alloc(lpGUID, mapi_lpbyte);

	// Get table of message stores
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->GetMsgStoresTable(0, &mapi_table);
	}
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	// Restrict the table to the appropriate Exchange store
	SProp.Value.bin.cb  = sizeof(GUID);
	SProp.Value.bin.lpb = mapi_lpbyte;

	hr = HrQueryAllRows(mapi_table, &aptEntryID, &SRestrict, NULL, 0, &lpRows);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	if (!lpRows->cRows) {
		hr = MAPI_E_NOT_FOUND;
		BRUTUS_SUB_EXIT;
	}
	if ((lpRows->aRow->cValues != 1) || (lpRows->aRow->lpProps[0].ulPropTag != PR_ENTRYID)) {
		hr = E_FAIL;
		BRUTUS_SUB_EXIT;
	}

	// Open the message store
	flags = native_flags(ulFlags);
	flags |= MDB_NO_DIALOG;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenMsgStore((ULONG)0,
						 lpRows->aRow->lpProps[0].Value.bin.cb,
						 (LPENTRYID)lpRows->aRow->lpProps[0].Value.bin.lpb,
						 NULL,
						 flags,
						 &mapi_store);
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lpMDB = create_object<BRUTUS_IMsgStore_i, ::BRUTUS::IMsgStore, LPMDB>
			(mapi_store, poa_.in(), mapi_session_);
	} else
		BRUTUS_SUB_EXIT;


exit:
	if (mapi_table)
		mapi_table->Release();

	if (lpRows)
		FreeProws(lpRows);

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::OpenStoreFromGuid()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::MailboxLogon(::BRUTUS::IMsgStore_ptr lpMDB,
						      const char * lpszMsgStoreDN,
						      const char * lpszMailboxDN,
						      ::BRUTUS::BDEFINE ulCreateFlags,
						      ::BRUTUS::BDEFINE ulOpenFlags,
						      ::BRUTUS::IMsgStore_out lppMailboxMDB
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::MailboxLogon()");

	lppMailboxMDB = ::BRUTUS::IMsgStore::_nil();

	HRESULT hr = S_OK;
	ULONG flags = 0;
	ULONG mapi_entry_id_count = 0;
	ENTRYID *mapi_entry_id = NULL;
	LPMDB input_message_store = NULL;
	LPMDB mailbox_store = NULL;
	LPEXCHANGEMANAGESTORE mapi_exchange_manage_store = NULL;
	CORBA::ULongLong p = (CORBA::ULongLong)NULL;

	// sanity checks
	if (CORBA::is_nil(lpMDB) || !strlen(lpszMsgStoreDN)) {
		hr = MAPI_E_INVALID_PARAMETER;
		BRUTUS_SUB_EXIT;
	}

	::BRUTUS::BRESULT br;
	if (!CORBA::is_nil(lpMDB))
		if (::BRUTUS::BRUTUS_S_OK != lpMDB->GetLocalPointer(p)) {
			BRUTUS_LOG_SUB_ERR("Could not retrive local pointer from IMsgStore");
			br = ::BRUTUS::BRUTUS_UNKNOWN_ERROR;
			BRUTUS_SUB_EXIT;
		}
	input_message_store = (LPMDB)p;
	if (!valid_mapi_interface_pointer<IMsgStore>(input_message_store)) {
		br = ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
		input_message_store = NULL;
		BRUTUS_SUB_EXIT;
	}

	hr = input_message_store->QueryInterface(IID_IExchangeManageStore,
						 (LPVOID*) &mapi_exchange_manage_store);
	if (S_OK != hr)
		BRUTUS_SUB_EXIT;

	flags = native_flags(ulCreateFlags);
	hr = mapi_exchange_manage_store->CreateStoreEntryID((char*)lpszMsgStoreDN,
							    (char*)lpszMailboxDN,
							    flags,
							    &mapi_entry_id_count,
							    &mapi_entry_id);
	if (!mapi_entry_id || (S_OK != hr))
		BRUTUS_SUB_EXIT;

	flags = native_flags(ulOpenFlags);
	flags |= MDB_NO_DIALOG;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenMsgStore(0,
						 mapi_entry_id_count,
						 mapi_entry_id,
						 NULL,
						 flags,
						 &mailbox_store);
	}
	if ((S_OK == hr) || (MAPI_W_ERRORS_RETURNED == hr)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		lppMailboxMDB = create_object<BRUTUS_IMsgStore_i, ::BRUTUS::IMsgStore, LPMDB>
			(mailbox_store, poa_.in(), mapi_session_);
	} else if (MAPI_E_FAILONEPROVIDER == hr) { // required behavior by MSDN (http://support.microsoft.com/?kbid=260141)
		hr = MAPI_E_CALL_FAILED;
		BRUTUS_SUB_EXIT;
	} else
		BRUTUS_SUB_EXIT;

exit:
	if (mapi_entry_id)
		MAPIFreeBuffer(mapi_entry_id);

	if (mapi_exchange_manage_store)
		mapi_exchange_manage_store->Release();

	if (input_message_store)
		input_message_store->Release();

	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::MailboxLogon()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::GetOutlookFolder(::BRUTUS::OUTLOOK_FOLDER Which,
							  ::BRUTUS::IMsgStore_ptr lpMDB,
							  ::BRUTUS::IMAPIFolder_out Folder
							  ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetOutlookFolder()");

	// initiate out value asap
	Folder = ::BRUTUS::IMAPIFolder::_nil();

	HRESULT hr;
	::BRUTUS::BRESULT br;
	ULONG obj_type;
	ULONG prop_tag = 0;
	LPUNKNOWN unk = NULL;
	LPMAPIFOLDER root_folder = NULL;
	LPMAPIFOLDER outlook_folder = NULL;
	LPMDB msg_store = NULL;
	LPSPropValue pv = NULL;
	CORBA::ULongLong p = (CORBA::ULongLong)NULL;

	// get message store
	if (CORBA::is_nil(lpMDB)) {
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = get_default_message_store(mapi_session_, &msg_store);
		if ((S_OK != hr) && (MAPI_W_ERRORS_RETURNED != hr))
			BRUTUS_SUB_EXIT;
	} else {
		if (::BRUTUS::BRUTUS_S_OK != lpMDB->GetLocalPointer(p)) {
			BRUTUS_LOG_SUB_ERR("Could not retrive local pointer from IMsgStore");
			return ::BRUTUS::BRUTUS_UNKNOWN_ERROR;
		}
		msg_store = (LPMDB)p;
		if (!valid_mapi_interface_pointer<IMsgStore>(msg_store))
			return ::BRUTUS::BRUTUS_MAPI_E_INVALID_PARAMETER;
	}

	// crate property tag
	switch (Which) {
	case ::BRUTUS::OUTLOOK_OUTBOX :
		prop_tag = PT_BINARY | (0x35E2<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_WASTEBASKET :
		prop_tag = PT_BINARY | (0x35E3<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_SENTMAIL :
		prop_tag = PT_BINARY | (0x35E4<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_VIEWS :
		prop_tag = PT_BINARY | (0x35E5<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_COMMON_VIEWS :
		prop_tag = PT_BINARY | (0x35E6<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_FINDER :
		prop_tag = PT_BINARY | (0x35E7<<16); // on private message store
		break;
	case ::BRUTUS::OUTLOOK_CALENDAR :
		prop_tag = PT_BINARY | (0x36D0<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_CONTACTS :
		prop_tag = PT_BINARY | (0x36D1<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_JOURNAL :
		prop_tag = PT_BINARY | (0x36D2<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_NOTES :
		prop_tag = PT_BINARY | (0x36D3<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_TASKS :
		prop_tag = PT_BINARY | (0x36D4<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_REMINDERS :
		prop_tag = PT_BINARY | (0x36D5<<16); // on root folder in private message store
		break;
	case ::BRUTUS::OUTLOOK_DRAFTS :
		prop_tag = PT_BINARY | (0x36D7<<16); // on root folder in private message store
		break;
	default :
		hr = MAPI_E_INVALID_PARAMETER;
		BRUTUS_SUB_EXIT;
	}

	// open outlook folder
	switch (Which) {
	case ::BRUTUS::OUTLOOK_OUTBOX :
	case ::BRUTUS::OUTLOOK_WASTEBASKET :
	case ::BRUTUS::OUTLOOK_SENTMAIL :
	case ::BRUTUS::OUTLOOK_VIEWS :
	case ::BRUTUS::OUTLOOK_COMMON_VIEWS :
	case ::BRUTUS::OUTLOOK_FINDER :
	{
		hr = HrGetOneProp(msg_store, prop_tag, &pv);
		if (S_OK != hr)
			BRUTUS_SUB_EXIT;

		hr = msg_store->OpenEntry(pv->Value.bin.cb, (LPENTRYID)pv->Value.bin.lpb, NULL, MAPI_BEST_ACCESS, &obj_type, &unk);
		if (S_OK != hr)
			BRUTUS_SUB_EXIT;
		outlook_folder = (LPMAPIFOLDER)unk;
	}
	break;
	case ::BRUTUS::OUTLOOK_CALENDAR :
	case ::BRUTUS::OUTLOOK_CONTACTS :
	case ::BRUTUS::OUTLOOK_JOURNAL :
	case ::BRUTUS::OUTLOOK_NOTES :
	case ::BRUTUS::OUTLOOK_TASKS :
	case ::BRUTUS::OUTLOOK_REMINDERS :
	case ::BRUTUS::OUTLOOK_DRAFTS :
	{
		hr = msg_store->OpenEntry(0, 0, NULL, MAPI_BEST_ACCESS, &obj_type, &unk);
		if (S_OK != hr)
			BRUTUS_SUB_EXIT;
		root_folder = (LPMAPIFOLDER)unk;

		hr = HrGetOneProp(root_folder, prop_tag, &pv);
		if (S_OK != hr)
			BRUTUS_SUB_EXIT;

		hr = root_folder->OpenEntry(pv->Value.bin.cb, (LPENTRYID)pv->Value.bin.lpb, NULL, MAPI_BEST_ACCESS, &obj_type, &unk);
		if (S_OK != hr)
			BRUTUS_SUB_EXIT;
		outlook_folder = (LPMAPIFOLDER)unk;
	}
	break;
	default :
		hr = MAPI_E_INVALID_PARAMETER;
		BRUTUS_SUB_EXIT;
	}

	// convert to Brutus object
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		Folder = create_object<BRUTUS_IMAPIFolder_i, ::BRUTUS::IMAPIFolder, LPMAPIFOLDER>
			(outlook_folder, poa_.in(), mapi_session_);
	}

exit:
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	if (pv)
		MAPIFreeBuffer(pv);

	if (root_folder)
		root_folder->Release();

	if (msg_store)
		msg_store->Release();

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetOutlookFolder()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::DispatchNotifications(::CORBA::ULong Flags
							       ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::DispatchNotifications()");

	HRESULT hr;
	{ // No multithread protection!
		hr = HrDispatchNotifications((unsigned long)Flags);
	}

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
		BRUTUS_SUB_EXIT;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::DispatchNotifications()");
exit:
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::QueryProps(const ::BRUTUS::ENTRYID & Eid,
						    const ::BRUTUS::SPropTagArray & PropTagArray,
						    ::BRUTUS::seq_SPropValue_out PropValueArray
						    ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((::CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::QueryProps()");

	ENTRYID *entry_id = NULL;
	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&Eid, 0, entry_id_size, entry_id)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	BRUTUS::seq_SPropValue_var brutus_props;
	try {
		brutus_props = new BRUTUS::seq_SPropValue;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	brutus_props->length(0);

	LPMAPIPROP prop_object = NULL;
	unsigned long obj_type;
	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenEntry(entry_id_size,
					      entry_id,
					      &IID_IMAPIProp,
					      0, // only read access is needed
					      &obj_type,
					      (LPUNKNOWN*)&prop_object);
	}
	MAPIFreeBuffer(entry_id);

	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("IMAPISession::OpenEntry() failed");
		BRUTUS_SUB_EXIT;
	}

	SPropTagArray *tags = NULL;
	if (!proptag_array_brutus_to_mapi(&PropTagArray, 0, tags)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		prop_object->Release();
		throw CORBA::NO_MEMORY();
	}

	SPropValue *props = NULL;
	unsigned long count = 0;
	hr = prop_object->GetProps(tags, 0, &count, &props);
	MAPIFreeBuffer(tags);
	prop_object->Release();
	prop_object = NULL;

	if ((MAPI_W_ERRORS_RETURNED == hr) || (S_OK == hr))
		spropvalue_array_mapi_to_brutus(count, props, brutus_props, mapi_session_, true, poa_);
	else
		MAPIFreeBuffer(props);

exit:
	PropValueArray = brutus_props._retn();

	if (prop_object)
		prop_object->Release();

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::QueryProps()");
	return br;
}

::BRUTUS::BRESULT BRUTUS_IMAPISession_i::QueryMessage(const ::BRUTUS::ENTRYID & Eid,
						      const ::BRUTUS::SPropTagArray & ObjPropTagArray,
						      const ::BRUTUS::SPropTagArray & AdditionalObjPropTagArray,
						      const ::BRUTUS::SPropTagArray & AttachPropTagArray,
						      const ::BRUTUS::SPropTagArray & RecpPropTagArray,
						      ::BRUTUS::seq_SPropValue_out ObjPropValueArray,
						      ::BRUTUS::seq_SPropValue_out AdditionalObjPropValueArray,
						      ::BRUTUS::SRowSet_out AttachRows,
						      ::BRUTUS::SRowSet_out RecpRows
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((::CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::QueryMessage()");

	ENTRYID *entry_id = NULL;
	unsigned long entry_id_size;
	if (!entryid_brutus_to_mapi(&Eid, 0, entry_id_size, entry_id)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}

	// initiate out values
	BRUTUS::seq_SPropValue_var brutus_obj_props;
	try {
		brutus_obj_props = new BRUTUS::seq_SPropValue;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	brutus_obj_props->length(0);
	BRUTUS::seq_SPropValue_var brutus_additional_obj_props;
	try {
		brutus_additional_obj_props = new BRUTUS::seq_SPropValue;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	brutus_additional_obj_props->length(0);
	BRUTUS::SRowSet_var brutus_attach_rows;
	try {
		brutus_attach_rows = new BRUTUS::SRowSet;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	brutus_attach_rows->length(0);
	BRUTUS::SRowSet_var brutus_recp_rows;
	try {
		brutus_recp_rows = new BRUTUS::SRowSet;
	}
	catch (std::bad_alloc &) {
		BRUTUS_LOG_SUB_ERR("No memory");
		throw CORBA::NO_MEMORY();
	}
	brutus_recp_rows->length(0);

	// early declarations
	LPMAPITABLE mapi_table = NULL;
	LPSRowSet mapi_rowset = NULL;
	SPropValue *props = NULL;
	unsigned long count = 0;

	// open message object
	LPMESSAGE msg_object = NULL;
	unsigned long obj_type;
	HRESULT hr;
	{
		ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);
		hr = mapi_session_->OpenEntry(entry_id_size,
					      entry_id,
					      &IID_IMessage,
					      0, // only read access is needed
					      &obj_type,
					      (LPUNKNOWN*)&msg_object);
	}
	MAPIFreeBuffer(entry_id);
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("IMAPISession::OpenEntry() failed");
		BRUTUS_SUB_EXIT;
	}

	/*
	 * AttachmentTable
	 */
	SPropTagArray *tags = NULL;
	if (!proptag_array_brutus_to_mapi(&AttachPropTagArray, 0, tags)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		msg_object->Release();
		throw CORBA::NO_MEMORY();
	}

	hr = msg_object->GetAttachmentTable(0, &mapi_table);
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("IMessage::GetAttachmentTable() failed");
		BRUTUS_SUB_EXIT;
	}
	hr = HrQueryAllRows(mapi_table,
			    tags,
			    NULL,
			    NULL,
			    0,
			    &mapi_rowset);
	mapi_table->Release();
	mapi_table = NULL;
	MAPIFreeBuffer(tags);
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("HrQueryAllRows() failed");
		BRUTUS_SUB_EXIT;
	}
	if (!srow_set_mapi_to_brutus(mapi_rowset, brutus_attach_rows, mapi_session_, true, poa_)) {
		mapi_rowset = NULL;
		BRUTUS_LOG_SUB_BUG("Conversion error");
		BRUTUS_SUB_EXIT;
	}

	/*
	 * RecipientTable
	 */
	if (!proptag_array_brutus_to_mapi(&RecpPropTagArray, 0, tags)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		msg_object->Release();
		throw CORBA::NO_MEMORY();
	}

	hr = msg_object->GetRecipientTable(0, &mapi_table);
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("IMessage::GetRecipientTable() failed");
		BRUTUS_SUB_EXIT;
	}
	hr = HrQueryAllRows(mapi_table,
			    tags,
			    NULL,
			    NULL,
			    0,
			    &mapi_rowset);
	mapi_table->Release();
	mapi_table = NULL;
	MAPIFreeBuffer(tags);
	if (S_OK != hr) {
		BRUTUS_LOG_SUB_ERR("HrQueryAllRows() failed");
		BRUTUS_SUB_EXIT;
	}
	if (!srow_set_mapi_to_brutus(mapi_rowset, brutus_recp_rows, mapi_session_, true, poa_)) {
		mapi_rowset = NULL;
		BRUTUS_LOG_SUB_BUG("Conversion error");
		BRUTUS_SUB_EXIT;
	}

	/*
	 * Additional object properties
	 */
	if (AdditionalObjPropTagArray.length()) {
		if (!proptag_array_brutus_to_mapi(&AdditionalObjPropTagArray, 0, tags)) {
			BRUTUS_LOG_SUB_ERR("No memory");
			msg_object->Release();
			throw CORBA::NO_MEMORY();
		}

		hr = msg_object->GetProps(tags, 0, &count, &props);
		MAPIFreeBuffer(tags);
		if ((MAPI_W_ERRORS_RETURNED == hr) || (S_OK == hr)) {
			if (!spropvalue_array_mapi_to_brutus(count, props, brutus_additional_obj_props, mapi_session_, true, poa_)) {
				BRUTUS_LOG_SUB_BUG("Conversion error");
				BRUTUS_SUB_EXIT;
			}
		} else {
			MAPIFreeBuffer(props);
			BRUTUS_LOG_SUB_ERR("IMessage::GetProps() failed");
			BRUTUS_SUB_EXIT;
		}
	}

	/*
	 * Object properties
	 */
	if (!proptag_array_brutus_to_mapi(&ObjPropTagArray, 0, tags)) {
		BRUTUS_LOG_SUB_ERR("No memory");
		msg_object->Release();
		throw CORBA::NO_MEMORY();
	}

	hr = msg_object->GetProps(tags, 0, &count, &props);
	MAPIFreeBuffer(tags);
	if ((MAPI_W_ERRORS_RETURNED == hr) || (S_OK == hr)) {
		if (!spropvalue_array_mapi_to_brutus(count, props, brutus_obj_props, mapi_session_, true, poa_))
			BRUTUS_LOG_SUB_BUG("Conversion error");
	} else {
		MAPIFreeBuffer(props);
		BRUTUS_LOG_SUB_ERR("IMessage::GetProps() failed");
	}

exit:
	ObjPropValueArray = brutus_obj_props._retn();
	AdditionalObjPropValueArray = brutus_additional_obj_props._retn();
	AttachRows = brutus_attach_rows._retn();
	RecpRows = brutus_recp_rows._retn();

	MAPIFreeBuffer(mapi_rowset);

	if (mapi_table)
		mapi_table->Release();
	if (msg_object)
		msg_object->Release();

	::BRUTUS::BRESULT br;
	if (!hresult_to_bresult(hr, br)) {
		BRUTUS_LOG_SUB_BUG("Could not convert HRESULT into BRESULT");
		br = ::BRUTUS::BRUTUS_INTERNAL_ERROR;
	}

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::QueryMessage()");
	return br;
}

static inline LPWSTR asci_to_unicode(LPCSTR str)
{
	LPWSTR wstr = NULL;

	DWORD len = lstrlen(str);
	if (len > 0) {
		// get buffer size and allocate utf8 string buffer
		int wlen = MultiByteToWideChar(CP_ACP, 0, str, -1, NULL, 0);
		wstr = (LPWSTR)calloc(wlen, sizeof(wchar_t));
		if (!wstr)
			goto out;

		// convert to UTF8
		if (!MultiByteToWideChar(CP_UTF8, 0, str, -1, wstr, wlen)) {
			free(wstr);
			wstr = NULL;
			goto out;
		}
	}

out:
	return wstr;
}

::CORBA::Boolean BRUTUS_IMAPISession_i::GetServerTime(const char *server,
						      ::BRUTUS::TIME_OF_DAY_INFO_out tod
						      ACE_ENV_ARG_DECL)
	ACE_THROW_SPEC((::CORBA::SystemException))
{
	BRUTUS_LOG_REF;
	BRUTUS_LOG_SUB_INF("Entering BRUTUS_IMAPISession_i::GetServerTime()");

	::CORBA::Boolean retv = false;
	LPTSTR server_name = NULL;
	TIME_OF_DAY_INFO *win32_tod = NULL;

	if (strlen(server))
		server_name = (char*)server;

	LPWSTR w_server_name = asci_to_unicode(server_name);
	if (NERR_Success == NetRemoteTOD(w_server_name, (LPBYTE*) &win32_tod))
		retv = true;

	if (retv) {
		if (win32_tod) {
			tod.tod_elapsedt = win32_tod->tod_elapsedt;
			tod.tod_msecs = win32_tod->tod_msecs ;
			tod.tod_hours = win32_tod->tod_hours;
			tod.tod_mins = win32_tod->tod_mins ;
			tod.tod_secs = win32_tod->tod_secs;
			tod.tod_hunds = win32_tod->tod_hunds;
			tod.tod_timezone = win32_tod->tod_timezone;
			tod.tod_tinterval = win32_tod->tod_tinterval;
			tod.tod_day = win32_tod->tod_day;
			tod.tod_month = win32_tod->tod_month;
			tod.tod_year = win32_tod->tod_year;
			tod.tod_weekday = win32_tod->tod_weekday;
		} else
			retv = false;
	}

	if (w_server_name)
		free(w_server_name);

	if (win32_tod)
		NetApiBufferFree(win32_tod);

	BRUTUS_LOG_SUB_INF("Leaving BRUTUS_IMAPISession_i::GetServerTime()");
	return retv;
}
