/*
 *    Brutus header file for the IMAPISession servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __IMAPISESSIONS_IMPL_H_
#define __IMAPISESSIONS_IMPL_H_

#include <vector>

#include <idl/IMAPISessionS.h>
#include <idl/BrutusLogonC.h>
//#include <idl/BrutusCheckC.h>
#include <NT_Service/macros.h>
#include <NT_Service/advise_utils.h>
#include <NT_Service/poa_utils.h>
#include <NT_Service/servant_impl/MAPIFunctionsS_impl.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>


#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

//Class BRUTUS_IMAPISession_i
class SERVANTLIB_API BRUTUS_IMAPISession_i : public virtual POA_BRUTUS::IMAPISession,
                                             public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_IMAPISession_i(LPMAPISESSION MAPISession,
                              CORBA::ULong MaxNotifyDelay,
                              PortableServer::POA_ptr Poa,
                              SessionRegistry *Registry,
                              unsigned long RegistryID,
                              HANDLE &Stop,
                              ::BRUTUS::BrutusCheck_ptr LifeLine,
                              const unsigned long LifeCheckDelay);

        virtual ~BRUTUS_IMAPISession_i(void)
                {
                        if (stop_)
                                SetEvent(stop_);

                        if (client_dead_handle_)
                                SetEvent(client_dead_handle_);

                        try {
                                // The POA guaranties that we only get here after all
                                // threads have stopped accessing the object. There is
                                // therefore no need to use any locking.
                                if (mapi_session_) {
                                        for (unsigned long n = 0; n < advise_sinks_.length(); n++)
                                                mapi_session_->Unadvise(advise_sinks_[n]);

                                        advise_sinks_.Clear();

                                        mapi_session_->Logoff(0, 0, 0); // Investigate flag!!

                                        mapi_session_->Release();
                                }

                                if (id_) // there might not be a registry if this is a proxy
                                        registry_->sign_out(id_, 0);
                        }
                        catch (...) {
                        }
                };

        virtual ::BRUTUS::BRESULT GetLastError(::BRUTUS::BRESULT ReturnCode,
                                               ::BRUTUS::BDEFINE ulFlags,
                                               ::BRUTUS::MAPIERROR_out lppMAPIError
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetMsgStoresTable(::BRUTUS::BDEFINE ulFlags,
                                                    ::BRUTUS::IMAPITable_out lppTable
                                                    ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenMsgStore(const ::BRUTUS::ENTRYID &lpEntryID,
                                               const char *lpInterface,
                                               ::BRUTUS::BDEFINE ulFlags,
                                               ::BRUTUS::IMsgStore_out lppMDB
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenAddressBook(const char *lpInterface,
                                                  ::BRUTUS::BDEFINE ulFlags,
                                                  ::BRUTUS::IAddrBook_out lppAdrBook
                                                  ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenProfileSection(const char *lpUID,
                                                     const char *lpInterface,
                                                     ::BRUTUS::BDEFINE ulFlags,
                                                     ::BRUTUS::IProfSect_out lppProfSect
                                                     ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetStatusTable(::BRUTUS::BDEFINE ulFlags,
                                                 ::BRUTUS::IMAPITable_out lppTable
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenEntry(const ::BRUTUS::ENTRYID &lpEntryID,
                                            const char *lpInterface,
                                            ::BRUTUS::BDEFINE ulFlags,
                                            ::BRUTUS::BDEFINE_out lpulObjType,
                                            ::BRUTUS::IUnknown_out lppUnk
                                            ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT CompareEntryIDs(const ::BRUTUS::ENTRYID &lpEntryID1,
                                                  const ::BRUTUS::ENTRYID &lpEntryID2,
                                                  ::BRUTUS::BDEFINE ulFlags,
                                                  CORBA::Boolean_out lpulResult
                                                  ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Advise(const ::BRUTUS::ENTRYID &lpEntryID,
                                         ::BRUTUS::BDEFINE ulEventMask,
                                         ::BRUTUS::IMAPIAdviseSink_ptr lpAdviseSink,
                                         CORBA::ULong_out lpulConnection
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Unadvise(CORBA::ULong ulConnection
                                           ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryDefaultMessageOpt(const char *lpszAdrType,
                                                         ::BRUTUS::BDEFINE ulFlags,
                                                         ::BRUTUS::seq_SPropValue_out lppOptions
                                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT EnumAdrTypes(::BRUTUS::BDEFINE ulFlags,
                                               ::BRUTUS::seq_string_out lpppszAdrTypes
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryIdentity(::BRUTUS::ENTRYID_out lppEntryID
                                                ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT SetDefaultStore(::BRUTUS::BDEFINE ulFlags,
                                                  const ::BRUTUS::ENTRYID &lpEntryID
                                                  ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT AdminServices(::BRUTUS::BDEFINE ulFlags,
                                                ::BRUTUS::IMsgServiceAdmin_out lppServiceAdmin
                                                ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong addLifeline(::BRUTUS::BrutusCheck_ptr LifeLine
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((::CORBA::SystemException));

        virtual void ping(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT CreateIProp(::BRUTUS::IPropData_out lppPropData
                                              ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenTnefStreamEx(::BRUTUS::IStream_ptr lpStreamName,
                                                   const char *lpszStreamName,
                                                   ::BRUTUS::BDEFINE ulFlags,
                                                   ::BRUTUS::IMessage_ptr lpMessage,
                                                   ::CORBA::Short wKeyVal,
                                                   ::BRUTUS::IAddrBook_ptr lpAdressBook,
                                                   ::BRUTUS::ITnef_out lppTNEF
                                                   ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT MAPIAdminProfiles(::BRUTUS::BDEFINE ulFlags,
                                                    ::BRUTUS::IProfAdmin_out ProfAdmin
                                                    ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetDefaultMsgStore(::BRUTUS::BDEFINE ulFlags,
                                                     ::BRUTUS::IMsgStore_out lpMDB
                                                     ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetPublicMsgStore(::BRUTUS::BDEFINE ulFlags,
                                                    ::BRUTUS::IMsgStore_out lpMDB
                                                    ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenStoreFromGuid(const char *lpGUID,
                                                    ::BRUTUS::BDEFINE ulFlags,
                                                    ::BRUTUS::IMsgStore_out lpMDB
                                                    ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT MailboxLogon(::BRUTUS::IMsgStore_ptr lpMDB,
                                               const char *lpszMsgStoreDN,
                                               const char *lpszMailboxDN,
                                               ::BRUTUS::BDEFINE ulCreateFlags,
                                               ::BRUTUS::BDEFINE ulOpenFlags,
                                               ::BRUTUS::IMsgStore_out lppMailboxMDB
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetOutlookFolder(::BRUTUS::OUTLOOK_FOLDER Which,
                                                   ::BRUTUS::IMsgStore_ptr lpMDB,
                                                   ::BRUTUS::IMAPIFolder_out Folder
                                                   ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));


        virtual ::BRUTUS::BRESULT DispatchNotifications(::CORBA::ULong Flags
                                                        ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryProps(const ::BRUTUS::ENTRYID & Eid,
                                             const ::BRUTUS::SPropTagArray & PropTagArray,
                                             ::BRUTUS::seq_SPropValue_out PropValueArray
                                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((::CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryMessage(const ::BRUTUS::ENTRYID & Eid,
                                               const ::BRUTUS::SPropTagArray & ObjPropTagArray,
                                               const ::BRUTUS::SPropTagArray & AdditionalObjPropTagArray,
                                               const ::BRUTUS::SPropTagArray & AttachPropTagArray,
                                               const ::BRUTUS::SPropTagArray & RecpPropTagArray,
                                               ::BRUTUS::seq_SPropValue_out ObjPropValueArray,
                                               ::BRUTUS::seq_SPropValue_out AdditionalObjPropValueArray,
                                               ::BRUTUS::SRowSet_out AttachRows,
                                               ::BRUTUS::SRowSet_out RecpRows
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((::CORBA::SystemException));

        virtual ::CORBA::Boolean GetServerTime(const char *server,
                                               ::BRUTUS::TIME_OF_DAY_INFO_out tod
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((::CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 ::BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        void checkLifelines(void);

private:
        BRUTUS_IMAPISession_i(void)
                { };

        class LifeCheck {
        public:
                LifeCheck(BRUTUS::BrutusCheck_ptr Checker,
                          CORBA::ULong & CheckerID)
                        : life_line(::BRUTUS::BrutusCheck::_duplicate(Checker)),
                          id(CheckerID++)
                        {
                        };

                ~LifeCheck(void)
                        {
                                CORBA::release(life_line);
                        };

                BRUTUS::BrutusCheck_ptr life_line;
                CORBA::ULong id;
        };

        void remove_life_check(const CORBA::ULong Id)
                {
                        std::vector<LifeCheck*>::iterator lit = life_lines_.begin();

                        unsigned long n = 0;
                        for (n = 0; n < life_lines_.size(); n++, lit++) {
                                if (Id == life_lines_.at(n)->id) {
                                        delete life_lines_.at(n);
                                        life_lines_.erase(lit);
                                        break;
                                }
                        }
                };

        LPMAPISESSION mapi_session_;
        unsigned long notify_delay_;
        PortableServer::POA_var poa_;
        SessionRegistry *registry_;
        unsigned long id_;
        unsigned long ref_count_;
        HANDLE stop_;

        CORBA::ULong life_line_counter_;
        std::vector<LifeCheck*> life_lines_;
        HANDLE client_dead_handle_;

        Sinks advise_sinks_;
        ACE_RW_Mutex mutex_;
        ACE_RW_Mutex life_check_mutex_;
};


#endif /* __IMAPISESSIONS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
