/*
 *    Brutus header file for the IProviderAdmin servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __IPROVIDERADMINS_IMPL_H_
#define __IPROVIDERADMINS_IMPL_H_

#include <idl/IProviderAdminS.h>

#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */


class SERVANTLIB_API BRUTUS_IProviderAdmin_i : public virtual POA_BRUTUS::IProviderAdmin,
                                               public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_IProviderAdmin_i(LPPROVIDERADMIN ProviderAdmin,
                                LPMAPISESSION MAPISession,
                                PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::BRESULT GetLastError(BRUTUS::BRESULT ReturnCode,
                                               CORBA::ULong ulFlags,
                                               BRUTUS::MAPIERROR_out lppMAPIError
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT GetProviderTable(CORBA::ULong ulFlags,
                                                   BRUTUS::IMAPITable_out lppTable
                                                   ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT CreateProvider(const char * lpszProvider,
                                                 const BRUTUS::seq_SPropValue & lpProps,
                                                 CORBA::ULong ulFlags,
                                                 CORBA::String_out lpUID
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT DeleteProvider(char *& lpUID
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT OpenProfileSection(const char *lpUID,
                                                     const char *lpInterface,
                                                     CORBA::ULong ulFlags,
                                                     BRUTUS::IProfSect_out lppProfSect
                                                     ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_IProviderAdmin_i (void)
                { };

        virtual ~BRUTUS_IProviderAdmin_i (void)
                { 
                        try {
                                provider_admin_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        LPPROVIDERADMIN provider_admin_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __IPROVIDERADMINS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
