/*
 *    Brutus header file for the IStream servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __ISTREAMS_IMPL_H_
#define __ISTREAMS_IMPL_H_

#include <idl/IStreamS.h>
#include <NT_Service/macros.h>

#include <objidl.h>
#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

//Class BRUTUS_IStream_i
class SERVANTLIB_API BRUTUS_IStream_i : public virtual POA_BRUTUS::IStream
{
public:

        BRUTUS_IStream_i(LPSTREAM Stream,
                         LPMAPISESSION MAPISession,
                         PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::BRESULT Clone(BRUTUS::IStream_out ppstm
                                        ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Commit(BRUTUS::BDEFINE grfCommitFlags
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT CopyTo(BRUTUS::IStream_ptr pstm,
                                         CORBA::ULongLong cb,
                                         CORBA::ULongLong_out pcbRead,
                                         CORBA::ULongLong_out pcbWritten
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT LockRegion(CORBA::ULongLong libOffset,
                                             CORBA::ULongLong cb,
                                             BRUTUS::BDEFINE dwLockType
                                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Revert(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Seek(CORBA::LongLong dlibMove,
                                       BRUTUS::STREAM_SEEK dwOrigin,
                                       CORBA::ULongLong_out plibNewPosition
                                       ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT SetSize(CORBA::ULongLong libNewSize
                                          ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Stat(BRUTUS::STATSTG_out pstatstg,
                                       BRUTUS::STATFLAG grfStatFlag
                                       ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT UnlockRegion(CORBA::ULongLong libOffset,
                                               CORBA::ULongLong cb,
                                               BRUTUS::BDEFINE dwLockType
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Read(BRUTUS::seq_octet_out pv,
                                       CORBA::ULong cb,
                                       CORBA::ULong_out pcbRead
                                       ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT Write(const BRUTUS::seq_octet & pv,
                                        CORBA::ULong cb,
                                        CORBA::ULong_out pcbWritten
                                        ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        // Skeleton constructor 
        BRUTUS_IStream_i(void)
                { };
        
        virtual ~BRUTUS_IStream_i(void)
                { 
                        try {
                                istream_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };


        LPSTREAM istream_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __ISTREAMS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
