/*
 *    Brutus header file for the ITableData servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __ITABLEDATAS_IMPL_H_
#define __ITABLEDATAS_IMPL_H_

#include <idl/ITableDataS.h>

#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapiutil.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_ITableData_i : public virtual POA_BRUTUS::ITableData,
                                           public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_ITableData_i(LPTABLEDATA TableData,
                            LPMAPISESSION MAPISession,
                            PortableServer::POA_ptr Poa);
  
        virtual ::BRUTUS::BRESULT HrGetView(const BRUTUS::SSortOrderSet & lpSSortOrderSet,
                                            BRUTUS::IMAPITable_out lppMAPITable
                                            ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrModifyRow(const BRUTUS::SRow & lpSRow
                                              ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrDeleteRow(const BRUTUS::SPropValue & lpSPropValue
                                              ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrQueryRow(const BRUTUS::SPropValue & lpSPropValue,
                                             BRUTUS::SRow_out lppSRow,
                                             CORBA::ULong & lpuliRow
                                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrEnumRow(CORBA::ULong ulRowNumber,
                                            BRUTUS::SRow_out lppSRow
                                            ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrNotify(BRUTUS::BDEFINE ulFlags,
                                           const BRUTUS::seq_SPropValue & lpSPropValue
                                           ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrInsertRow(CORBA::ULong uliRow,
                                              const BRUTUS::SRow & lpSRow
                                              ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrModifyRows(BRUTUS::BDEFINE ulFlags,
                                               const BRUTUS::SRowSet & lpSRowSet
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT HrDeleteRows(BRUTUS::BDEFINE ulFlags,
                                               const BRUTUS::SRowSet & lprowsetToDelete,
                                               CORBA::ULong_out cRowsDeleted
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_ITableData_i(void)
                { };
  
        virtual ~BRUTUS_ITableData_i(void)
                {
                        try {
                                table_data_->Release();
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        LPTABLEDATA table_data_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


#endif /* __ITABLEDATAS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
