/*
 *    Brutus header file for the ITnef servant.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __ITNEFS_IMPL_H_
#define __ITNEFS_IMPL_H_

#include <vector>

#include <idl/ITnefS.h>
#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>
#include <tnef.h>

// to hold pointers and interfaces that must be valid until
// ITnef::Finish() has been called
struct MemResource {
        void *mem; // blob
        bool com_obj; // COM object?
};

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_ITnef_i : public virtual POA_BRUTUS::ITnef,
                                      public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_ITnef_i(LPITNEF Tnef,
                       LPMAPISESSION MAPISession,
                       PortableServer::POA_ptr Poa);
  
        virtual ::BRUTUS::BRESULT AddProps(BRUTUS::BDEFINE ulFlags,
                                           CORBA::ULong ulElemID,
                                           const BRUTUS::seq_octet & lpvData,
                                           BRUTUS::IStream_ptr stream,
                                           const BRUTUS::SPropTagArray & lpPropList
                                           ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT ExtractProps(BRUTUS::BDEFINE ulFlags,
                                               const BRUTUS::SPropTagArray & lpPropList,
                                               BRUTUS::STnefProblemArray_out lpProblems
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT Finish(BRUTUS::BDEFINE ulFlags,
                                         CORBA::UShort_out lpKey,
                                         BRUTUS::STnefProblemArray_out lpProblems
                                         ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT OpenTaggedBody(BRUTUS::IMessage_ptr lpMessage,
                                                 BRUTUS::BDEFINE ulFlags,
                                                 BRUTUS::IStream_out lppStream
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT SetProps(BRUTUS::BDEFINE ulFlags,
                                           CORBA::ULong ulElemID,
                                           const BRUTUS::SPropValue & lpProps
                                           ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT EncodeRecips(BRUTUS::BDEFINE ulFlags,
                                               BRUTUS::IMAPITable_ptr lpRecipientTable
                                               ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT FinishComponent(BRUTUS::BDEFINE ulFlags,
                                                  CORBA::ULong ulComponentID,
                                                  const BRUTUS::SPropTagArray & lpCustomPropList,
                                                  const BRUTUS::SPropValue & lpCustomProps,
                                                  const BRUTUS::SPropTagArray & lpPropList,
                                                  BRUTUS::STnefProblemArray_out lpProblems
                                                  ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));
  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        // Skeleton constructor 
        BRUTUS_ITnef_i(void)
                { };

        
        virtual ~BRUTUS_ITnef_i(void)
                { 
                        try {
                                i_tnef_->Release();
                                for (unsigned long n=0; n<mems.size(); n++) {
                                        if (!mems[n]->com_obj) {
                                                MAPIFreeBuffer(mems[n]->mem);
                                        } else {
                                                ((LPUNKNOWN)(mems[n]->mem))->Release();
                                        }
                                        delete mems[n];
                                }
                                mems.clear();

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };


        LPITNEF i_tnef_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        // vector of pointers to free when finish() is called
        std::vector<struct MemResource*> mems;

        ACE_RW_Mutex mutex_;
};


#endif /* __ITNEFS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
