/*
 *    Brutus source file implementing the LogServer servant.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <NT_Service/servant_impl/log_serverS_impl.h>
#include <NT_Service/defines.h>

static inline const char *log_severity_to_str(const BRUTUS::LogServer::log_severity Severity)
{
        switch (Severity) {
        case BRUTUS::LogServer::log_header :
                return "*** LEADING LOG ENTRY ***";
        case BRUTUS::LogServer::log_bug :
                return "BUG";
        case BRUTUS::LogServer::log_critical :
                return "CRITICAL";
        case BRUTUS::LogServer::log_err :
                return "ERROR";
        case BRUTUS::LogServer::log_inf :
                return "INFORMATION";
        case BRUTUS::LogServer::log_private :
                return "PRIVATE";
        default :
                return "UNKNOWN";
        }
}


BRUTUS_LogServer_i::BRUTUS_LogServer_i(void)
{ 
        entry_count_ = 0;
        time(&cur_time_);
        gmt_ = gmtime(&cur_time_);

        log_file_.open(BRUTUS_LOG, ios::in | ios::out | ios::trunc);
        log_file_.flush(); 
        log_file_.close();
}

void BRUTUS_LogServer_i::log(const ::BRUTUS::LogServer::LogEntry & Entry
                             ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);                

        const char *severity = log_severity_to_str(Entry.severity);

        time(&cur_time_);
        gmt_ = gmtime(&cur_time_);

        log_file_.open(BRUTUS_LOG, ios::in | ios::out | ios::app);
        log_file_ << "#" << entry_count_++ << "\n"
                  << "Severity   : " << severity                    << "\n"
                  << "UTC Time   : " << asctime(gmt_) 
                  << "Mark       : " << (const char*)Entry.mark     << "\n"
                  << "Host       : " << (const char*)Entry.hostname << "\n"
                  << "Process ID : " << Entry.process_id            << "\n"
                  << "File       : " << (const char*)Entry.file     << "\n"
                  << "Line       : " << Entry.line                  << "\n"
                  << "Message    : " << (const char*)Entry.message  << "\n"
                  << "\n";

        log_file_.flush();
        log_file_.close();
}

CORBA::ULong BRUTUS_LogServer_i::log_leader(const ::BRUTUS::LogServer::LogEntry & Entry
                                            ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);                

        const char *severity = log_severity_to_str(Entry.severity);
        
        time(&cur_time_);
        gmt_ = gmtime(&cur_time_);

        log_file_.open(BRUTUS_LOG, ios::in | ios::out | ios::app);
        log_file_ << "#" << entry_count_++ << "\n"
                  << "Severity   : " << severity                    << "\n"
                  << "UTC Time   : " << asctime(gmt_) 
                  << "Mark       : " << (const char*)Entry.mark     << "\n"
                  << "Host       : " << (const char*)Entry.hostname << "\n"
                  << "Process ID : " << Entry.process_id            << "\n"
                  << "File       : " << (const char*)Entry.file     << "\n"
                  << "Line       : " << Entry.line                  << "\n"
                  << "Message    : " << (const char*)Entry.message  << "\n"
                  << "\n";

        log_file_.flush();
        log_file_.close();

        return (entry_count_ - 1);

}

void BRUTUS_LogServer_i::log_follower(::CORBA::ULong leader,
                                      const ::BRUTUS::LogServer::LogEntry & Entry
                                      ACE_ENV_ARG_DECL)
        ACE_THROW_SPEC((CORBA::SystemException))
{
        ACE_Write_Guard<ACE_RW_Mutex> guard(mutex_);                

        const char *severity = log_severity_to_str(Entry.severity);

        time(&cur_time_);
        gmt_ = gmtime(&cur_time_);

        log_file_.open(BRUTUS_LOG, ios::in | ios::out | ios::app);
        log_file_ << "#" << entry_count_++ << "\n"
                  << "Severity   : " << severity                    << "\n"
                  << "UTC Time   : " << asctime(gmt_) 
                  << "Mark       : " << (const char*)Entry.mark     << "\n"
                  << "Host       : " << (const char*)Entry.hostname << "\n"
                  << "Process ID : " << Entry.process_id            << "\n"
                  << "File       : " << (const char*)Entry.file     << "\n"
                  << "Line       : " << Entry.line                  << "\n"
                  << "Message    : " << (const char*)Entry.message  << "\n"
                  << "Following  : " << leader << "\n"
                  << "\n";

        log_file_.flush();
        log_file_.close();
}

