/*
 *    Brutus source file for the IUnknown and SRestriction servants.
 *    Copyright (C) 2004 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */


#ifndef __TYPESS_IMPL_H_
#define __TYPESS_IMPL_H_

#include <idl/typesS.h>

#include <NT_Service/macros.h>

#define MAPIGUID_H // do not include mapiguid.h
#include <mapix.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
#pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

class SERVANTLIB_API BRUTUS_IUnknown_i : public virtual POA_BRUTUS::IUnknown,
                                         public virtual PortableServer::RefCountServantBase

{
public:

        BRUTUS_IUnknown_i(LPUNKNOWN Unknown,
                          LPMAPISESSION MAPISession,
                          PortableServer::POA_ptr Poa);
  
        virtual ::BRUTUS::BRESULT QueryInterface(const char *iid,
                                                 BRUTUS::IUnknown_out ppvObject
                                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(CORBA::ULong InstanceID
                             ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC ((CORBA::SystemException));

private:
        // Skeleton constructor 
        BRUTUS_IUnknown_i(void)
                { };

        virtual ~BRUTUS_IUnknown_i(void)
                {
                        try {
                                unknown_->Release();                        
                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        LPUNKNOWN unknown_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;

        ACE_RW_Mutex mutex_;
};


class SERVANTLIB_API BRUTUS_SRestrictionContainer_i : public virtual POA_BRUTUS::SRestrictionContainer,
                                                      public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SRestrictionContainer_i(::SRestriction *Res,
                                       LPMAPISESSION MAPISession,
                                       PortableServer::POA_ptr Poa,
                                       PortableServer::ServantManager_ptr Activator);

        virtual ::BRUTUS::SRestriction * content(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void Destroy(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SRestrictionContainer_i(void)
                { };
  
        virtual ~BRUTUS_SRestrictionContainer_i(void)
                { 
                        MAPIFreeBuffer((void*)res_);
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };
  
        ::SRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
        PortableServer::ServantManager_var activator_;
};


//Class BRUTUS_SAndRestriction_i
class SERVANTLIB_API BRUTUS_SAndRestriction_i : public virtual POA_BRUTUS::SAndRestriction,
                                                public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SAndRestriction_i(::SAndRestriction *Res,
                                 LPMAPISESSION MAPISession,
                                 PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::seq_SRestriction * lpRes(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SAndRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SAndRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SAndRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};

//Class BRUTUS_SOrRestriction_i
class SERVANTLIB_API BRUTUS_SOrRestriction_i : public virtual POA_BRUTUS::SOrRestriction,
                                               public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SOrRestriction_i(::SOrRestriction *Res,
                                LPMAPISESSION MAPISession,
                                PortableServer::POA_ptr Poa);

        virtual BRUTUS::seq_SRestriction * lpRes(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SOrRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SOrRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SOrRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SNotRestriction_i
class SERVANTLIB_API BRUTUS_SNotRestriction_i : public virtual POA_BRUTUS::SNotRestriction,
                                                public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SNotRestriction_i(::SNotRestriction *Res,
                                 LPMAPISESSION MAPISession,
                                 PortableServer::POA_ptr Poa);

        virtual CORBA::ULong ulReserved(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void ulReserved(CORBA::ULong ulReserved
                                ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::SRestriction * lpRes(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));


private:
        BRUTUS_SNotRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SNotRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SNotRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SContentRestriction_i
class SERVANTLIB_API BRUTUS_SContentRestriction_i : public virtual POA_BRUTUS::SContentRestriction,
                                                    public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SContentRestriction_i(::SContentRestriction *Res,
                                     LPMAPISESSION MAPISession,
                                     PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::FUZZY_TYPE ulFuzzyLevel(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::SPropValue * lpProp(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SContentRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SContentRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SContentRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SPropertyRestriction_i
class SERVANTLIB_API BRUTUS_SPropertyRestriction_i : public virtual POA_BRUTUS::SPropertyRestriction,
                                                     public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SPropertyRestriction_i(::SPropertyRestriction *Res,
                                      LPMAPISESSION MAPISession,
                                      PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::RELOP_TYPE relop(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::SPropValue * lpProp(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SPropertyRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SPropertyRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SPropertyRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SComparePropsRestriction_i
class SERVANTLIB_API BRUTUS_SComparePropsRestriction_i : public virtual POA_BRUTUS::SComparePropsRestriction,
                                                         public virtual PortableServer::RefCountServantBase

{
public:
        BRUTUS_SComparePropsRestriction_i(::SComparePropsRestriction *Res,
                                          LPMAPISESSION MAPISession,
                                          PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::RELOP_TYPE relop(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag1(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag2(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SComparePropsRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SComparePropsRestriction_i (void)
                { };

        ::SComparePropsRestriction *res_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SBitMaskRestriction_i
class SERVANTLIB_API BRUTUS_SBitMaskRestriction_i : public virtual POA_BRUTUS::SBitMaskRestriction,
                                                    public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_SBitMaskRestriction_i(::SBitMaskRestriction *Res,
                                     LPMAPISESSION MAPISession,
                                     PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::BMR_TYPE relBMR(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulMask(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SBitMaskRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SBitMaskRestriction_i (void)
                { };

        ::SBitMaskRestriction *res_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SSizeRestriction_i
class SERVANTLIB_API BRUTUS_SSizeRestriction_i : public virtual POA_BRUTUS::SSizeRestriction,
                                                 public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_SSizeRestriction_i(::SSizeRestriction *Res,
                                  LPMAPISESSION MAPISession,
                                  PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::RELOP_TYPE relop(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong cb(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SSizeRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SSizeRestriction_i (void)
                { };

        ::SSizeRestriction *res_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SExistRestriction_i
class SERVANTLIB_API BRUTUS_SExistRestriction_i : public virtual POA_BRUTUS::SExistRestriction,
                                                  public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_SExistRestriction_i(::SExistRestriction *Res,
                                   LPMAPISESSION MAPISession,
                                   PortableServer::POA_ptr Poa);

        virtual CORBA::ULong ulReserved1(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void ulReserved1(CORBA::ULong ulReserved1
                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulPropTag(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual CORBA::ULong ulReserved2(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual void ulReserved2(CORBA::ULong ulReserved2
                                 ACE_ENV_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SExistRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SExistRestriction_i (void)
                { };

        ::SExistRestriction *res_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SSubRestriction_i
class SERVANTLIB_API BRUTUS_SSubRestriction_i : public virtual POA_BRUTUS::SSubRestriction,
                                                public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_SSubRestriction_i(::SSubRestriction *Res,
                                 LPMAPISESSION MAPISession,
                                 PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::BDEFINE ulSubObject(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::SRestriction * lpRes(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        BRUTUS_SSubRestriction_i (void)
                { };
  
        virtual ~BRUTUS_SSubRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SSubRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};


//Class BRUTUS_SCommentRestriction_i
class SERVANTLIB_API BRUTUS_SCommentRestriction_i : public virtual POA_BRUTUS::SCommentRestriction,
                                                    public virtual PortableServer::RefCountServantBase
{
public:
        BRUTUS_SCommentRestriction_i(::SCommentRestriction *Res,
                                     LPMAPISESSION MAPISession,
                                     PortableServer::POA_ptr Poa);

        virtual ::BRUTUS::SRestriction * lpRes(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

        virtual ::BRUTUS::seq_SPropValue * lpProp(ACE_ENV_SINGLE_ARG_DECL)
                ACE_THROW_SPEC((CORBA::SystemException));

private:
        //Constructor 
        BRUTUS_SCommentRestriction_i (void)
                { };
  
        //Destructor 
        virtual ~BRUTUS_SCommentRestriction_i (void)
                { 
                        try {

                                mapi_session_->Release();
                        }
                        catch (...) {
                                BRUTUS_LOG_ERR("Exception caught");
                        }
                };

        ::SCommentRestriction *res_;
        LPMAPISESSION mapi_session_;
        PortableServer::POA_var poa_;
};



#endif /* __TYPESS_IMPL_H_  */

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
