/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Authors: Jules Colding <colding@omesc.com>
 *
 *    Camel Brutus folder class.
 *
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU General Public License as
 *    published by the Free Software Foundation; either version 2 of
 *    the License, or (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/* system includes */
#include <errno.h>
#include <time.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

/* camel includes */
#include <camel/camel-folder.h>
#include <camel/camel-folder-search.h>
#include <camel/camel-store.h>
#include <camel/camel-i18n.h>
#include <camel/camel-session.h>
#include <camel/camel-exception.h>
#include <camel/camel-stream-mem.h>

#include <libedataserver/e-iconv.h>
#include <libedataserver/md5-utils.h>

/* camel-brutus includes */
#include "camel-brutus-folder.h"
#include "camel-brutus-folder-summary.h"
#include "camel-brutus-store.h"
#include "camel-brutus-private.h"

/* pure brutus includes */
#include <idl_output/IMAPISession.h>
#include <idl_output/return_codes.h>
#include <idl_output/types.h>

/* evolution-brutus includes */
#include <server/brutus_conv.h>
#include <server/brutus_mapi.h>
#include <server/brutus_smtp.h>
#include <server/brutus_cpid.h>
#include <server/brutus_util.h>
#include <server/brutus.h>

static CamelOfflineFolderClass *parent_class = NULL;

/*
 * helper functions
 */

/* static CamelMimeMessage* */
/* brutus_foo(void) */
/* { */
/*	CamelMimeMessage *msg; */
/*	CamelInternetAddress *addr; */
/*	const char *text; */
/*	CamelMultipart *mp; */
/*	CamelMimePart *part; */
/*	CamelContentType *ct = NULL; */
/*	char *content_type = NULL; */

/*	ct = camel_content_type_new("text", "plain"); */
/*	camel_content_type_set_param(ct, "charset", "UTF-8"); */
/*	content_type = camel_content_type_format(ct); */

/*	msg = camel_mime_message_new(); */
/*	camel_mime_message_set_subject(msg, "Talking to myself again"); */

/*	addr = camel_internet_address_new(); */
/*	camel_internet_address_add(addr, "NotZed", "notzed@somewhere.com"); */
/*	camel_mime_message_set_from(msg, addr); */
/*	camel_address_remove((CamelAddress*)addr, -1); */

/*	camel_internet_address_add(addr, "NotZed", "notzed@somewhere.else.com"); */
/*	camel_mime_message_set_recipients(msg, "To", addr); */
/*	camel_object_unref(addr); */

/*	mp = camel_multipart_new(); */
/*	camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER(mp), "multipart/mixed"); */
/*	camel_multipart_set_boundary(mp, NULL); */
/*	camel_multipart_set_preface(mp, "This is a message in MIME format.  Get with the decade\n"); */
/*	camel_multipart_set_postface(mp, "\nSome useless after-message text you should never see\n\n"); */

/*	part = camel_mime_part_new(); */
/*	text = "Hello NotZed,\nTalking to yourself again?\n\n  NotZed\n"; */
/*	camel_mime_part_set_content(part, text, strlen(text), "text/plain"); */
/*	camel_mime_part_set_content_type(part, content_type); */
/*	camel_content_type_unref(ct); */
/*	camel_multipart_add_part(mp, part); */
/*	camel_object_unref(part); */

/* /\*	part = camel_mime_part_new(); *\/ */
/* /\*	text = "<html>\n<body>\nHello NotZed,<br>\nTalking to yourself again?<br>\n&nbsp;&nbsp;NotZed<br>\n</body>\n</html>\n"; *\/ */
/* /\*	camel_mime_part_set_content(part, text, strlen(text), "text/html"); *\/ */
/* /\*	camel_multipart_add_part(mp, part); *\/ */
/* /\*	camel_object_unref(part); *\/ */

/*	camel_medium_set_content_object(CAMEL_MEDIUM(msg), CAMEL_DATA_WRAPPER(mp)); */

/*	return msg; */
/* } */


/*
 * Converts an object of typoe MAPI_MESSAGE to a CamelMimeMessage
 * TODO: Attachments
 */
static CamelMimeMessage*
camel_brutus_mapi_message_to_camel_mime_message(CamelFolder *folder,
						const char *uid,
						CamelException *ex)
{
	CamelMimeMessage *msg = NULL;
	CamelMimePart *mime_part = NULL;
	CamelMultipart *multipart = NULL;
	CamelMessageInfoBase *info = NULL;
	CamelBrutusFolder *brutus_folder = CAMEL_BRUTUS_FOLDER(folder);
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(folder->parent_store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelInternetAddress *from_addr = NULL;
	CamelInternetAddress *to_addr = NULL;
	CamelInternetAddress *cc_addr = NULL;
	CamelInternetAddress *bcc_addr = NULL;
	CamelContentType *ct = NULL;
	const char *charset = NULL;
	char *content_type = NULL;
	char *msg_id = NULL;
	char *tmp = NULL;
	gchar *tmp_name = NULL;
	gchar *tmp_value = NULL;
	gboolean inbound;
	gboolean plain_text;
	int p = 0;
	CORBA_long n;
	CORBA_long m;
	CORBA_long k;
	CORBA_long g;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	BRUTUS_ENTRYID *eid = NULL;
	BRUTUS_seq_SPropValue *prop_values = NULL;
	BRUTUS_seq_SPropValue *msg_prop_values = NULL;
	BRUTUS_seq_SPropValue *ex_msg_prop_values = NULL;
	BRUTUS_SRowSet *attach_row_set = NULL;
	BRUTUS_SRowSet *recp_row_set = NULL;
	CORBA_Environment ev[1];
#ifdef BRUTUS_SPY
	char *smtp_header = NULL;
	struct _camel_header_raw *camel_header_raw = NULL;
	struct _camel_header_raw *camel_header_tmp = NULL;

	// get everything
	static BRUTUS_SPropTagArray attach_proptag_array = {  // for the attachment table
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {    // for the recipients table
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {     // for the message object
		._maximum = 0,
		._length = 0,
		._buffer = NULL,
	};
#else
	CamelStream *cache_stream = NULL;
	CamelStream *msg_stream = NULL;

	// get selected properties
	static BRUTUS_BDEFINE attach_props[1] = {             // for the attachment table
		/* 0 */ BRUTUS_BRUTUS_PR_ATTACH_FLAGS,              // All columns would be returned unless at least one tag is listed
	};
	static BRUTUS_SPropTagArray attach_proptag_array = {
		._maximum = 1,
		._length = 1,
		._buffer = attach_props,
	};
	static BRUTUS_BDEFINE recp_props[3] = {               // for the recipients table
		/* 0 */ BRUTUS_BRUTUS_PR_RECIPIENT_TYPE,            // MAPI_TO, MAPI_CC, MAPI_BCC
		/* 1 */ BRUTUS_BRUTUS_PR_DISPLAY_NAME,
		/* 2 */ BRUTUS_BRUTUS_PR_EMAIL_ADDRESS,
		/* 3 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray recp_proptag_array = {
		._maximum = 3,
		._length = 3,
		._buffer = recp_props,
	};
	static BRUTUS_BDEFINE msg_props[9] = {                // for the message object
		/* 0 */  BRUTUS_BRUTUS_PR_INTERNET_CPID,             // charset encoding of message body
		/* 1 */  BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE,      // to deduce if inbound or outbound
		/* 2 */  BRUTUS_BRUTUS_PR_SENDER_NAME,               // display name of sender
		/* 3 */  BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS,      // email address of sender
		/* 4 */  BRUTUS_BRUTUS_PR_INTERNET_MESSAGE_ID        // RFC 2822 Message-Id
		/* 5 */  BRUTUS_BRUTUS_PR_BODY,
		/* 6 */  BRUTUS_BRUTUS_PR_DISPLAY_TO,
		/* 7 */  BRUTUS_BRUTUS_PR_DISPLAY_CC,
		/* 8 */  BRUTUS_BRUTUS_PR_DISPLAY_FROM,
		/* 9 */ /* BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray msg_proptag_array = {
		._maximum = 9,
		._length = 9,
		._buffer = msg_props,
	};
#endif
	static BRUTUS_BDEFINE ex_msg_props[2] = {                // for the message object
		/* 0 */ BRUTUS_BRUTUS_PR_BODY,
		/* 1 */ BRUTUS_BRUTUS_PR_BODY_HTML,
		/* 2 *//*  BRUTUS_BRUTUS_PR_, */
	};
	static BRUTUS_SPropTagArray ex_msg_proptag_array = {
		._maximum = 2,
		._length = 2,
		._buffer = ex_msg_props,
	};

	CORBA_exception_init(ev);

	// get info from folder summary
	info = (CamelMessageInfoBase*)camel_folder_summary_uid(folder->summary, uid);
	if (!info) {
		brutus_update_folder_summary(folder, TRUE, ex);
		info = (CamelMessageInfoBase*)camel_folder_summary_uid(folder->summary, uid);
		if (!info)
			goto no_info;
	}

#ifndef BRUTUS_SPY
	// present in cache?
	cache_stream  = camel_data_cache_get(brutus_folder->msg_cache, "cache", uid, NULL);
	if (cache_stream) {

		// write to msg_stream
		msg_stream = camel_stream_mem_new();
		camel_stream_reset(msg_stream);
		camel_stream_write_to_stream(cache_stream, msg_stream);
		camel_object_unref(cache_stream);
		cache_stream = NULL;

		// convert into CamelMimeMessage
		msg = camel_mime_message_new();
		camel_stream_reset(msg_stream);
		if (-1 == camel_data_wrapper_construct_from_stream((CamelDataWrapper *)msg, msg_stream)) {
			camel_object_unref(msg);
			msg = NULL;

			if (EINTR == errno) {
				camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("User cancelled"));
			} else {
				BRUTUS_FOLDER_LOCK(brutus_folder, cache_mutex);
				camel_data_cache_remove(brutus_folder->msg_cache, "cache", uid, NULL); // might be corrupt, safer to renew later
				BRUTUS_FOLDER_UNLOCK(brutus_folder, cache_mutex);
				camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM, _("Cannot get message %s: %s"),
						     uid, g_strerror(errno));
			}
		}
		camel_object_unref(msg_stream);

		if (msg)
			goto out;
	}
#endif
	// create ENTRYID
	eid = brutus_string_to_entryid(uid);
	if (!eid)
		goto no_mem;

	// get selected properties from message object
	br = BRUTUS_IMAPISession_QueryMessage(brutus_folder->mapi_session,
					      eid,
					      &msg_proptag_array,
					      &ex_msg_proptag_array,
					      &attach_proptag_array,
					      &recp_proptag_array,
					      &msg_prop_values,
					      &ex_msg_prop_values,
					      &attach_row_set,
					      &recp_row_set,
					      ev);
	if (ORBIT2_EX(ev)) {
		PRINT_ORBIT2_EX(ev);
		goto error;
	}
	if ((BRUTUS_BRUTUS_MAPI_W_ERRORS_RETURNED != br) && (BRUTUS_BRUTUS_S_OK != br)) {
		d("Could not query message object for data");
		goto error;
	}

#ifdef BRUTUS_SPY
	// get full SMTP header if present
	n = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS);
	if (-1 != n) {
		smtp_header = brutus_unfold_rfc822_header(msg_prop_values->_buffer[n].Value._u.lpszA);
		if (smtp_header) {
			camel_header_raw = brutus_get_camel_header_raw(smtp_header);
			free(smtp_header);
			smtp_header = NULL;
		} else {
			d("No memory");
			goto no_mem;
		}
	} else
		camel_header_raw = NULL;
#endif

	// determine origin, inbound or outbound?
	n = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_RECEIVED_BY_ADDRTYPE); // set by incoming transport provider
	if (-1 != n)
		inbound = TRUE;
	else
		inbound = FALSE; // draft, sent or being edited

	// instantiate mime message
	msg = camel_mime_message_new();
	if (!msg)
		goto no_mem;

	// date
	camel_mime_message_set_date(msg, info->date_sent, brutus_timezone_to_gmt_offset(priv->client_time_zone));

	// subject
	if (info->subject)
		camel_mime_message_set_subject(msg, info->subject);

	// Message-Id:
	n = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_INTERNET_MESSAGE_ID);
	if (-1 != n)
		msg_id = camel_header_msgid_decode(msg_prop_values->_buffer[n].Value._u.lpszA);
	else {
		d("Invalid RFC 822 message header");
#ifndef BRUTUS_SPY
		goto error;
#endif
	}
	if (!msg_id) {
		d("No memory or invalid RFC 822 message header");
#ifndef BRUTUS_SPY
		goto error;
#endif
	}
	if (msg_id)
		camel_mime_message_set_message_id(msg, msg_id);

	/*
	 * Addresses
	 */
	from_addr = camel_internet_address_new();
	if (!from_addr)
		goto no_mem;
	n = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_SENDER_NAME);
	m = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_SENDER_EMAIL_ADDRESS);
	camel_internet_address_add(from_addr,
				   (-1 != n) ? msg_prop_values->_buffer[n].Value._u.lpszA : "",
				   (-1 != m) ? msg_prop_values->_buffer[m].Value._u.lpszA : "");
	camel_mime_message_set_from(msg, from_addr);

	if ((recp_row_set->_length) // anyone?
	    && (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(recp_row_set->_buffer[0]._buffer[0].ulPropTag))) { // we don't know if STORE_SUBMIT_OK is set
		for (k = 0; k<recp_row_set->_length; k++) { // loop all recipients
			n = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_RECIPIENT_TYPE);
			m = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_DISPLAY_NAME);
			g = brutus_get_srow_index(&recp_row_set->_buffer[k], BRUTUS_BRUTUS_PR_EMAIL_ADDRESS);

			if (BRUTUS_BRUTUS_MAPI_TO & recp_row_set->_buffer[k]._buffer[n].Value._u.l) {
				if (!to_addr) {
					to_addr = camel_internet_address_new();
					if (!to_addr)
						goto no_mem;
				}
				camel_internet_address_add(to_addr,
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA : "",  // name  - if PR_EMAIL_ADDRESS is available use the display name, if not use the empty string
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA : recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA); // email - if PR_EMAIL_ADDRESS is available use it, otherwise use the PR_DISPLAY_NAME property
				continue;
			}
			if (BRUTUS_BRUTUS_MAPI_CC & recp_row_set->_buffer[k]._buffer[n].Value._u.l) {
				if (!cc_addr) {
					cc_addr = camel_internet_address_new();
					if (!cc_addr)
						goto no_mem;
				}
				camel_internet_address_add(cc_addr,
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA : "",
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA : recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA);
				continue;
			}
			if (BRUTUS_BRUTUS_MAPI_BCC & recp_row_set->_buffer[k]._buffer[n].Value._u.l) {
				if (!bcc_addr) {
					bcc_addr = camel_internet_address_new();
					if (!bcc_addr)
						goto no_mem;
				}
				camel_internet_address_add(bcc_addr,
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA : "",
							   (-1 != g) ? recp_row_set->_buffer[k]._buffer[g].Value._u.lpszA : recp_row_set->_buffer[k]._buffer[m].Value._u.lpszA);
				continue;
			}
		}
	}
	camel_mime_message_set_recipients(msg, CAMEL_RECIPIENT_TYPE_TO, to_addr);
	camel_mime_message_set_recipients(msg, CAMEL_RECIPIENT_TYPE_CC, cc_addr);
	camel_mime_message_set_recipients(msg, CAMEL_RECIPIENT_TYPE_BCC, bcc_addr);

#ifdef BRUTUS_SPY
	// write out all rfc2822 headers if they exists
	camel_header_tmp = camel_header_raw;
	while (camel_header_tmp) {
		d(camel_header_tmp->name);
		d(camel_header_tmp->value);
		camel_header_tmp = camel_header_tmp->next;
	}

	// add all MAPI properties as custom X-MAPI- headers
	p = 0;
	do {
		prop_values = p ?  ex_msg_prop_values : msg_prop_values;
		if (2 == p)
			break;
		p++;

		for (n = 0; n < prop_values->_length; n++) {
			if (BRUTUS_BRUTUS_PR_TRANSPORT_MESSAGE_HEADERS == prop_values->_buffer[n].ulPropTag)
				continue;
			tmp_name = g_strjoin(NULL,
					     "X-MAPI-(",
					     brutus_proptype_to_str(BRUTUS_PROP_TYPE(prop_values->_buffer[n].ulPropTag)),
					     ")-(",
					     brutus_proptag_to_str(prop_values->_buffer[n].ulPropTag),
					     ")",
					     NULL);
			if (BRUTUS_BRUTUS_PT_ERROR == BRUTUS_PROP_TYPE(prop_values->_buffer[n].ulPropTag))
				tmp_value = (gchar*)brutus_bresult_to_str((BRUTUS_BRESULT)prop_values->_buffer[n].Value._u.err);
			else
				tmp_value = brutus_print_spropvalue(&prop_values->_buffer[n]);

			camel_medium_set_header(CAMEL_MEDIUM(msg), (const char*)tmp_name, (const char*)tmp_value);

			d(tmp_name);
			d(tmp_value);

			g_free(tmp_name);
			if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(prop_values->_buffer[n].ulPropTag))
				g_free(tmp_value);
		}
	} while (TRUE);
	/*
	 * TODO:
	 *
	 * Check if an ePlugin could expand the right-click menu with an additional
	 * property viewing UI, instead of this mess with X- headers.
	 *
	 * Add RecipientTable and AttachmentTable support.
	 */
#endif

	/*
	 * Body part
	 */
	mime_part = camel_mime_part_new();
	if (!mime_part)
		goto no_mem;
	camel_mime_part_set_encoding(mime_part, CAMEL_TRANSFER_ENCODING_8BIT);

	multipart = camel_multipart_new();
	if (!multipart)
		goto no_mem;
	camel_data_wrapper_set_mime_type(CAMEL_DATA_WRAPPER(multipart), "multipart/mixed"); // or alternative of both PR_BODY and PR_BODY_HTML exists?
	camel_multipart_set_boundary(multipart, NULL);

	// determine body content
	plain_text = TRUE;
	if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(ex_msg_prop_values->_buffer[1].ulPropTag)) { // PR_BODY_HTML
		tmp = ex_msg_prop_values->_buffer[1].Value._u.lpszA;
		plain_text = FALSE;
	} else if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(ex_msg_prop_values->_buffer[0].ulPropTag)) // PR_BODY
		tmp = ex_msg_prop_values->_buffer[0].Value._u.lpszA;
	else // TODO: use IMAPIProp::OpenProperty with IStream to get big body texts - ignore for now...
		tmp = "";

	// determine charset encoding on body part
	n = brutus_get_spropvalue_index(msg_prop_values, BRUTUS_BRUTUS_PR_INTERNET_CPID);
	if (BRUTUS_BRUTUS_PT_ERROR != BRUTUS_PROP_TYPE(msg_prop_values->_buffer[n].ulPropTag))
		charset = brutus_cpid_to_label(msg_prop_values->_buffer[n].Value._u.l);
	charset = charset ? e_iconv_charset_name(charset) : "us-ascii";
	ct = camel_content_type_new("text", plain_text ? "plain" : "html");
	camel_content_type_set_param(ct, "charset", charset);
	content_type = camel_content_type_format(ct);

	// set body content
	camel_mime_part_set_content(mime_part, tmp, strlen(tmp), plain_text ? "text/plain" : "text/html");
	camel_mime_part_set_content_type(mime_part, content_type);
	camel_content_type_unref(ct);
	g_free(content_type);

	// add content part to mime multipart
	camel_multipart_add_part(multipart, mime_part);
	camel_object_unref(mime_part);
	mime_part = NULL;

	// set content object
	camel_medium_set_content_object(CAMEL_MEDIUM(msg), CAMEL_DATA_WRAPPER(multipart));

#ifndef BRUTUS_SPY
	// add to cache
	BRUTUS_FOLDER_LOCK(brutus_folder, cache_mutex);
	cache_stream = camel_data_cache_add(brutus_folder->msg_cache, "cache", uid, NULL);
	if (cache_stream) {
		if ((-1 == camel_data_wrapper_write_to_stream((CamelDataWrapper *)msg, cache_stream))
		    || (-1 == camel_stream_flush(cache_stream))) {
			d("Could not write to message cache");
			camel_data_cache_remove(brutus_folder->msg_cache, "cache", uid, NULL); // no space on device?
		}
		camel_object_unref(cache_stream);
	}
	BRUTUS_FOLDER_UNLOCK(brutus_folder, cache_mutex);
#endif
	goto out;

error:
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not construct the message - Error"));
	goto out;

no_mem:
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not construct the message - No memory"));
	goto out;

no_info:
	camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
			    _("Could not construct the message - It does not exist on the remote server"));
	goto out;

out:
	if (mime_part)
		camel_object_unref(mime_part);

	if (multipart)
		camel_object_unref(multipart);

	if (camel_exception_is_set(ex) && msg) {
		camel_object_unref(msg);
		msg = NULL;
	}

	if (from_addr)
		camel_object_unref(from_addr);

	if (to_addr)
		camel_object_unref(to_addr);

	if (cc_addr)
		camel_object_unref(cc_addr);

	if (bcc_addr)
		camel_object_unref(bcc_addr);

	if (info)
		camel_message_info_free(info);

	free(msg_id);

	CORBA_free(eid);
	CORBA_free(msg_prop_values);
	CORBA_free(ex_msg_prop_values);
	CORBA_free(attach_row_set);
	CORBA_free(recp_row_set);
	CORBA_exception_free(ev);

#ifdef BRUTUS_SPY
	camel_header_raw_clear(&camel_header_raw);
#endif

#ifdef BRUTUS_DEBUG
	camel_mime_message_dump(msg, 1);
#endif

	return msg;
}


/*
 * virtual methods
 */

static int brutus_folder_getv(CamelObject *object,
			      CamelException *ex,
			      CamelArgGetV *args);

void camel_brutus_folder_sync(CamelFolder *folder,
			      gboolean expunge,
			      CamelException *ex);

/* message manipulation */
void camel_brutus_folder_append_message(CamelFolder *folder,
					CamelMimeMessage *message,
					const CamelMessageInfo *info,
					char **appended_uid,
					CamelException *ex);

/* uid based access operations */
CamelMimeMessage *camel_brutus_folder_get_message(CamelFolder *folder,
						  const char *uid,
						  CamelException *ex);

/* search api */
gboolean camel_brutus_folder_has_search_capability(CamelFolder *folder);

/* GPtrArray *camel_brutus_folder_search_by_expression(CamelFolder *folder, */
/*						    const char *expr, */
/*						    CamelException *ex); */

/* GPtrArray *camel_brutus_folder_search_by_uids(CamelFolder *folder, */
/*					      const char *expr, */
/*					      GPtrArray *uids, */
/*					      CamelException *ex); */

/* void camel_brutus_folder_search_free(CamelFolder *folder, */
/*				     GPtrArray *result); */

/* summary info */
void camel_brutus_folder_refresh_info(CamelFolder *folder,
				      CamelException *ex);

CamelMessageInfo *camel_brutus_folder_get_message_info(CamelFolder *folder,
						       const char *uid);

/* moving and deleting */
void camel_brutus_folder_transfer_messages_to(CamelFolder *source,
					      GPtrArray *uids,
					      CamelFolder *dest,
					      GPtrArray **transferred_uids,
					      gboolean delete_originals,
					      CamelException *ex);

void camel_brutus_folder_delete(CamelFolder *folder);

void camel_brutus_folder_rename(CamelFolder *folder,
				const char *new);

/* stop/restart getting events */
void camel_brutus_folder_freeze(CamelFolder *folder);

void camel_brutus_folder_thaw(CamelFolder *folder);

gboolean camel_brutus_folder_is_frozen(CamelFolder *folder);

///////////////////////////////////////


void
camel_brutus_folder_refresh_info(CamelFolder *folder,
				 CamelException *ex)
{
	d(folder->full_name);

	// incremental update only is the default case
	brutus_update_folder_summary(folder, FALSE, ex);
}

void camel_brutus_folder_sync(CamelFolder *folder,
			      gboolean expunge,
			      CamelException *ex)
{
	d("");
}

/* message manipulation */
void
camel_brutus_folder_append_message(CamelFolder *folder,
				   CamelMimeMessage *message,
				   const CamelMessageInfo *info,
				   char **appended_uid,
				   CamelException *ex)
{
	d("");
}

/* uid based access operations */
CamelMimeMessage*
camel_brutus_folder_get_message(CamelFolder *folder,
				const char *uid,
				CamelException *ex)
{
	d("");
	return camel_brutus_mapi_message_to_camel_mime_message(folder, uid, ex);
}

/* search api */
gboolean
camel_brutus_folder_has_search_capability(CamelFolder *folder)
{
	d("");

	return FALSE;
}

/*
 * Commented out for now
 */

/* GPtrArray* */
/* camel_brutus_folder_search_by_expression(CamelFolder *folder, */
/*					 const char *expr, */
/*					 CamelException *ex) */
/* { */
/*	GPtrArray *retv = NULL; */

/*	d(""); */

/*	return retv; */
/* } */

/* GPtrArray* */
/* camel_brutus_folder_search_by_uids(CamelFolder *folder, */
/*				   const char *expr, */
/*				   GPtrArray *uids, */
/*				   CamelException *ex) */
/* { */
/*	GPtrArray *retv = NULL; */

/*	d(""); */

/*	return retv; */
/* } */

/* void */
/* camel_brutus_folder_search_free(CamelFolder *folder, */
/*				GPtrArray *result) */
/* { */
/* } */


/* message info */
CamelMessageInfo*
camel_brutus_folder_get_message_info(CamelFolder *folder,
				     const char *uid)
{
	d("");
	return camel_folder_summary_uid(folder->summary, uid);
}

void
camel_brutus_folder_transfer_messages_to(CamelFolder *source,
					 GPtrArray *uids,
					 CamelFolder *dest,
					 GPtrArray **transferred_uids,
					 gboolean delete_originals,
					 CamelException *ex)
{
	d("");
}

void
camel_brutus_folder_delete(CamelFolder *folder)
{
	d("");

}

void
camel_brutus_folder_rename(CamelFolder *folder,
			   const char *new)
{
	d("");

}

/* stop/restart getting events */
void
camel_brutus_folder_freeze(CamelFolder *folder)
{
	d("");

}

void
camel_brutus_folder_thaw(CamelFolder *folder)
{
	d("");

}

gboolean
camel_brutus_folder_is_frozen(CamelFolder *folder)
{
	gboolean retv = FALSE;

	d("");

	return retv;
}

static void
camel_brutus_folder_class_init(CamelBrutusFolderClass *camel_brutus_folder_class)
{
	CamelFolderClass *camel_folder_class = CAMEL_FOLDER_CLASS(camel_brutus_folder_class);

	d("");

	parent_class = CAMEL_OFFLINE_FOLDER_CLASS(camel_type_get_global_classfuncs(camel_offline_folder_get_type ()));

	((CamelObjectClass *) camel_folder_class)->getv = brutus_folder_getv;

	camel_folder_class->refresh_info = camel_brutus_folder_refresh_info;
	camel_folder_class->sync = camel_brutus_folder_sync;
	camel_folder_class->append_message = camel_brutus_folder_append_message;
	camel_folder_class->get_message = camel_brutus_folder_get_message;
	camel_folder_class->has_search_capability = camel_brutus_folder_has_search_capability;
/*	camel_folder_class->search_by_expression = camel_brutus_folder_search_by_expression; */
/*	camel_folder_class->search_by_uids = camel_brutus_folder_search_by_uids; */
/*	camel_folder_class->search_free = camel_brutus_folder_search_free; */
	camel_folder_class->get_message_info = camel_brutus_folder_get_message_info;
	camel_folder_class->transfer_messages_to = camel_brutus_folder_transfer_messages_to;
	camel_folder_class->delete = camel_brutus_folder_delete;
	camel_folder_class->rename = camel_brutus_folder_rename;
	camel_folder_class->freeze = camel_brutus_folder_freeze;
	camel_folder_class->thaw = camel_brutus_folder_thaw;
	camel_folder_class->is_frozen = camel_brutus_folder_is_frozen;
}

static void
camel_brutus_folder_init(gpointer object,
			 gpointer klass)
{
	CamelBrutusFolder *brutus_folder = CAMEL_BRUTUS_FOLDER(object);
	CamelFolder *folder = CAMEL_FOLDER(object);

	d("");
	folder->description = NULL;
	folder->summary = NULL;
	folder->permanent_flags =
		CAMEL_MESSAGE_ANSWERED
		| CAMEL_MESSAGE_ANSWERED_ALL
		| CAMEL_MESSAGE_DELETED
		| CAMEL_MESSAGE_DRAFT
		| CAMEL_MESSAGE_FLAGGED
		| CAMEL_MESSAGE_SEEN
		| CAMEL_MESSAGE_JUNK
		| CAMEL_MESSAGE_USER;

	folder->folder_flags =
		CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY
		/* | CAMEL_FOLDER_HAS_SEARCH_CAPABILITY */;

	brutus_folder->mapi_session = CORBA_OBJECT_NIL;
	brutus_folder->mapi_folder = CORBA_OBJECT_NIL;
	brutus_folder->search = NULL;
	brutus_folder->summary_mutex = g_mutex_new();
	brutus_folder->cache_mutex = g_mutex_new();
	brutus_folder->msg_cache = NULL;
}

static void
camel_brutus_folder_finalize(CamelObject *object)
{
	CORBA_boolean cb = CORBA_TRUE;
	CamelBrutusFolder *brutus_folder = CAMEL_BRUTUS_FOLDER(object);
	CORBA_Environment ev[1];

	CORBA_exception_init(ev);

	cb = CORBA_Object_is_nil(brutus_folder->mapi_folder, ev);
	if (!ORBIT2_EX(ev) && !cb) {
		BRUTUS_IMAPIFolder_Destroy(brutus_folder->mapi_folder, 0, ev);
		PRINT_IF_ORBIT2_EX(ev);

		CORBA_Object_release(brutus_folder->mapi_folder, ev);
	}

	cb = CORBA_Object_is_nil(brutus_folder->mapi_session, ev);
	if (!ORBIT2_EX(ev) && !cb) 
		CORBA_Object_release(brutus_folder->mapi_session, ev);

	CORBA_exception_free(ev);

	if (brutus_folder->summary_mutex) 
		g_mutex_free(brutus_folder->summary_mutex);

	if (brutus_folder->cache_mutex) 
		g_mutex_free(brutus_folder->cache_mutex);

	if (brutus_folder->search) 
		camel_object_unref(brutus_folder->search);

	if (brutus_folder->msg_cache) 
		camel_object_unref(brutus_folder->msg_cache);
}

static int
brutus_folder_getv(CamelObject *object,
		   CamelException *ex,
		   CamelArgGetV *args)
{
	CamelFolder *folder = (CamelFolder*)object;
	int i;
	guint32 tag;
	int unhandled = 0;

	d("");
	for (i = 0; i < args->argc; i++) {
		CamelArgGet *arg = &args->argv[i];

		tag = arg->tag;
		switch (tag & CAMEL_ARG_TAG) {
		case CAMEL_OBJECT_ARG_DESCRIPTION:
			if (!folder->description) {
				CamelURL *uri = ((CamelService*)folder->parent_store)->url;
				folder->description = g_strdup_printf("%s@%s:%s", uri->user, uri->host, folder->full_name);
			}
			*arg->ca_str = folder->description;
			break;
		default:
			unhandled++;
			continue;
		}
		arg->tag = (tag & CAMEL_ARG_TYPE) | CAMEL_ARG_IGNORE;
	}
	if (unhandled)
		return ((CamelObjectClass *)parent_class)->getv(object, ex, args);

	return 0;
}

CamelType
camel_brutus_folder_get_type(void)
{
	static CamelType camel_brutus_folder_type = CAMEL_INVALID_TYPE;

	d("");

	if (camel_brutus_folder_type == CAMEL_INVALID_TYPE) {
		camel_brutus_folder_type =
			camel_type_register(camel_offline_folder_get_type (),
					    "CamelBrutusFolder",
					    sizeof(CamelBrutusFolder),
					    sizeof(CamelBrutusFolderClass),
					    (CamelObjectClassInitFunc)camel_brutus_folder_class_init,
					    NULL,
					    (CamelObjectInitFunc)camel_brutus_folder_init,
					    (CamelObjectFinalizeFunc)camel_brutus_folder_finalize);
	}

	return camel_brutus_folder_type;
}

/*
 * The service must be connected when calling this function
 */
CamelFolder*
camel_brutus_folder_new(CamelStore *store,
			const char *folder_name,
			guint32 flags,
			CamelException *ex)
{
	CamelBrutusStore *brutus_store = CAMEL_BRUTUS_STORE(store);
	CamelBrutusStorePrivate *priv = brutus_store->priv;
	CamelFolder *folder;
	CamelBrutusFolder *brutus_folder;
	CORBA_boolean cb = CORBA_TRUE;
	BRUTUS_BRESULT br = BRUTUS_BRUTUS_UNKNOWN_ERROR;
	CORBA_string iid;
	BRUTUS_ENTRYID *eid = NULL;
	int i;
	char *name;
	char *eid_str = NULL;
	char *summary_file_path = NULL;
	char *cache_file_path = NULL;
	guchar digest[16] = { 0 }; // md5 hash of long term ENTRYID
	gchar digest_str[33] = { '\0' }; // stringified version of digest
	BRUTUS_BDEFINE obj_type;
	BRUTUS_IMAPIFolder new_mapi_folder = CORBA_OBJECT_NIL;
	CORBA_Environment ev[1];

	if (!folder_name || !strlen(folder_name))
		return NULL;

	CORBA_exception_init(ev);
	iid = CORBA_string_dup("");

	folder = CAMEL_FOLDER(camel_object_new(camel_brutus_folder_get_type()));
	brutus_folder = CAMEL_BRUTUS_FOLDER(folder);
	name = strrchr(folder_name, '/');
	if (name)
		name++;
	else
		name = (char*)folder_name;
	camel_folder_construct(folder, store, folder_name, name);

	d(folder_name);
	d(name);

	eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, folder_name);

	if (!eid) {
		if (!(flags & CAMEL_STORE_FOLDER_CREATE)) // not in the hash table, do not create it
			goto ex;
		
		// not in the hash table, create it
		BRUTUS_IMAPIFolder mapi_folder = CORBA_OBJECT_NIL;
		CORBA_string pr_display_name = CORBA_string_dup(name);
		CORBA_string pr_comment = CORBA_string_dup("");
		char *parent_name = g_strdup(folder_name);
		char *tmp = strrchr (parent_name, '/');
		*tmp = '\0';

		d(parent_name);
		eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, parent_name);
		g_free(parent_name);
		if (!eid) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Internal data corruption - Restart Evolution"));
			goto cleanup;
		}

		br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
						   eid,
						   iid,
						   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
						   &obj_type,
						   &mapi_folder,
						   ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Communication error"));
			goto cleanup;
		}
		if (BRUTUS_BRUTUS_MAPI_E_NOT_FOUND == br) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Internal data corruption - Restart Evolution"));
			goto cleanup;
		}
		if (BRUTUS_BRUTUS_S_OK != br)
			goto cleanup;

		d(pr_display_name);
		d(pr_comment);
		br = BRUTUS_IMAPIFolder_CreateFolder(mapi_folder,
						     BRUTUS_BRUTUS_FOLDER_GENERIC,
						     pr_display_name,
						     pr_comment,
						     iid,
						     BRUTUS_BRUTUS_OPEN_IF_EXISTS, // folder created after Evolution startup
						     &new_mapi_folder,
						     ev);
		if (BRUTUS_BRUTUS_MAPI_E_COLLISION == br) {
			camel_exception_set(ex, CAMEL_EXCEPTION_FOLDER_INVALID_PATH,
					    _("Could not create folder - Identically named sibling folders not allowed"));
			goto cleanup;
		}
		if (BRUTUS_BRUTUS_S_OK == br) {
			br = BRUTUS_IMAPIProp_SaveChanges((BRUTUS_IMAPIProp)mapi_folder, BRUTUS_BRUTUS_KEEP_OPEN_READWRITE, ev);
			if (BRUTUS_BRUTUS_S_OK == br) {
				brutus_insert_in_folder_tree(store, parent_name, folder_name, new_mapi_folder, ex);
				eid = (BRUTUS_ENTRYID*)g_hash_table_lookup(priv->entryid_from_full_name_hash, folder_name);
			}
		}

	cleanup:
		cb = CORBA_Object_is_nil(mapi_folder, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPIFolder_Destroy(mapi_folder, 0, ev);
			PRINT_IF_ORBIT2_EX(ev);
			CORBA_Object_release(mapi_folder, ev);
		}
		CORBA_free(pr_display_name);
		CORBA_free(pr_comment);
		if (camel_exception_is_set(ex))
			goto ex;
	} else { // present in hash table, re-opening
		br = BRUTUS_IMAPISession_OpenEntry(priv->mapi_session,
						   eid,
						   iid,
						   BRUTUS_BRUTUS_MAPI_BEST_ACCESS,
						   &obj_type,
						   &new_mapi_folder,
						   ev);
		if (ORBIT2_EX(ev)) {
			PRINT_ORBIT2_EX(ev);
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Communication error"));
			goto ex;
		}
		if (BRUTUS_BRUTUS_MAPI_E_NOT_FOUND == br) {
			camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
					    _("Internal data corruption - Restart Evolution"));
			goto ex;
		}
		if (BRUTUS_BRUTUS_MAPI_E_NO_ACCESS == br) {
			camel_exception_set(ex, CAMEL_EXCEPTION_FOLDER_INSUFFICIENT_PERMISSION,
					    _("Could not open folder - Access Denied"));
			goto ex;
		}
	}
	if (BRUTUS_BRUTUS_S_OK == br) {
		brutus_folder->mapi_session = (BRUTUS_IMAPISession)CORBA_Object_duplicate((CORBA_Object)priv->mapi_session, ev);;
		brutus_folder->mapi_folder = new_mapi_folder;
	} else {
		cb = CORBA_Object_is_nil(new_mapi_folder, ev);
		if (!ORBIT2_EX(ev) && !cb) {
			BRUTUS_IMAPIFolder_Destroy(new_mapi_folder, 0, ev);
			PRINT_IF_ORBIT2_EX(ev);

			CORBA_Object_release(new_mapi_folder, ev);
		}
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("Could not open folder"));
		goto ex;
	}

	// folder summary and message cache, must have md5 hash of eid to construct the save path 
	// to avoid too long directory or file names - therefore delayed till now
	eid_str = brutus_entryid_to_string(eid);
	md5_get_digest(eid_str, strlen(eid_str), digest);
	free(eid_str);
	for (i = 0; i < sizeof(digest); i++) 
		snprintf(&digest_str[2*i], 3, "%02x", (guint8)digest[i]);

	// construct the paths
	summary_file_path = g_strdup_printf ("%s/%s/%s", priv->storage_path, FOLDER_SUMMARIES, digest_str);
	cache_file_path = g_strdup_printf ("%s/%s/%s", priv->storage_path, FOLDER_MESSAGE_CACHE, digest_str);

	// create summary and message cache
	folder->summary = camel_brutus_summary_new(folder, summary_file_path);
	g_free(summary_file_path);
	brutus_folder->msg_cache = camel_data_cache_new(cache_file_path, 0, ex);
	g_free(cache_file_path);
	camel_data_cache_set_expire_age(brutus_folder->msg_cache, MESSAGE_CACHE_TIMEOUT);

	goto out;

ex:
	camel_object_unref(folder);
	folder = NULL;

out:
	CORBA_free(iid);
	if (ORBIT2_EX(ev)) {
		camel_exception_set(ex, CAMEL_EXCEPTION_SYSTEM,
				    _("CORBA exception caught"));

	}
	CORBA_exception_free(ev);

	return folder;
}
