/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Camel Brutus store class. 
 *
 *    Authors:
 *        Jules Colding <colding@omesc.com>
 *
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *    MA 02111-1307 USA
 */

#ifndef CAMEL_BRUTUS_STORE_H
#define CAMEL_BRUTUS_STORE_H 1

#include <camel/camel-store.h>
#include <camel/camel-offline-store.h>

#include <idl_output/IMAPISession.h>
#include <idl_output/IMAPIFolder.h>
#include <idl_output/IMAPITable.h>
#include <idl_output/IMsgStore.h>

#include <session/brutus-proxy.h>
#include <session/brutusd-util.h>

G_BEGIN_DECLS

#define CAMEL_BRUTUS_STORE_TYPE     (camel_brutus_store_get_type ())
#define CAMEL_BRUTUS_STORE(obj)     (CAMEL_CHECK_CAST((obj), CAMEL_BRUTUS_STORE_TYPE, CamelBrutusStore))
#define CAMEL_BRUTUS_STORE_CLASS(k) (CAMEL_CHECK_CLASS_CAST ((k), CAMEL_BRUTUS_STORE_TYPE, CamelBrutusStoreClass))
#define CAMEL_IS_BRUTUS_STORE(o)    (CAMEL_CHECK_TYPE((o), CAMEL_BRUTUS_STORE_TYPE))

struct BrutusMsgStore {
	BRUTUS_IMsgStore msg_store;
	guint32 support_mask;
};

/* store-private data */
typedef struct _CamelBrutusStorePrivate {
	GThread *orb_thread; // ORB run thread

	CORBA_ORB orb;                    // store-global ORB
	PortableServer_POA root_poa;      // store-global POA

	gboolean use_ssliop; // SSLIOP or not?

	CORBA_char *mapi_profile;
	BRUTUS_IMAPISession mapi_session; // store-global IMAPISession
	CORBA_unsigned_long instance_id;  // IMAPISession instance id

	BRUTUS_BrutusProxy brutus_proxy;   // IPv4/6 capable Brutus proxy
	struct BrutusCheckProxy lifeline;  // lifeline callback, local and proxy

	long exchange_time_zone; // The Exchange Server timezone offset from GMT in minutes 
	long client_time_zone;   // The evolution-brutus timezone offset from GMT in minutes 

	BRUTUS_IMAPITable mapi_msg_store_table; // Table of all message stores for this account
	GPtrArray *msg_stores; // Array of struct BrutusMsgStore to all message stores retrieved from the table above. Must hold the connect_mutex to access

	/* A tree of CamelFolderInfo structure nodes for *all* folders */
	/* accessible on Exchange. Caller must hold the folder_tree_mutex to access */
	CamelFolderInfo *folder_tree; 

	/* The folder_tree_mutex must be acquired before */
	/* accessing any of these hash tables  */
	GHashTable *fi_from_entryid_hash;        // get folder info node pointer (value) from pointer to BRUTUS_ENTRYID (key) - entry id must be freed
	GHashTable *fi_from_full_name_hash;      // get folder info node pointer (value) from full_name (key)
	GHashTable *entryid_from_full_name_hash; // get pointer to BRUTUS_ENTRYID (value) from full_name (key) - entry id must be freed

	gchar *base_url;         // base service URL
	gchar *storage_path;     // local on-disk store-private directory
	gchar *brutus_logon_str; // corbaloc IOR of remote BrutusLogOn object

        /* ORB startup */
	GMutex *proceed_orb_ok_mutex; // protects the condition below
	GCond *proceed_orb_ok;        // signals that the ORB is ready
	gboolean corba_ready;         // is CORBA up and running?

        /* ORB shutdown */
	GMutex *proceed_orb_destroy_mutex; // protects the condition below
	GCond *proceed_orb_destroy;        // signals that the ORB is destroyed 
	gboolean orb_destroy;              // is CORBA down?

	GMutex *connect_mutex;     // serialize connect and disconnect operations
	GMutex *folder_tree_mutex; // serialize operations on the folder tree and related hash tables
} CamelBrutusStorePrivate;

typedef struct {
	CamelOfflineStore parent_object;

	CamelBrutusStorePrivate *priv; // Misc Brutus specific data and objects
} CamelBrutusStore;

typedef struct {
	CamelOfflineStoreClass parent_class;

} CamelBrutusStoreClass;

/* Standard Camel function */
extern CamelType camel_brutus_store_get_type(void);

/* Inserts a new folder into the folder hierarchy tree under the parent. 
 * Returns TRUE if successful, FALSE otherwise. Will not free mapi_folder. */
extern gboolean brutus_insert_in_folder_tree(CamelStore *store,
					     const char *parent_name,
					     const char *full_name,
					     BRUTUS_IMAPIFolder mapi_folder, 
					     CamelException *ex);

G_END_DECLS

#endif /* CAMEL_BRUTUS_STORE_H */
