/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Misc Brutus specific definitions. 
 *
 *    Authors:
 *        Jules Colding <colding@omesc.com>
 *
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *    MA 02111-1307 USA
 */

#ifndef BRUTUS_H
#define BRUTUS_H 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib/gmacros.h>
#include <execinfo.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <unistd.h>

G_BEGIN_DECLS

/*************************
 *      Capabilities     *
 ************************/

/* define BRUTUS_DEBUG=yes in the environment if you want debugging output */
#define ENV_DEBUG "BRUTUS_DEBUG"

/* Prepend before all output */
#define DEBUG_PREFIX "\n** BRUTUS_DEBUG"

/* Do we support SSLIOP? */
#define BRUTUS_HAVE_SSL 1
#undef BRUTUS_HAVE_SSL // not tested yet

/* Do we support encrypted passwords? */
#define BRUTUS_HAVE_ENCRYPTED_AUTH 1
#undef BRUTUS_HAVE_ENCRYPTED_AUTH  // not implemented yet


/*************************
 *      Utility macros   *
 ************************/

/*************************
 *       Debug macros    *
 ************************/

#undef ORBIT2_EX
#define ORBIT2_EX(_ev_) ((_ev_) && ((_ev_)->_major != CORBA_NO_EXCEPTION))

#undef PRINT_ORBIT2_EX
#ifdef BRUTUS_DEBUG
#define PRINT_ORBIT2_EX(_ev_)                                                                                                        \
        G_STMT_START {                                                                                                               \
                if (getenv(ENV_DEBUG)) {                                                                                             \
		        switch ((_ev_)->_major) {                                                                                    \
		        case CORBA_NO_EXCEPTION :                                                                                    \
			        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - No exception.\n", getppid(), getpid());               \
			        break;                                                                                               \
		        case CORBA_USER_EXCEPTION :                                                                                  \
			        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s@%d: User Exception : %s\n", getppid(), getpid(),   \
                                        __FILE__, __LINE__, CORBA_exception_id ((_ev_)));                                            \
			        break;                                                                                               \
		        case CORBA_SYSTEM_EXCEPTION :                                                                                \
			        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s@%d: System Exception : %s\n", getppid(), getpid(), \
                                        __FILE__, __LINE__, CORBA_exception_id ((_ev_)));                                            \
			        break;                                                                                               \
		        default :                                                                                                    \
			        break;                                                                                               \
		        }                                                                                                            \
                        fflush(NULL);                                                                                                \
		}                                                                                                                    \
	} G_STMT_END
#else
#define PRINT_ORBIT2_EX(_ev_) G_STMT_START { } G_STMT_END
#endif

#undef PRINT_IF_ORBIT2_EX
#ifdef BRUTUS_DEBUG
#define PRINT_IF_ORBIT2_EX(_ev_)                                                                                                             \
        G_STMT_START {                                                                                                                       \
                if ((_ev_) && ((_ev_)->_major != CORBA_NO_EXCEPTION) && (getenv(ENV_DEBUG))) {                                               \
		        switch ((_ev_)->_major) {                                                                                            \
		        case CORBA_USER_EXCEPTION :                                                                                          \
			        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: User Exception : %s\n", getppid(), getpid(),   \
                                        __FUNCTION__, __FILE__, __LINE__, CORBA_exception_id ((_ev_)));                                      \
			        break;                                                                                                       \
		        case CORBA_SYSTEM_EXCEPTION :                                                                                        \
			        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: System Exception : %s\n", getppid(), getpid(), \
                                        __FUNCTION__, __FILE__, __LINE__, CORBA_exception_id ((_ev_)));                                      \
			        break;                                                                                                       \
		        default :                                                                                                            \
			        break;                                                                                                       \
		        }                                                                                                                    \
                        fflush(NULL);                                                                                                        \
		}                                                                                                                            \
	} G_STMT_END
#else
#define PRINT_IF_ORBIT2_EX(_ev_) G_STMT_START { } G_STMT_END
#endif

#undef d
#ifdef BRUTUS_DEBUG
#define d(_str_)                                                                                                                                            \
        G_STMT_START {                                                                                                                                      \
                if (getenv(ENV_DEBUG)) {                                                                                                                    \
                        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: %s\n", getppid(), getpid(), __FUNCTION__, __FILE__, __LINE__, _str_); \
                        fflush(NULL);                                                                                                                       \
                }                                                                                                                                           \
        } G_STMT_END
#else
#define d(_str_) G_STMT_START { } G_STMT_END
#endif

#undef dfmt
#ifdef BRUTUS_DEBUG
#define dfmt(_fmt_, _str_)                                                                                                                                       \
        G_STMT_START {                                                                                                                                           \
                if (getenv(ENV_DEBUG)) {                                                                                                                         \
                        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: "_fmt_"\n", getppid(), getpid(), __FUNCTION__, __FILE__, __LINE__, _str_); \
                        fflush(NULL);                                                                                                                            \
                }                                                                                                                                                \
        } G_STMT_END
#else
#define dfmt(_fmt_, _str_) G_STMT_START { } G_STMT_END
#endif

#undef dfmt2
#ifdef BRUTUS_DEBUG
#define dfmt2(_fmt_, _str1_, _str2_)                                                                                                                                      \
        G_STMT_START {                                                                                                                                                    \
                if (getenv(ENV_DEBUG)) {                                                                                                                                  \
                        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: "_fmt_"\n", getppid(), getpid(), __FUNCTION__, __FILE__, __LINE__, _str1_, _str2_); \
                        fflush(NULL);                                                                                                                                     \
                }                                                                                                                                                         \
        } G_STMT_END
#else
#define dfmt2(_fmt_, _str1_, _str2_) G_STMT_START { } G_STMT_END
#endif

#undef dbt
#ifdef BRUTUS_DEBUG
#define dbt()                                                                                                                                                                        \
        G_STMT_START {                                                                                                                                                               \
                if (getenv(ENV_DEBUG)) {                                                                                                                                             \
                        void *array[20];                                                                                                                                             \
                        size_t size;                                                                                                                                                 \
                        size_t i;                                                                                                                                                    \
                        char **strings;                                                                                                                                              \
                                                                                                                                                                                     \
                        size = backtrace(array, 20);                                                                                                                                 \
                        strings = backtrace_symbols(array, size);                                                                                                                    \
                        g_print(DEBUG_PREFIX" (ppid = %d / pid = %d) - %s() in %s@%d: %s\n\n", getppid(), getpid(), __FUNCTION__, __FILE__, __LINE__, "Printing stack backtrace.."); \
                        for (i = 0; i < size; i++)                                                                                                                                   \
                                g_print("\t%s\n", strings[i]);                                                                                                                       \
                        free(strings);                                                                                                                                               \
                        fflush(NULL);                                                                                                                                                \
                }                                                                                                                                                                    \
        } G_STMT_END
#else
#define dbt() G_STMT_START { } G_STMT_END
#endif

G_END_DECLS

#endif /* BRUTUS_H */
