/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Brutus header file for Brutus MAPI utility functions.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA 02111-1307 USA
 */

#ifndef _BRUTUS_MAPI_H_
#define _BRUTUS_MAPI_H_

#include <idl_output/types.h>
#include <idl_output/return_codes.h>
#include <idl_output/IMAPITable.h>
#include <idl_output/IMAPIProp.h>
#include <idl_output/IMAPISession.h>
#include <idl_output/IMsgStore.h>
#include <idl_output/IMAPIFolder.h>

G_BEGIN_DECLS



/*
 * Will copy the content of src to dest
 */
extern void brutus_copy_spropvalue(BRUTUS_SPropValue *dest,
				   const BRUTUS_SPropValue *src);

/*
 * Will free the content of the SPropValue
 */
extern void brutus_free_spropvalue_content(BRUTUS_SPropValue *pv,
					   PortableServer_POA poa);

/*
 * Will walk and free the content of the SPropValue sequence
 */
extern void brutus_free_seq_spropvalue_content(BRUTUS_seq_SPropValue *prop_values,
					       PortableServer_POA poa);

/*
 * Will walk and free the content of the address list
 */
extern void brutus_free_addrlist_content(BRUTUS_ADRLIST *addr_list,
					 PortableServer_POA poa);

/*
 * Returns TRUE if the ENTRYID is shortterm, FALSE otherwise
 */
extern gboolean brutus_entryid_is_shortterm(const BRUTUS_ENTRYID *Eid);

/*
 * Will open the message and return one propvalue from it or NULL in error.
 */
extern BRUTUS_SPropValue *brutus_open_object_get_one_prop(BRUTUS_IMAPISession Session,
							  BRUTUS_IMAPIFolder Folder,
							  BRUTUS_ENTRYID *Eid,
							  const BRUTUS_BDEFINE PropTag);

/*
 * Will return a valid reference or CORBA_OBJECT_NIL
 */
extern BRUTUS_IMessage brutus_open_message(BRUTUS_IMAPISession Session,
					   BRUTUS_IMAPIFolder Folder,
					   const char *UID);

/*
 * Will return a stringified entryid
 */
extern char *brutus_entryid_to_string(const BRUTUS_ENTRYID *Eid);

/*
 * Will return a pointer to a BRUTUS_ENTRYID 
 */
extern BRUTUS_ENTRYID *brutus_string_to_entryid(const char *Str);

/*
 * Will return TRUE if they are equal, FALSE otherwise
 */
extern gboolean brutus_entryid_cmp(gconstpointer a,
				   gconstpointer b);

/* 
 * Will return the parent folder or CORBA_OBJECT_NIL in error
 */
extern BRUTUS_IMAPIFolder brutus_get_parent_folder(BRUTUS_IMAPISession Session,
						   BRUTUS_IMAPIProp Object);
 
/*
 * Will return a copy of a BRUTUS_ENTRYID
 */
extern BRUTUS_ENTRYID *brutus_entryid_dup(const BRUTUS_ENTRYID *orig);

/*
 * Converts a binary BLOB to a BRUTUS_ENTRYID
 */
extern void brutus_sbinary_to_entryid(const BRUTUS_SBinary *Bin,
				      BRUTUS_ENTRYID **Eid);

/*
 * Will return PropTag property as a string or NULL in error
 */
extern char *brutus_get_property_as_string(BRUTUS_IMAPIProp Object,
					   const BRUTUS_BDEFINE PropTag);

/*
 * Will return a GUID string or NULL in error
 */
extern char *brutus_get_guid_str(BRUTUS_IMAPIProp Object,
				 const BRUTUS_BDEFINE PropTag);

/*
 * Will return a BRUTUS_ENTRYID for the given proptag or NULL in error
 */
extern BRUTUS_ENTRYID *brutus_get_entryid(BRUTUS_IMAPIProp Object,
					  const BRUTUS_BDEFINE PropTag);

/*
 * Will return a stringified version of the value. NULL is 
 * returned for values with length zero or if the property 
 * type is unknown. Caller must free the returned string.
 */
extern char *brutus_print_spropvalue(const BRUTUS_SPropValue *PropValue);

/* 
 * Will return the index of the queried property tag in 
 * the given row or -1 if not found.
 */
extern CORBA_long brutus_get_srow_index(const BRUTUS_SRow *row,
					const BRUTUS_BDEFINE PropTag);

/* 
 * Will return the index of the queried property tag in 
 * the given SPropValue array or -1 if not found.
 */
extern CORBA_long brutus_get_spropvalue_index(const BRUTUS_seq_SPropValue *props,
					      const BRUTUS_BDEFINE PropTag);

/*
 * This is the Brutus equivalent of HrQueryAllRows()
 */
extern BRUTUS_BRESULT brutus_QueryAllRows(BRUTUS_IMAPITable ptable,
					  BRUTUS_SPropTagArray *ptags,
					  BRUTUS_SRestrictionContainer pres,
					  BRUTUS_SSortOrderSet *psort,
					  CORBA_long crowsMax,
					  BRUTUS_SRowSet **pprows,
					  CORBA_Environment *ev);

/*
 * This is the Brutus equivalent of HrGetOneProp()
 */
extern BRUTUS_BRESULT brutus_GetOneProp(BRUTUS_IMAPIProp MapiProp,
					const BRUTUS_BDEFINE PropTag,
					BRUTUS_SPropValue **PropValue,
					CORBA_Environment *ev);

/*
 * MsgStore is a reference to the default message store
 * upon return or CORBA_OBJECT_NIL
 */
extern BRUTUS_BRESULT brutus_get_default_message_store(BRUTUS_IMAPISession Session,
						       PortableServer_POA ParentPoa,
						       BRUTUS_IMsgStore *MsgStore,
						       CORBA_Environment *ev);

/*
 * RootFolder is a reference to the root folder in MsgStore
 * upon return or NIL
 */
extern BRUTUS_BRESULT brutus_get_root_folder(BRUTUS_IMsgStore MsgStore,
					     BRUTUS_IMAPIFolder *RootFolder,
					     CORBA_Environment *ev);

/*
 * MsgStore is normally the default message store for the profile.
 * Folder is a reference to the requested Outlook specific folder
 * upon return or NIL. Currently predefined tags are:
 *
 *   BRUTUS_BRUTUS_PR_OUTLOOK_CALENDAR
 *   BRUTUS_BRUTUS_PR_OUTLOOK_CONTACTS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_JOURNAL
 *   BRUTUS_BRUTUS_PR_OUTLOOK_NOTES
 *   BRUTUS_BRUTUS_PR_OUTLOOK_TASKS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_REMINDERS
 *   BRUTUS_BRUTUS_PR_OUTLOOK_DRAFTS
 *
 * These tags are at the least valid for the default message store.
 */
extern BRUTUS_BRESULT brutus_get_outlook_folder(BRUTUS_IMsgStore MsgStore,
						const BRUTUS_BDEFINE PropTag,
						BRUTUS_IMAPIFolder *Folder,
						CORBA_Environment *ev);

/*
 * Will return a string listing the values of the ListBy property tag 
 * for all objects that are returned when GetContentsTable() are 
 * invoked on Folder.
 */
extern char *brutus_list_folder(BRUTUS_IMAPIFolder Folder,
				const BRUTUS_BDEFINE ListBy,
				CORBA_Environment *ev);

/*
 * Will return a string listing the values of all properties 
 * for the object.
 */
extern char *brutus_print_properties(BRUTUS_IMAPIProp PropObject,
				     CORBA_Environment *ev);

G_END_DECLS

#endif // _BRUTUS_MAPI_H_


