/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *    Misc proxy macros definitions. 
 *
 *    Authors:
 *        Jules Colding <colding@omesc.com>
 *
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *    MA 02111-1307 USA
 */

#ifndef PROXY_LOG_MACROS_H
#define PROXY_LOG_MACROS_H 1

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib.h>
#include <glib/gmacros.h>
#include <glib/gstdio.h>
#include <glib/gprintf.h>

G_BEGIN_DECLS

extern FILE *log_file;

/* define BRUTUSD_LOG=yes in the environment if you want debugging output */
#define ENV_DEBUG "BRUTUSD_LOG"

/* Prepend before all output */
#define DEBUG_PREFIX "\n* "

/****************************
 *       Logging macros     *
 ****************************/

#undef ORBIT2_EX
#define ORBIT2_EX(_ev_) ((_ev_) && ((_ev_)->_major != CORBA_NO_EXCEPTION))

#undef PRINT_ORBIT2_EX
#ifdef BRUTUS_DEBUG
#define PRINT_ORBIT2_EX(_ev_)                                                                     \
        G_STMT_START {                                                                            \
                if (getenv(ENV_DEBUG)) {                                                          \
		        switch ((_ev_)->_major) {                                                 \
		        case CORBA_NO_EXCEPTION :                                                 \
			        g_fprintf(log_file, DEBUG_PREFIX"No exception.\n");               \
			        break;                                                            \
		        case CORBA_USER_EXCEPTION :                                               \
			        g_fprintf(log_file, DEBUG_PREFIX"%s@%d: User Exception : %s\n",   \
                                        __FILE__, __LINE__, CORBA_exception_id ((_ev_)));         \
			        break;                                                            \
		        case CORBA_SYSTEM_EXCEPTION :                                             \
			        g_fprintf(log_file, DEBUG_PREFIX"%s@%d: System Exception : %s\n", \
                                        __FILE__, __LINE__, CORBA_exception_id ((_ev_)));         \
			        break;                                                            \
		        default :                                                                 \
			        break;                                                            \
		        }                                                                         \
                        fflush(log_file);                                                         \
		}                                                                                 \
	} G_STMT_END
#else
#define PRINT_ORBIT2_EX(_ev_) G_STMT_START { } G_STMT_END
#endif

#undef PROXY_LOG
#ifdef BRUTUS_DEBUG
#define PROXY_LOG(_str_) G_STMT_START { if (getenv(ENV_DEBUG)) g_fprintf(log_file, DEBUG_PREFIX"%s@%d: %s\n", __FILE__, __LINE__, _str_); fflush(log_file); } G_STMT_END
#else
#define PROXY_LOG(_str_) G_STMT_START { } G_STMT_END
#endif


G_END_DECLS

#endif /* PROXY_LOG_MACROS_H */

