/*
 *    Brutus header file defining a ServerRequestInterceptor.
 *    Copyright (C) 2005 OMC Denmark ApS.
 *
 *    This program is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 2 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef _SERVER_REQUEST_INTERCEPTOR_H_
#define _SERVER_REQUEST_INTERCEPTOR_H_

#include <vector>
#include <ace/config-all.h>

#if !defined (ACE_LACKS_PRAGMA_ONCE)
# pragma once
#endif /* ACE_LACKS_PRAGMA_ONCE */

#include <TAO/PI_Server/PI_Server.h>
#include <TAO/tao/PortableInterceptorC.h>
#include <TAO/tao/LocalObject.h>

#include <idl/BrutusSessionC.h>
#include <idl/BrutusLogonC.h>

#if defined(_MSC_VER)
#if (_MSC_VER >= 1200)
#pragma warning(push)
#endif /* _MSC_VER >= 1200 */
#pragma warning(disable:4250)
#endif /* _MSC_VER */

class ServerRequestInterceptor : public virtual PortableInterceptor::ServerRequestInterceptor,
			         public virtual TAO_Local_RefCounted_Object
{
public:
	ServerRequestInterceptor(bool forward,
                                 CORBA::ORB_ptr ORB);

	virtual char *name(ACE_ENV_SINGLE_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException));

	virtual void destroy(ACE_ENV_SINGLE_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException));

	virtual void receive_request_service_contexts(PortableInterceptor::ServerRequestInfo_ptr ri
						      ACE_ENV_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException,
				PortableInterceptor::ForwardRequest));

	virtual void receive_request(PortableInterceptor::ServerRequestInfo_ptr ri
				     ACE_ENV_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException,
				PortableInterceptor::ForwardRequest));

	virtual void send_reply(PortableInterceptor::ServerRequestInfo_ptr ri
				ACE_ENV_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException));

	virtual void send_exception(PortableInterceptor::ServerRequestInfo_ptr ri
				    ACE_ENV_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException,
				PortableInterceptor::ForwardRequest));

	virtual void send_other(PortableInterceptor::ServerRequestInfo_ptr ri
				ACE_ENV_ARG_DECL_WITH_DEFAULTS)
		ACE_THROW_SPEC((CORBA::SystemException,
				PortableInterceptor::ForwardRequest));

private:
	ServerRequestInterceptor(void)
		{
		};

        // private due to reference counting
	~ServerRequestInterceptor(void)
		{
			try {
				for (unsigned long n=0; n<session_objs_.size(); n++) {
					free(session_objs_[n].location);
					free(logon_objs_[n].location);
				}
				session_objs_.resize(0);
				logon_objs_.resize(0);
			}
			catch (...) {
			}
		};

	struct session_loc {
		char *location;
		BRUTUS::BrutusSession_var obj;
	};
	struct logon_loc {
		char *location;
		BRUTUS::BrutusLogOn_var obj;
	};

	bool forward_;
	CORBA::ORB_var orb_;

        std::vector<struct session_loc> session_objs_;
        std::vector<struct logon_loc> logon_objs_;
        ACE_RW_Mutex mutex_;
};

#if defined(_MSC_VER) && (_MSC_VER >= 1200)
#pragma warning(pop)
#endif /* _MSC_VER */


#endif // _SERVER_REQUEST_INTERCEPTOR_H_

/*
 * Local variables:
 * mode: C++
 * c-basic-offset: 8
 * tab-width: 8
 * indent-tabs-mode: nil
 * End:
 */
